/**
 * 
 */
package com.up.sell.controller;

import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import com.up.sell.mapper.MemberMapper;
import com.up.sell.util.Constant;
import com.up.sell.util.JWTUtil;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.Member;


/**
 * @author mingguangz
 * 所有的controller继承这个类
 */
public abstract class AbstractController {
	
	@Autowired
	private MemberMapper memberMapper;
	
	protected String getCurrentURL(HttpServletRequest request) {
		return ServletUtils.getCurrentURL(request);
	}
	
	protected String getBaseURL(HttpServletRequest request) {
		return ServletUtils.getBaseURL(request);
	}
	
	/**
	 * 获得访问者的IP
	 * @param request
	 * @return
	 * @throws Exception
	 */
	protected String getIpAddress(HttpServletRequest request) {
		return ServletUtils.getIpAddress(request);
	}
		
	protected void addCookie(String name,String value,HttpServletResponse response){
		this.addCookie(name, value, ServletUtils.DEFAULT_COOKIE_PATH, ServletUtils.DEFAULT_COOKIE_MAXAGE,response);
	}
	protected void addCookie(String name,String value,Integer maxAge,HttpServletResponse response){
		this.addCookie(name, value, ServletUtils.DEFAULT_COOKIE_PATH, maxAge,response);
	}
	protected void addCookie(String name,String value,String path,Integer maxAge,HttpServletResponse response){
		ServletUtils.addCookie(name, value, maxAge, response);
	}
	protected void removeCookie(String name,HttpServletResponse response){
		ServletUtils.removeCookie(name, response);
	}
	protected void removeCookie(String name,String path,HttpServletResponse response){
		ServletUtils.removeCookie(name, path, response);
	}
	protected Cookie getCookieByName(String name, HttpServletRequest request){
	    return getCookies(request).get(name);  
	}
	protected Map<String,Cookie> getCookies(HttpServletRequest request){  
	    return ServletUtils.getCookies(request);
	}
	protected String getCookieValueByName(String name,HttpServletRequest request){
	    return getCookieValues(request).get(name);  
	}
	protected Map<String,String> getCookieValues(HttpServletRequest request){  
	    return ServletUtils.getCookieValues(request);
	}
	
	protected void responseOutWithJson (HttpServletResponse response,  String result) {
		ServletUtils.printWriter(response, result);
	}
	
	protected Member getLoginMember(HttpServletRequest request) {
		String token = request.getHeader(Constant.APP_HEADER_ACCESS_TOKEN);
		if(StringUtils.isBlank(token)) {
			token = request.getParameter(Constant.APP_HEADER_ACCESS_TOKEN);
		}
		if("-1".equals(token)){
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			Member mem=this.memberMapper.selectByPrimaryKey(Integer.valueOf(parameters.get("loginUserId")+""));
			return mem;
		}
		return JWTUtil.unsign(token, Member.class);
	}
}
