package com.up.sell.controller.api;

import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.codec.binary.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alipay.api.AlipayApiException;
import com.alipay.api.internal.util.AlipaySignature;
import com.google.common.collect.Maps;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.mapper.MemberOrderMapper;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.DateUtils;
import com.up.sell.vo.Member;
import com.up.sell.vo.MemberOrder;


@CrossOrigin
@RestController("appAliPayCallBackApi")
@RequestMapping("aliPayCallBack")
public class AlipayCallBackApi {
	private static final Logger log = LoggerFactory.getLogger(AlipayCallBackApi.class);
	
	public final static String CHARSET = ConfigurationUtils.getStringValue("ALI_CHARSET");
	public final static String publicKey = ConfigurationUtils.getStringValue("ALI_PAY_PUBLIC_KEY");
	
	@Autowired
	private MemberOrderMapper memberOrderMapper;
	
	@Autowired
	private MemberMapper memberMapper;
	
	@RequestMapping("callBack")
	public void callback(HttpServletRequest request,HttpServletResponse response) throws IOException {
		log.info("alipay callBack run222.....");
		//获取支付宝POST过来反馈信息
		Map<String,String[]> requestParams = request.getParameterMap();
		String orderNo = new String(request.getParameter("out_trade_no").getBytes("ISO-8859-1"),"UTF-8");
		String orderFee = new String(request.getParameter("total_amount").getBytes("ISO-8859-1"),"UTF-8");
		String trade_no = new String(request.getParameter("trade_no").getBytes("ISO-8859-1"),"UTF-8");
		String buyer_logon_id = new String(request.getParameter("buyer_logon_id").getBytes("ISO-8859-1"),"UTF-8");
		String gmt_payment = new String(request.getParameter("gmt_payment").getBytes("ISO-8859-1"),"UTF-8");
		log.info("orderNo : "+orderNo+"   "+"orderFee : "+orderFee+"  "+"trade_No : "+trade_no);
		//1、校验签名
		Boolean signVerified = checkSign(requestParams);
		log.info("signVerified");	
		if(!signVerified) {
			response.getWriter().write("failed");
		
		}else {
			
			response.getWriter().write("success");
			MemberOrder order = memberOrderMapper.selectByPrimaryKey(orderNo);
			//2、支付金额校验
			log.info("orderFee:{}",orderFee);
			log.info("payAmount------>"+ order.getPayAmount().toString());
			double fee = Double.parseDouble(orderFee);
			Double payAmount = order.getPayAmount();
			if(fee!=payAmount){
					response.getWriter().write("failed");
			}else {
			log.info("支付成功进入");
				//支付成功 更新对应记录
				Integer memberId = order.getMemberId();
				Map<String, Object> map = Maps.newHashMap();
				map.put("memberId", memberId);
				MemberOrder memberOrder = memberOrderMapper.selectMemberOrder(map);
				if(Objects.nonNull(memberOrder)) {
					memberOrder.setServiceFlag(0);
					memberOrderMapper.updateByPrimaryKeySelective(memberOrder);	
				}
				order.setBeginTime(new Date());//服务开始日期
				if(StringUtils.equals("普通会员", order.getGoodsName())) {
					order.setEndTime(getEndDate(new Date(),"1"));//服务结束日期
				}else {
					order.setEndTime(getEndDate(new Date(),"2"));//服务结束日期
				}
				    order.setOrderStatus(2);
					order.setBuyerEmail(buyer_logon_id);
					order.setServiceFlag(1);//服务是否可用：1可用，0不可用，支付成功时设为可用
					order.setPayTime(DateUtils.format(gmt_payment, "yyyy-MM-dd hh:mm:ss"));//支付时间
					order.setTradeNo(trade_no);//支付流水号
					memberOrderMapper.updateByPrimaryKey(order);
					//更新用户表查看次数
					Member m = memberMapper.selectByPrimaryKey(order.getMemberId());
					log.info("memberId---->" + order.getMemberId());
					Member me = new Member();
					me.setId(memberId);
					me.setViewCount(m.getViewCount()+order.getViewCount());
					memberMapper.updateByPrimaryKeySelective(me);
			 log.info("支付成功结束");
			}
		}
	}
	
	public Boolean checkSign(Map<String,String[]> requestParams){
		//获取支付宝POST过来反馈信息
		Map<String,String> params = new HashMap<String,String>();
		for (Iterator<String> iter = requestParams.keySet().iterator(); iter.hasNext();) {
		    String name = (String) iter.next();
		    String[] values = (String[]) requestParams.get(name);
		    String valueStr = "";
		    for (int i = 0; i < values.length; i++) {
		        valueStr = (i == values.length - 1) ? valueStr + values[i]
		                    : valueStr + values[i] + ",";
		  	}
		    //乱码解决，这段代码在出现乱码时使用。
			params.put(name, valueStr);
		}
		//切记alipaypublickey是支付宝的公钥，请去open.alipay.com对应应用下查看。
		boolean flag = false;
		try {
			flag = AlipaySignature.rsaCheckV1(params, publicKey, CHARSET,"RSA2");
		} catch (AlipayApiException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return flag;
	}

	/**
	 * 获取服务结束日期
	 * @param date
	 * @param type
	 * @return
	 */
	public static Date getEndDate(Date date,String type) {
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(new Date());
		if(StringUtils.equals("1", type)) {
			calendar.add(Calendar.MONTH, 1);
			calendar.add(Calendar.DAY_OF_MONTH, -1);
			return calendar.getTime();
		}else {
			calendar.add(Calendar.YEAR, 1);
			calendar.add(Calendar.DAY_OF_YEAR, -1);
			return calendar.getTime();
		}
	}
	
	public static void main(String[] args) {
		Double a =10.00;
		String s ="10.00";
		double d = Double.parseDouble(s);
		if(d==a) {
			System.out.println(11);
		}else {
			System.out.println(22);
		}
	}

}
