/**
 * 
 */
package com.up.sell.controller.api;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.google.common.collect.Maps;
import com.up.sell.controller.AbstractController;
import com.up.sell.service.AreasService;
import com.up.sell.service.BrandService;
import com.up.sell.service.DictionaryService;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.Constant;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.BrandContacts;
import com.up.sell.vo.BrandContactsArea;
import com.up.sell.vo.BrandContactsError;
import com.up.sell.vo.BrandContactsErrorArea;
import com.up.sell.vo.BrandDatabase;
import com.up.sell.vo.BrandExpandArea;
import com.up.sell.vo.ResultObject;

/**
 * @author zmg
 *
 */
@RestController
@RequestMapping(value = "/brand")
public class BrandController extends AbstractController {
	private static final Logger logger = LoggerFactory.getLogger(BrandController.class);

	@Autowired
	private DictionaryService dictionaryService;
	@Autowired
	private AreasService areasService;
	@Autowired
	private BrandService brandService;
	
	@PostMapping("/filter")
	public ResultObject filter(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			//面积区间
			Map all = new HashMap();
			all.put("d_name", "全部");
			all.put("d_value", "");
			List<Map<String, Object>> acreageList = this.dictionaryService.selectMapByParentKey("126");
			acreageList.remove(0);
			acreageList.add(0, all);
			resultObject.put("acreageList", acreageList);
			//拓展区域
			Map areas2 = new HashMap<>();
			areas2.put("area_name", "全部");
			areas2.put("id", "");
			areas2.put("regions_id", "");
			List<Map> all2Lev = new ArrayList<Map>();
			all2Lev.add(areas2);
			all = new HashMap();
			all.put("regions_name", "全部");
			all.put("id", "");
			all.put("areasList", all2Lev);
			List<Map> expandAreaList = this.areasService.selectBigAreas();
			expandAreaList.add(0, all);
			resultObject.put("expandAreaList", expandAreaList);
			//业态
			Map brandBusiness2 = new HashMap<>();
			brandBusiness2.put("businessName", "全部");
			brandBusiness2.put("id", "");
			brandBusiness2.put("p_id", "");
			all2Lev = new ArrayList<Map>();
			all2Lev.add(brandBusiness2);
			all = new HashMap();
			all.put("businessName", "全部");
			all.put("id", "");
			all.put("levelTwoList", all2Lev);
			List<Map> brandBusinessList = brandService.getBrandBusinessList(true);
			brandBusinessList.add(0, all);
			resultObject.put("brandBusinessList", brandBusinessList);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");

			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 查询品牌列表
	 * @param request
	 * cityId、brandName、areasId、regionsId、provinceId、brandBusiness、pageSize、pageNum、topNum
	 *
	 * brand_nature 1-直营 2-加盟
	 *
	 * @return
	 */
	@PostMapping("/list")
	public ResultObject list(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			parameters.put("picDomain", ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN));
			Integer pageSize = Integer.valueOf(parameters.get("pageSize").toString());
			Integer pageNum = Integer.valueOf(parameters.get("pageNum").toString());
			List<Map> brandList = new ArrayList<Map>();
//			if(pageNum == 1) {
//				Integer topNum = parameters.get("topNum")==null?10:Integer.valueOf(parameters.get("topNum").toString());
//				PageHelper.startPage(1, topNum, false);
//				brandList.addAll(this.brandService.selectBrandTopList(parameters));
//			}
			Page<Object> startPage = PageHelper.startPage(pageNum, pageSize, true);
			brandList.addAll(this.brandService.selectBrandList(parameters));
			resultObject.put("brandList", brandList);
			resultObject.put("totalCount", startPage.getTotal());
			resultObject.put("hotBrandList", brandService.getHotBrandList(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 品牌详情页
	 * @param request
	 * id
	 * @return
	 */
	@PostMapping("/detail")
	public ResultObject detail(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			logger.info("detail id ==> "+ parameters.get("id").toString());
			Map detail = this.brandService.selectBrandDetail(Integer.valueOf(parameters.get("id").toString()));
			String brandPic = (String) detail.get("brand_pic");
			if(StringUtils.isNotBlank(brandPic)) {
				String[] brandPics = StringUtils.split(brandPic, ",");
				for(int i=0;i<brandPics.length;i++) {
					brandPics[i]=ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN)+brandPics[i];
				}
				resultObject.put("brandPic", brandPics);
			} else {
				resultObject.put("brandPic", new String[] {"https://up-sell.oss-cn-beijing.aliyuncs.com/upload/2018/10/30/1540885144956.png"});
			}
			detail.remove("brand_pic");
			detail.put("logo", ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN)+detail.get("logo").toString());
			resultObject.put("detail", detail);
			parameters.put("picDomain", ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN));
			resultObject.put("brandRecommended", brandService.getBrandRecommended(parameters));
			resultObject.put("brandContacts", this.brandService.selectBrandContacts(Integer.valueOf(parameters.get("id").toString())));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 品牌发布页面、联系人纠错、我要成为联系人、初始化数据
	 * @param request
	 * @return
	 */
	@PostMapping("/brandReleasedPage")
	public ResultObject brandReleasedPage(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			//拓展区域
			resultObject.put("expandAreaList", this.areasService.selectByParentId("0"));
			//业态
			resultObject.put("brandBusinessList", brandService.getBrandBusinessList(false));
			//品牌定位
			resultObject.put("brandPositioningList", this.dictionaryService.selectMapByParentKey("123"));
			//岗位
			resultObject.put("brandPostList", this.dictionaryService.selectMapByParentKey("127"));
			
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 我要成为品牌联系人接口
	 * @param request
	 * {
			"provinceId": "110000",
			"cityId": "110100",
			"brandDatabaseId": 1,
			"mobile": "13241981234",
			"userName": "李四",
			"post": 1,
			"brandContactsAreaList": [{
				"provinceId": "120000",
				"regionsId": 1
			}, {
				"provinceId": "130000",
				"regionsId": 1
			}]
		}
	 * @return
	 */
	@PostMapping("/auth/createContact")
	public ResultObject createContact(HttpServletRequest request,BrandContacts brandContacts) {
		ResultObject resultObject = new ResultObject();
		try {
			brandContacts.setMemberId(this.getLoginMember(request).getId());
			String brandContactsArea = brandContacts.getBrandContactsArea();
			if(StringUtils.isNotBlank(brandContactsArea)) {
				String[] beas = StringUtils.split(brandContactsArea, ";");
				for(String bea : beas) {
					String[] rp = StringUtils.split(bea, "-");
					BrandContactsArea bca1 = new BrandContactsArea();
					bca1.setRegionsId(Integer.valueOf(rp[0]));
					bca1.setProvinceId(rp[1]);
					brandContacts.getBrandContactsAreaList().add(bca1);
				}
			} else {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("请传联系人的负责区域");
			}
			
			this.brandService.createContact(brandContacts);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 品牌联系人纠错
	 * @param request
	 * @param brandContactsError
	 * 
	 * @return
	 */
	@PostMapping("/auth/createContactError")
	public ResultObject createContactError(HttpServletRequest request,BrandContactsError brandContactsError) {
		ResultObject resultObject = new ResultObject();
		try {
			brandContactsError.setMemberId(this.getLoginMember(request).getId());
			String brandContactsArea = brandContactsError.getBrandContactsArea();
			if(StringUtils.isNotBlank(brandContactsArea)) {
				String[] beas = StringUtils.split(brandContactsArea, ";");
				for(String bea : beas) {
					String[] rp = StringUtils.split(bea, "-");
					BrandContactsErrorArea bca1 = new BrandContactsErrorArea();
					bca1.setRegionsId(Integer.valueOf(rp[0]));
					bca1.setProvinceId(rp[1]);
					brandContactsError.getBrandContactsErrorAreaList().add(bca1);
				}
			} else {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("请传联系人的负责区域");
			}
			this.brandService.createContactError(brandContactsError);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 品牌发布页面、初始化数据
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/brandReleased")
	public ResultObject brandReleased(HttpServletRequest request,BrandDatabase brandDatabase) {
		ResultObject resultObject = new ResultObject();
		try {
			brandDatabase.setMemberId(this.getLoginMember(request).getId());
			Map<String, Object> parameters = Maps.newHashMap();
			parameters.put("brandName", brandDatabase.getBrandName());
			resultObject = brandService.valideBrandName(parameters);
			String status = resultObject.get("status").toString();
			if(StringUtils.equals("false", status)) {
				return resultObject;
			}
			try {
				String brandBusiness = brandDatabase.getBrandBusiness();
				if(StringUtils.isNotBlank(brandBusiness)) {
					String[] bbs = StringUtils.split(brandBusiness, "-");
					if(StringUtils.isNotBlank(bbs[0])) {
						brandDatabase.setBrandBusinessOne(Integer.valueOf(bbs[0]));
					}
					if(bbs.length>1 && StringUtils.isNotBlank(bbs[1])) {
						brandDatabase.setBrandBusinessTwo(Integer.valueOf(bbs[1]));
					}
				} /*else {
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("请传业态");
				}*/
			} catch (Exception e1) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("业态参数格式异常");
			}
			
			try {
				String brandExpandArea = brandDatabase.getBrandExpandArea();
				if(StringUtils.isNotBlank(brandExpandArea)) {
					String[] beas = StringUtils.split(brandExpandArea, ";");
					for(String bea : beas) {
						String[] rp = StringUtils.split(bea, "-");
						BrandExpandArea brandExpandAreaBean = new BrandExpandArea();
						brandExpandAreaBean.setRegionsId(Integer.valueOf(rp[0]));
						brandExpandAreaBean.setProvinceId(rp[1]);
						brandDatabase.getBrandExpandAreaList().add(brandExpandAreaBean);
					}
				} /*else {
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("请传拓展区域");
				}*/
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("拓展区域参数格式异常");
			}
			
			try {
				if(StringUtils.isNotBlank(brandDatabase.getMobile1())) {
					BrandContacts brandContacts = new BrandContacts();
					brandContacts.setProvinceId(brandDatabase.getProvinceId());
					brandContacts.setCityId(brandDatabase.getCityId());
					brandContacts.setMobile(brandDatabase.getMobile1());
					brandContacts.setUserName(brandDatabase.getUserName1());
					brandContacts.setPost(brandDatabase.getPost1());
					String brandContactsArea = brandDatabase.getBrandContactsArea1();
					if (StringUtils.isBlank(brandDatabase.getBrandNature())){
						resultObject.setCode(Constant.CODE_EXCEPTION);
						resultObject.setMessage("请选择品牌经营性质");
					}

					if(StringUtils.isNotBlank(brandContactsArea)) {
						String[] beas = StringUtils.split(brandContactsArea, ";");
						for(String bea : beas) {
							String[] rp = StringUtils.split(bea, "-");
							BrandContactsArea bca1 = new BrandContactsArea();
							bca1.setRegionsId(Integer.valueOf(rp[0]));
							bca1.setProvinceId(rp[1]);
							brandContacts.getBrandContactsAreaList().add(bca1);
						}
					} else {
						resultObject.setCode(Constant.CODE_EXCEPTION);
						resultObject.setMessage("请传联系人1的负责区域");
					}
					brandDatabase.getBrandContactsList().add(brandContacts);
				}
				if(StringUtils.isNotBlank(brandDatabase.getMobile2())) {
					BrandContacts brandContacts = new BrandContacts();
					brandContacts.setProvinceId(brandDatabase.getProvinceId());
					brandContacts.setCityId(brandDatabase.getCityId());
					brandContacts.setMobile(brandDatabase.getMobile2());
					brandContacts.setUserName(brandDatabase.getUserName2());
					brandContacts.setPost(brandDatabase.getPost2());
					String brandContactsArea = brandDatabase.getBrandContactsArea2();
					if(StringUtils.isNotBlank(brandContactsArea)) {
						String[] beas = StringUtils.split(brandContactsArea, ";");
						for(String bea : beas) {
							String[] rp = StringUtils.split(bea, "-");
							BrandContactsArea bca2 = new BrandContactsArea();
							bca2.setRegionsId(Integer.valueOf(rp[0]));
							bca2.setProvinceId(rp[1]);
							brandContacts.getBrandContactsAreaList().add(bca2);
						}
					} else {
						resultObject.setCode(Constant.CODE_EXCEPTION);
						resultObject.setMessage("请传联系人2的负责区域");
					}
					brandDatabase.getBrandContactsList().add(brandContacts);
				}
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("联系人参数格式异常");
			}
			
			this.brandService.saveBrandDatabase(brandDatabase);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 查看品牌联系人的联系电话
	 * @param request
	 * @param brandContactId
	 * @return
	 */
	@PostMapping("/auth/getBrandContact")
	public ResultObject getBrandContact(HttpServletRequest request,Integer brandContactId) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject = this.brandService.getBrandContact(brandContactId, this.getLoginMember(request).getId());
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	/**
	 * 验证重复品牌
	 * @param request
	 * @param brandContactId
	 * @return
	 */
	@PostMapping("/auth/valideBrandName")
	public ResultObject valideBrandName(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = this.brandService.valideBrandName(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
   }
	
	
	/**
	 * 回显纠错联系人信息
	 * @param request
	 * @param brandContactId
	 * @return
	 */
	@PostMapping("/auth/showContactError")
	public ResultObject showContactError(HttpServletRequest request,Integer brandContactId) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject = this.brandService.getContactError(brandContactId, this.getLoginMember(request).getId());
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
}
