package com.up.sell.controller.api;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.up.sell.controller.AbstractController;
import com.up.sell.service.AgreementService;
import com.up.sell.service.BrowseLogService;
import com.up.sell.service.DictionaryService;
import com.up.sell.service.FourHundredService;
import com.up.sell.service.MemberService;
import com.up.sell.util.CheckParametersUtil;
import com.up.sell.util.Constant;
import com.up.sell.util.JWTUtil;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.Member;
import com.up.sell.vo.ResultObject;

/**
 * 浏览痕迹
 * @author gust
 *
 */
@RestController
@RequestMapping("/browseLog")
public class BrowseLogController extends AbstractController{

	private static final Logger logger = LoggerFactory.getLogger(BrowseLogController.class);
	
	@Autowired
	private	BrowseLogService browseLogService;
	@Autowired
	private FourHundredService fourHundredService;
	
	/**
	 * 浏览痕迹列表
	 * @param request
	 * @return
	 */
	@PostMapping("/findPage")
	public ResultObject list(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("pageSize"), "pageSize")
				.put(parameters.get("pageNum"), "pageNum")
				.put(parameters.get("projectId"), "projectId")
				.checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage("缺少必要参数，请检查入参！");
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			return browseLogService.selectPage(parameters);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	/**
	 * 更新浏览次数
	 * @param request
	 * @return
	 */
	@PostMapping("/updateBrowseLog")
	public ResultObject updateBrowseLog(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("userId"), "userId")
				.put(parameters.get("projectId"), "projectId")
				.checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage("缺少必要参数，请检查入参！");
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			this.browseLogService.updateEntity(parameters);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	@PostMapping("/bangDingOne")
	public ResultObject bangDingOne(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("linkPhone"), "linkPhone")
				.checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage("缺少必要参数，请检查入参！");
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			this.fourHundredService.bangDingOne(parameters.get("linkPhone")+"");
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
}
