/**
 * 
 */
package com.up.sell.controller.api;

import java.util.Date;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.up.sell.controller.AbstractController;
import com.up.sell.service.MemberCustomerService;
import com.up.sell.service.ProjectService;
import com.up.sell.util.Constant;
import com.up.sell.util.ResultException;
import com.up.sell.vo.MemberCustomer;
import com.up.sell.vo.ResultObject;

/**
 * @author zmg
 *
 */
@RestController
@RequestMapping(value = "/customer")
public class CustomerController extends AbstractController {
	private static final Logger logger = LoggerFactory.getLogger(CustomerController.class);
	@Autowired
	private MemberCustomerService memberCustomerService;
	@Autowired
	private ProjectService projectService;
	/**
	 * 客源
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/list")
	public ResultObject list(HttpServletRequest request,String customerName) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject = this.memberCustomerService.getMyCustomers(this.getLoginMember(request).getId(),customerName);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 添加客户
	 * @param request
	 * @param memberCustomer
	 * @return
	 */
	@PostMapping("/auth/add")
	public ResultObject add(HttpServletRequest request,MemberCustomer memberCustomer) {
		ResultObject resultObject = new ResultObject();
		try {
			memberCustomer.setMemberId(this.getLoginMember(request).getId());
			memberCustomer.setCreateTime(new Date());
			this.memberCustomerService.saveMemberCustomer(memberCustomer);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 客户详情页
	 * @param request
	 * @param customerId
	 * @return
	 */
	@PostMapping("/auth/detail")
	public ResultObject detail(HttpServletRequest request,Integer customerId) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject = this.memberCustomerService.getCustomerDetail(customerId);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 客户交易详情
	 * @param request
	 * @param projectType
	 * @param orderNo
	 * @return
	 */
	@PostMapping("/auth/projectOrderTrackList")
	public ResultObject projectOrderTrackList(HttpServletRequest request,Integer projectType,String orderNo) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject.putAll(this.projectService.getOrderStatuse(projectType, orderNo));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
}
