/**
 * 
 */
package com.up.sell.controller.api;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.up.sell.controller.AbstractController;
import com.up.sell.service.InformationChannelService;
import com.up.sell.util.Constant;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.ResultObject;

/**
 * @author zmg
 *
 */
@RestController
@RequestMapping(value = "/information")
public class InformationChannelController extends AbstractController {
	private static final Logger logger = LoggerFactory.getLogger(InformationChannelController.class);
	@Autowired
	private InformationChannelService informationChannelService;
	
	/**
	 * 资讯列表接口
	 * @param request
	 *  pageSize
	 *  pageNum
	 * 	cityId：APP所在城市
	 * @return
	 */
	@PostMapping("/list")
	public ResultObject filter(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			Integer pageSize = Integer.valueOf(parameters.get("pageSize").toString());
			Integer pageNum = Integer.valueOf(parameters.get("pageNum").toString());
			Page<Object> startPage = PageHelper.startPage(pageNum, pageSize, true);
			resultObject.put("informationList", this.informationChannelService.selectInformationList(parameters));
			resultObject.put("totalCount", startPage.getTotal());
			resultObject.put("hotProjectList", informationChannelService.getHotProjectList(parameters));
			resultObject.put("hotBrandList", informationChannelService.getHotBrandList(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	/**
	 * 资讯详情接口
	 * @param request
	 * 	id
	 * @return
	 */
	@PostMapping("/detail")
	public ResultObject detail(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("informationDetail", this.informationChannelService.selectInformationDetail(Integer.valueOf(parameters.get("id").toString())));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	
	/**
	 * 
	 * @param request
	 * @return
	 */
	@PostMapping("/getInfoList")
	public ResultObject getInfoList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("informationList", this.informationChannelService.selectInforList(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
}
