/**
 * 
 */
package com.up.sell.controller.api;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;

import com.up.sell.controller.AbstractController;
import com.up.sell.redis.RedisBaseDao;
import com.up.sell.service.AsyncLoginLogService;
import com.up.sell.service.DictionaryService;
import com.up.sell.service.MemberService;
import com.up.sell.service.WxMiniService;
import com.up.sell.service.WxPublicAccountService;
import com.up.sell.util.CheckParametersUtil;
import com.up.sell.util.Constant;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.Member;
import com.up.sell.vo.MemberLoginLog;
import com.up.sell.vo.MemberRegisterLog;
import com.up.sell.vo.ResultObject;
import com.up.sell.vo.VisitorsLog;

/**
 * @author zmg
 *
 */
@RestController
public class LoginController extends AbstractController {
	
	private static final Logger logger = LoggerFactory.getLogger(LoginController.class);
	
	@Autowired
	DictionaryService dictionaryService;
	
	@Autowired
	MemberService memberService;
	
	@Autowired
	RedisBaseDao redisBaseDao;
	
	@Autowired
	AsyncLoginLogService asyncLoginLogService;
	
	@Autowired
	WxMiniService wxMiniService;
	
	@Autowired
	WxPublicAccountService wxPublicAccountService;
	
	
	@PostMapping("/login")
	public ResultObject login(HttpServletRequest req, String mobile,String validCode,String provinceId,String cityId,Integer type,Integer referrerId,Integer teamId,String source) {
		ResultObject resultObject = new ResultObject();
		try {
			logger.info("login mobile:{},type:{},source:{},provinceId:{},cityId:{}",mobile,type,source,provinceId,cityId);
			resultObject = memberService.login(mobile, validCode,provinceId,cityId,type,referrerId,teamId);
			if(StringUtils.equals(resultObject.get("code").toString(),"101")) {
				if(resultObject.get("loginFlag").equals(true)) {
					MemberLoginLog memberLoginLog = new MemberLoginLog();
					memberLoginLog.setToken(resultObject.get(Constant.APP_HEADER_ACCESS_TOKEN).toString());
					memberLoginLog.setIpAddress(getIpAddress(req));
					memberLoginLog.setMemberId(((Member) resultObject.get("memberInfo")).getId());
					memberLoginLog.setAppVersion(req.getHeader(Constant.APP_HEADER_APP_VERSION));
					memberLoginLog.setDeviceId(req.getHeader(Constant.APP_HEADER_DEVICE_ID));
					memberLoginLog.setDeviceName(req.getHeader(Constant.APP_HEADER_DEVICE_NAME));
					memberLoginLog.setSource(Integer.valueOf(source));
					this.asyncLoginLogService.saveMemberLog(memberLoginLog);
				} else {
					MemberRegisterLog memberRegisterLog = new  MemberRegisterLog();
					memberRegisterLog.setMemberId(((Member) resultObject.get("memberInfo")).getId());
					memberRegisterLog.setIpAddress(getIpAddress(req));
					memberRegisterLog.setAppVersion(req.getHeader(Constant.APP_HEADER_APP_VERSION));
					memberRegisterLog.setDeviceId(req.getHeader(Constant.APP_HEADER_DEVICE_ID));
					memberRegisterLog.setDeviceName(req.getHeader(Constant.APP_HEADER_DEVICE_NAME));
					memberRegisterLog.setSource(Integer.valueOf(source));
					this.asyncLoginLogService.saveMemberRegisterLog(memberRegisterLog);
				}
			}
			//判断用户是被冻结
			Member member = memberService.selectStatus(mobile);
			if (null != member && member.getFlag() == 2){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("您的账号已锁定，请拨打400-198-1188帮你解锁");
				return resultObject;
			}
		} catch (ResultException e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
			return resultObject;
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("登录失败");
			logger.error(e.getMessage(), e);
			return resultObject;
		}
		return resultObject;
    }
	
	@PostMapping("/sendsms")
	public ResultObject sendSMS(String mobile,String type) {
		logger.info("sendsms mobile:{},type:{}",mobile,type);
		ResultObject resultObject = new ResultObject();
		try {
			if(StringUtils.isBlank(mobile)) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("请传手机号参数");
				return resultObject;
			}
			if(StringUtils.isBlank(type)) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("请传短信类型");
				return resultObject;
			}
			//判断用户是被冻结
			Member member = memberService.selectStatus(mobile);
			if (null != member){
				if (member.getFlag() == 2){
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("您的账号已锁定，请拨打400-198-1188帮你解锁");
					return resultObject;
				}
			}
			Map<String, Object> sendSms2 = memberService.sendSms(mobile,type);
		//	resultObject.put("validCode", sendSms2.get("code"));
			boolean sendSms = (boolean) sendSms2.get("flag");
			if(!sendSms) {
			  	resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("发送短信失败");
				return resultObject;
			}
		} catch (ResultException e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
			return resultObject;
		}catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("发送短信异常");
			logger.error(e.getMessage(), e);
		}
        logger.info("sendsms end");
		return resultObject;
	}
	
	/**
	 * 弹框输入电话
	 * @param request
	 * @return
	 */
	@PostMapping("/findWxPhoneAlart")
	public ResultObject findWxPhoneAlart(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("phone"), "phone")
				.put(parameters.get("openid"), "openid")
				.put(parameters.get("validCode"), "validCode")
				.checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage(e.getMessage());
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			resultObject=this.wxMiniService.findWxPhoneAlart(parameters);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	/**
	 * 授权获取电话
	 * 获取微信唯一标识
	 */
	@PostMapping("/findWxPhone")
	public ResultObject findWxPhone(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("jsCode"), "jsCode")
				.put(parameters.get("openid"), "openid")
				/*.put(parameters.get("wxImg"), "wxImg")
				.put(parameters.get("wxName"), "wxName")*/
				.put(parameters.get("encrypted"), "encrypted")
				//.put(parameters.get("sessionKey"), "sessionKey")
				.put(parameters.get("iv"), "iv")
				.checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage(e.getMessage());
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			resultObject=this.wxMiniService.findWxPhone(parameters);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	
	
	@PostMapping("/addVisitorsLog")
	public ResultObject addVisitorsLog(HttpServletRequest request,VisitorsLog entity) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("visitorsOpenId"), "visitorsOpenId")
				.put(parameters.get("projectId"), "projectId")
				.put(parameters.get("type"), "type")
				.put(parameters.get("iconPath"), "iconPath")
				.put(parameters.get("createUserId"), "createUserId")
				.put(parameters.get("userName"), "userName").checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage(e.getMessage());
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			resultObject=this.wxMiniService.addVisitorsLog(parameters,entity);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	@PostMapping("/findVisitorsLogPage")
	public ResultObject findVisitorsLogPage(HttpServletRequest request,VisitorsLog entity) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("pageNum"), "pageNum")
				.put(parameters.get("pageSize"), "pageSize")
				.put(parameters.get("projectId"), "projectId")
				.put(parameters.get("createUserId"), "createUserId").checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage(e.getMessage());
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			resultObject=this.wxMiniService.findVisitorsLogPage(parameters);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	@PostMapping("/findUserByOpenid")
	public ResultObject findUserByOpenid(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("openid"), "openid")
				.checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage(e.getMessage());
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			resultObject=this.wxMiniService.findUserFlagByOpenid(parameters.get("openid")+"");
			logger.info("===============findUserByOpenid()返回报文："+resultObject+"");
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	
	/**
	 * 判断用户是否锁定,后台锁定某个手机号的用户前台app同样不能登录
	 * @param request
	 * @param entity
	 * @return
	 */
	@PostMapping("/ifLocked")
	public ResultObject ifLocked(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			String token = request.getHeader("access-token");
			if(StringUtils.isEmpty(token)) {
				resultObject.setCode(Constant.CODE_NOT_LOGIN);
				resultObject.setMessage("入参有误请检查token是否传了");
				return resultObject;
			}
			//判断用户是被冻结
			Member member = memberService.selectStatusById(parameters.get("loginIdentity")+"");
			if (null != member && member.getFlag() == 2){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("您的账号已锁定，请拨打400-198-1188帮你解锁");
				return resultObject;
			}
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 获取openid
	 * @param request
	 * @return
	 */
	@PostMapping("/findOpenidByJsCode")
	public ResultObject findOpenidByJsCode(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("jsCode"), "jsCode")
				.checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage(e.getMessage());
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			String jsCode=parameters.get("jsCode")+"";
			resultObject=this.wxMiniService.findOpenidByJsCode(jsCode);
			logger.info("===============findOpenidByJsCode()返回报文："+resultObject+"");
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	//================================================公众号 start==============================
	/**
	 * 公众号绑定openid与phone
	 * @param request
	 * @return
	 */
	@PostMapping("/bindingPhoneGzh")
	public ResultObject bindingPhoneGzh(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		Map<String,Object> parameters = ServletUtils.getParameters(request);
		try {
			CheckParametersUtil.getInstance()
			.put(parameters.get("phone"), "phone")
			.put(parameters.get("code"), "code")
			.checkParameter();
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_PARAM_NULL);
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
			return resultObject;
		}
		resultObject=this.wxPublicAccountService.bindingPhone(parameters);
		logger.info("===============bindingPhoneGzh()返回报文："+resultObject+"");
		return resultObject;
	}
	
	
	/**
	 * 向用户关注的公众号[优铺网络]推送消息
	 * @param request
	 * @return
	 */
	@PostMapping("/sendMsgToGzh")
	public ResultObject sendMsgToGzh(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			try {
				CheckParametersUtil.getInstance()
				.put(parameters.get("phone"), "phone")
				.put(parameters.get("remark"), "remark")
				.checkParameter();
			} catch (Exception e) {
				resultObject.setCode(Constant.CODE_PARAM_NULL);
				resultObject.setMessage(e.getMessage());
				logger.error(e.getMessage(), e);
				return resultObject;
			}
			resultObject=this.wxPublicAccountService.sendMsgToGzh(parameters);
			logger.info("===============sendMsgToGzh()返回报文："+resultObject+"");
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	//================================================公众号 end==============================
	
}
