package com.up.sell.controller.api;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.up.sell.controller.AbstractController;
import com.up.sell.service.DictionaryService;
import com.up.sell.service.MemberBankService;
import com.up.sell.util.Constant;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.ResultObject;

/**
 * 银行卡相关
 * @author gust
 *
 */
@RestController
@RequestMapping("bank/auth")
public class MemberBankController extends AbstractController{
	
	private static final Logger logger = LoggerFactory.getLogger(MemberBankController.class);
	
	@Autowired
	DictionaryService dictionaryService;

	@Autowired
	MemberBankService memberBankService;
	
	
	/**
	 * 获取银行下拉列表
	 */
	@PostMapping("/getBankList")
	public ResultObject getBankList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject.put("bankList", dictionaryService.selectBankList("135"));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 绑定银行卡
	 * memberId
	 * @param request
	 * @return
	 */
	@PostMapping("/bindBankCard")
	public ResultObject bindBankCard(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject= memberBankService.bindBankCard(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 验证银行卡号是否绑定过
	 * cardNo 银行卡号
	 * @param request
	 * @return
	 */
	@PostMapping("/validateCardNo")
	public ResultObject validateCardNo(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberBankService.validateCardNo(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 解绑银行卡
	 * @param request
	 * @return
	 */
	@PostMapping("unbindBankCard")
	public ResultObject unbindBankCard(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberBankService.unbindBankCard(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 获取绑定银行卡列表
	 */
	@PostMapping("/bindBankCardList")
	public ResultObject bindBankCardList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("bankList", memberBankService.bindBankCardList(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 获取可提现列表
	 * @param request
	 * @return
	 */
	@PostMapping("getEncashmentList")
	public ResultObject getEncashmentList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("data", memberBankService.getEncashmentList(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	/**
	 * 创建提现申请
	 * @param request
	 * @return
	 */
	@PostMapping("encashment")
	public ResultObject encashment(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberBankService.encashment(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 获取提现列表详情
	 * @param request
	 * @return
	 */
	@PostMapping("selectBillDetail")
	public ResultObject selectBillDetail(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberBankService.selectBillDetail(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
}
