package com.up.sell.controller.api;


import java.util.List;
import java.util.Map;
import java.util.Objects;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.up.sell.controller.AbstractController;
import com.up.sell.service.AgreementService;
import com.up.sell.service.DictionaryService;
import com.up.sell.service.MemberService;
import com.up.sell.service.ProjectService;
import com.up.sell.util.Constant;
import com.up.sell.util.JWTUtil;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.Member;
import com.up.sell.vo.ResultObject;

/**
 * 我的模块
 * @author gust
 *
 */
@RestController
@RequestMapping("member")
public class MemberController extends AbstractController{

	private static final Logger logger = LoggerFactory.getLogger(MemberController.class);
	
	@Autowired
	DictionaryService dictionaryService;
	
	@Autowired
	MemberService memberService;
	
	@Autowired
	AgreementService agreementService;
	
	@Autowired
	ProjectService projectService;
	
	/**
	 * 关注列表
	 * @param parameters 
	 *  pageSize
	 *  pageNum
	 *  memberId 
	 *  projectType：1是销售 2是招商
	 * @return
	 */
	@PostMapping("/auth/favoriteList")
	public ResultObject list(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			Integer pageSize = Integer.valueOf(parameters.get("pageSize").toString());
			Integer pageNum = Integer.valueOf(parameters.get("pageNum").toString());
			Page<Object> startPage = PageHelper.startPage(pageNum, pageSize, true);
			List<Map<String,Object>> list=memberService.selectFavoriteList(parameters);
			if(list !=null && list.size()>0){
				for(Map<String,Object> temp:list){
					temp.put("price_u", this.projectService.jsonPriceU(parameters.get("projectType")+"", temp.get("price_type")+""));
				}
			}
			resultObject.put("favoriteList",list);
			resultObject.put("totalCount", startPage.getTotal());
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 我的模块取消关注(可多选)
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/cancelFavorites")
	public ResultObject cancelFavorites(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("favoriteList", memberService.cancelFavorites(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 获取登录用户信息
	 * memberId
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/memberInfo")
	public ResultObject memberInfo(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberService.getMemberByPrimaryKey(Integer.valueOf(parameters.get("loginIdentity").toString()));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	/**
	 * 未读消息提醒：1-有未读消息；0-没有
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/findPushStatus")
	public ResultObject findPushStatus(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			int pushStatus = memberService.findPushStatus(Integer.valueOf(parameters.get("loginIdentity").toString()));
			resultObject.put("pushStatus", pushStatus);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	
	/**
	 * 获取用户部分属性：姓名，手机号（不需要登录）
	 * memberId
	 * @param request
	 * @return
	 */
	@PostMapping("/unauth/memberSubInfo")
	public ResultObject memberSubInfo(HttpServletRequest request) {
		logger.info("loginIdentity : {}",request.getParameter("loginIdentity").toString());
		ResultObject resultObject = new ResultObject();
		try {
//			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberService.getMemberSubInfoByPrimaryKey(Integer.valueOf(request.getParameter("loginIdentity").toString()));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	/**
	 * 修改用户信息
	 * memberId
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/updateMemberInfo")
	public ResultObject updateMemberInfo(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			memberService.updateByPrimaryKey(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 获取协议内容
	 * @param request
	 * agreementType
			0.     发布品牌协议
			1.     发布销售协议
			2.     发布招商协议
			3.     发展团队协议
			4.     上传身份证认证说明
			5.     上传营业执照认证说明
			6.     提现规则
			7.     绑定银行卡说明	
	 * @return
	 */
	@PostMapping("/getAgreement")
	public ResultObject getAgreement(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("data", agreementService.selectByName(Integer.valueOf(parameters.get("agreementType").toString())));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 微信分享
	 * @param request
	 * @return
	 */
	@PostMapping("/weChatShare")
	public ResultObject weChatShare(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberService.weChatShare(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 创建二维码(团队)
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/createQRCode")
	public ResultObject createQRCode(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberService.createQRCode(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 获取购买产品列表
	 * @param request
	 * @return
	 */
	@PostMapping("/selectGoodsList")
	public ResultObject selectGoodsList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberService.selectGoodsList(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 获取会员信息
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/selectMemberInfo")
	public ResultObject selectVipInfo(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberService.selectMemberInfo(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 获取订单列表
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/selectOrderList")
	public ResultObject selectOrderList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberService.selectOrderList(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 十元订单检测接口
	 */
	@PostMapping("/auth/isHavenTen")
	public ResultObject selectOrderOfTen(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		System.out.println("ishaveTen run.....");
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = memberService.selectOrderList(parameters);
			List<Map<String,Object>> lm = (List<Map<String, Object>>) resultObject.get("orderList");
			System.out.println("lm : "+lm.size());
			for(int i=0;i<lm.size();i++) {
				if(1l != (Long)lm.get(i).get("goodsId")) {
					resultObject.put("isTen", false);
				}else {
					resultObject.put("isTen", true);
				}
			}
		} catch (ResultException e) {
			System.out.println("e1 : "+e);
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			System.out.println("e2 : "+e);
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 获取消息列表
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/selectPushList")
	public ResultObject selectPushList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			String token = request.getHeader("access-token");
			Member member = JWTUtil.unsign(token, Member.class);
			parameters.put("mobile", member.getMobile());
			resultObject = memberService.selectPushList(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 更改消息状态
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/updatePushStatus")
	public ResultObject updatePushStatus(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			String token = request.getHeader("access-token");
			Member member = JWTUtil.unsign(token, Member.class);
			parameters.put("mobile", member.getMobile());
			if(Objects.isNull(parameters.get("id"))) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("系统异常");
				return resultObject;
			}
			resultObject = memberService.updatePushStatus(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 删除消息
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/delPush")
	public ResultObject delPush(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			String token = request.getHeader("access-token");
			Member member = JWTUtil.unsign(token, Member.class);
			parameters.put("mobile", member.getMobile());
			if(Objects.isNull(parameters.get("id"))) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("系统异常");
				return resultObject;
			}
			resultObject = memberService.delPush(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	/**
	 * 未读消息提醒
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/checkPushStatus")
	public ResultObject checkPushStatus(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			String token = request.getHeader("access-token");
			Member member = JWTUtil.unsign(token, Member.class);
			parameters.put("mobile", member.getMobile());
			resultObject = memberService.checkPushStatus(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
}
