/**
 * 
 */
package com.up.sell.controller.api;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.up.sell.controller.AbstractController;
import com.up.sell.mapper.PriceRangeMapper;
import com.up.sell.service.AreasService;
import com.up.sell.service.DictionaryService;
import com.up.sell.service.ProjectService;
import com.up.sell.util.CodeGeneration;
import com.up.sell.util.Constant;
import com.up.sell.util.DateUtils;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.Member;
import com.up.sell.vo.ProjectOrder;
import com.up.sell.vo.ProjectReleased;
import com.up.sell.vo.ResultObject;

/**
 * @author zmg
 *
 */
@RestController
@RequestMapping(value = "/project")
public class ProjectController extends AbstractController {
	private static final Logger logger = LoggerFactory.getLogger(ProjectController.class);
	@Autowired
	private ProjectService projectService;
	@Autowired
	private AreasService areasService;
	@Autowired
	private PriceRangeMapper priceRangeMapper;
	@Autowired
	private DictionaryService dictionaryService;
	/**
	 * 项目列表页筛选条件接口
	 * @param request
	 *  cityId：APP所在城市
	 *  projectType：1是销售 2是招商
	 * @return
	 */
	@PostMapping("/filter")
	public ResultObject filter(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			//区域
			Map all = new HashMap();
			all.put("area_name", "全部");
			all.put("id", "");
			all.put("regions_id", "");
			List<Map> districtList = areasService.selectByParentId((String)parameters.get("cityId"));
			districtList.add(0, all);
			resultObject.put("districtList", districtList);
			
			List<Map<String,Object>> houseTypeList = new ArrayList<Map<String,Object>>();
			Map<String,Object> houseType = new HashMap<String,Object>();
			houseType.put("type_id", "");
			houseType.put("type_name", "全部");
			houseTypeList.add(houseType);
			houseType = new HashMap<String,Object>();
			houseType.put("type_id", 1);
			houseType.put("type_name", "商业");
			houseTypeList.add(houseType);
			houseType = new HashMap<String,Object>();
			houseType.put("type_id", 2);
			houseType.put("type_name", "办公产业");
			houseTypeList.add(houseType);
			if(StringUtils.equals("1", (String)parameters.get("projectType"))) {
				houseType = new HashMap<String,Object>();
				houseType.put("type_id", 3);
				houseType.put("type_name", "住宅");
				houseTypeList.add(houseType);
			}
			resultObject.put("houseTypeList", houseTypeList);//建筑类型
			//价格区间
			List<Map> priceRangeList = this.priceRangeMapper.selectByPriceType(Integer.valueOf((String)parameters.get("projectType")), (String)parameters.get("cityId"));
			all = new HashMap();
			all.put("description", "全部");
			all.put("beginPrice", "");
			all.put("endPrice", "");
			priceRangeList.add(0, all);
			resultObject.put("priceRangeList", priceRangeList);
			//面积区间
			List<Map<String,Object>> acreageList = this.dictionaryService.selectMapByParentKey("137");
			all = new HashMap();
			all.put("d_name", "全部");
			all.put("d_value", "");
			acreageList.add(0, all);
			resultObject.put("acreageList", acreageList);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 查询项目列表
	 * @param parameters 
	 *  pageSize
	 *  pageNum
	 * 	cityId：APP所在城市
	 *  projectType：1是销售 2是招商
	 *  houseType：1商业 2是办公 3住宅
	 *  districtId：区域
	 *  beginPrice、endPrice、acreageRangeValue、projectName
	 * @return
	 */
	@PostMapping("/list")
	public ResultObject list(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			Integer pageSize = Integer.valueOf((String)parameters.get("pageSize"));
			Integer pageNum = Integer.valueOf((String)parameters.get("pageNum"));
			Page<Object> startPage = PageHelper.startPage(pageNum, pageSize, true);
			resultObject.put("projectList", this.projectService.selectProjectList(parameters));
			if(StringUtils.equals("2", parameters.get("projectType").toString())) {
				resultObject.put("hotProjectList", projectService.getRecommendProjectList(parameters)) ;
			}
			resultObject.put("totalCount", startPage.getTotal());
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 项目详情
	 * @param request
	 * projectId、houseType、projectType
	 * @return
	 */
	@PostMapping("/detail")
	public ResultObject detail(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			Member  loginUser=this.getLoginMember(request);
			resultObject = this.projectService.getProjectDetail(Integer.valueOf((String)parameters.get("projectId")), 
					Integer.valueOf((String)parameters.get("projectType")), 
					Integer.valueOf((String)parameters.get("houseType")),
					loginUser==null?null:loginUser.getId(),
							Integer.valueOf((String)parameters.get("cityId")));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 会员关注项目
	 * @param request
	 * 		projectId
	 * @return
	 */
	@PostMapping("/auth/favorite")
	public ResultObject memberFavoriteProject(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			this.projectService.memberFavoriteProject(Integer.valueOf((String)parameters.get("projectId")), this.getLoginMember(request).getId());
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 会员取消关注项目
	 * @param request
	 * 		projectId
	 * @return
	 */
	@PostMapping("/auth/cancelFavorite")
	public ResultObject memberCancelFavoriteProject(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			this.projectService.memberCancelFavoriteProject(Integer.valueOf((String)parameters.get("projectId")), this.getLoginMember(request).getId());
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 项目发布接口
	 * @param projectReleased
	 * @param projectPics
	 * @return
	 */
	@PostMapping("/auth/projectReleased")
	public ResultObject projectReleased(HttpServletRequest request,ProjectReleased projectReleased) {
		ResultObject resultObject = new ResultObject();
		try {
			projectReleased.setCreateTime(new Date());
			projectReleased.setMemberId(this.getLoginMember(request).getId());
			projectReleased.setProjectStatus(1);
			this.projectService.projectReleased(projectReleased);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 地图找房
	 * @param request
	 * 		cityId、projectType
	 * @return
	 */
	@PostMapping("/projectOnMap")
	public ResultObject projectOnMap(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("projectList", this.projectService.projectOnMap(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 项目报备申请
	 * @param request
	 * 	projectId、customerId、appointmentTime、description、brandName
	 * @return
	 */
	@PostMapping("/auth/projectReported")
	public ResultObject projectReported (HttpServletRequest request,ProjectOrder projectOrder) {
		ResultObject resultObject = new ResultObject();
		try {
			projectOrder.setOrderNo(CodeGeneration.getOrderNo());
			Member member=this.getLoginMember(request);
			projectOrder.setMemberId(member.getId());
			projectOrder.setCreateTime(new Date());
			projectOrder.setOrderStatus(1);
			this.projectService.projectReported(projectOrder);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 我的发布
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/myPush")
	public ResultObject myPush (HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			if(parameters.get("pageSize")!=null 
				&& parameters.get("pageNum")!=null
				&& StringUtils.isNotBlank(parameters.get("pageSize").toString())
				&& StringUtils.isNotBlank(parameters.get("pageNum").toString())) {
				Integer pageSize = Integer.valueOf((String)parameters.get("pageSize"));
				Integer pageNum = Integer.valueOf((String)parameters.get("pageNum"));
				PageHelper.startPage(pageNum, pageSize, false);
			}
			resultObject.put("myPush", this.projectService.myPush(this.getLoginMember(request).getId()));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 订单详情接口
	 * @param request
	 * @return
	 */
	@PostMapping("/findOrderEntity")
	public ResultObject findOrderEntity (HttpServletRequest request) {
		Map<String, Object> parameters = ServletUtils.getParameters(request);
		return this.projectService.findOrderEntity(parameters.get("orderNo")+"");
	}
	

	/**
	 * 签约详情-新增
	 * @param request
	 * @return
	 */
	@PostMapping("/findManyUser")
	public ResultObject findManyUser(HttpServletRequest request) {
		Map<String, Object> parameters = ServletUtils.getParameters(request);
		return this.projectService.findManyUser(parameters.get("orderNo")+"");
	}
	
	
}
