/**
 * 
 */
package com.up.sell.controller.api;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.github.pagehelper.PageHelper;
import com.up.sell.controller.AbstractController;
import com.up.sell.service.ProjectService;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.Constant;
import com.up.sell.util.ObjectMyUtils;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.Member;
import com.up.sell.vo.ResultObject;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * @author zmg
 *
 */
@Api(value="ProjectManagerController",tags={"项目流程审核接口集"})
@RestController
@RequestMapping(value = "/projectManager")
public class ProjectManagerController extends AbstractController {
	private static final Logger logger = LoggerFactory.getLogger(ProjectManagerController.class);
	
	@Autowired
	private ProjectService projectService;
	public final static String img_head = ConfigurationUtils.getStringValue("img_head");
	
	@PostMapping("/auth/projectList")
	public ResultObject projectList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			if(StringUtils.isNotBlank((String)parameters.get("pageSize")) && StringUtils.isNotBlank((String)parameters.get("pageNum"))) {
				Integer pageSize = Integer.valueOf((String)parameters.get("pageSize"));
				Integer pageNum = Integer.valueOf((String)parameters.get("pageNum"));
				PageHelper.startPage(pageNum, pageSize, false);
			}
			parameters.put("memberId", this.getLoginMember(request).getId()+"");
			List<Map<String,Object>> list = this.projectService.projectManagerProjectList(parameters);
			list.stream().map(s ->{
				s.put("pictures", img_head+s.get("pictures"));
				return s;
				}).collect(Collectors.toList());
			
			resultObject.put("projectList", list);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 查询某个项目的交易记录
	 * @param request
	 * projectId、customerName
	 * @return
	 */
	@PostMapping("/auth/orderList")
	public ResultObject orderList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			if(StringUtils.isNotBlank((String)parameters.get("pageSize")) && StringUtils.isNotBlank((String)parameters.get("pageNum"))) {
				Integer pageSize = Integer.valueOf((String)parameters.get("pageSize"));
				Integer pageNum = Integer.valueOf((String)parameters.get("pageNum"));
				PageHelper.startPage(pageNum, pageSize, false);
			}
			parameters.put("projectManagerId", this.getLoginMember(request).getId());
			resultObject = this.projectService.getProjectManagerProjectDetail(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 某个项目的详情页各个订单状态的订单数量统计
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/orderStatusCount")
	public ResultObject orderStatusCount(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			parameters.put("projectManagerId", this.getLoginMember(request).getId());
			resultObject = this.projectService.orderStatusCount(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 某一项目下状态的详细信息
	 * @param request
	 * @param orderNo
	 * @return
	 */
	@PostMapping("/auth/orderDetail")
	public ResultObject orderDetail(HttpServletRequest request,String orderNo) {
		logger.debug("orderNo : [{}]",orderNo);
		ResultObject resultObject = new ResultObject();
		try {
			resultObject = this.projectService.selectOrderDetail(orderNo);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 查看签约详情
	 * @param request
	 * @param orderNo
	 * @return
	 */
	@PostMapping("/findSignDetail")
	public ResultObject findSignDetail(HttpServletRequest request,String orderNo) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject = this.projectService.findSignDetail(orderNo);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	
	
	/**
	 * 节点流转：录到访，录认购，录意向，录签约
	 * @param request
	 * orderNo订单号
	 * orderStatus（0结束交易、2审批、3录到访、4录认购/意向、5录签约）
	 * picPath上传的图片,多张图片用;分开
	 * description备注
	 * endCause拒绝原因
	 * customerName客户姓名
	 * mobile客户手机号
	 * identityCard身份证号
	 * shopNumber商铺编号
	 * price签约价格
	 * signTime签约日期
	 * acreage签约面积
	 * @return
	 */
	@PostMapping("/auth/approve")
	public ResultObject approve(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			Member  member=this.getLoginMember(request);
			parameters.put("memberId",member==null?"1": member.getId());
			if(parameters.containsKey("mobile")){
				String mobile=parameters.get("mobile")+"";
				parameters.put("mobile", ObjectMyUtils.cutDouHao(mobile));
			}
			if(parameters.containsKey("identityCard")){
				String identityCard=parameters.get("identityCard")+"";
				parameters.put("identityCard", ObjectMyUtils.cutDouHao(identityCard));
			}
			resultObject=this.projectService.approve(parameters);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	/**
	 * 备件管理 
	 * @param request
	 * @return
	 */
	@PostMapping("/auth/picManage")
	public ResultObject picManage(HttpServletRequest request,String orderNo) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject.put("picList", this.projectService.picManage(orderNo));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 上传新的凭证
	 * @param request
	 * @param orderNo
	 * @return
	 */
	@PostMapping("/auth/addPic")
	public ResultObject addPic(HttpServletRequest request,String picPath,Integer id) {
		ResultObject resultObject = new ResultObject();
		try {
			this.projectService.updatePic(picPath, id);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 多次到访备注录入
	 */
	@ApiOperation("新增到访备注接口(多次)")
	@ApiImplicitParams({@ApiImplicitParam(name = "picPath", value = "图片路径", required = true, paramType = "create", dataType = "String"),
					@ApiImplicitParam(name = "description", value = "备注说明", required = true, paramType = "create", dataType = "String"),
					@ApiImplicitParam(name = "orderTrackId", value = "流程ID", required = true, paramType = "create", dataType = "String")})
	@PostMapping("/auth/addDescr")
	public ResultObject addDescAndPic(HttpServletRequest request,String picPath,String description,String orderTrackId
			,String orderNo, String customerName, String mobile) {
		ResultObject resultObject = new ResultObject();
		System.out.println("orderNo : "+orderNo+"    orderTrackId : "+orderTrackId);
		try {
			projectService.updateDescrAndPic(picPath, description, orderTrackId, orderNo, customerName, mobile);
		}catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
}
