package com.up.sell.controller.api;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.up.sell.controller.AbstractController;
import com.up.sell.service.AreasService;
import com.up.sell.service.InformationChannelService;
import com.up.sell.service.ShowInfoService;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.Constant;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.ResultObject;

@RestController
@RequestMapping("show")
public class ShowInfoController  extends AbstractController {

	private static final Logger logger = LoggerFactory.getLogger(ShowInfoController.class);
	
	@Autowired
	ShowInfoService showInfoService;
	
	@Autowired
	AreasService areaService;
	
	@Autowired
	private InformationChannelService informationChannelService;
	
	/**
	 * 首页接口
	 * @param request
	 * 	cityId：APP所在城市
	 * @return
	 */
	@PostMapping("/showInfo")
	public ResultObject showInfo(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			//检测登录来源
			String userAgent = request.getHeader("user-agent");
			logger.info("userAgent:{}", userAgent);
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			System.out.println(request.getHeader(Constant.APP_HEADER_ACCESS_TOKEN));
			resultObject.put("data", showInfoService.getShowInfo(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	
	/**
	 * 开通城市列表
	 * @return
	 */
	@PostMapping("/getOpenedCitiesList")
	public ResultObject getOpenedCitiesList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			resultObject.put("data", areaService.selectOpenedCities());
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	
	/**
	 * 获取本月新品/成交前10
	 * @return
	 */
	@PostMapping("/getH5Page")
	public ResultObject getH5Page(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("data", showInfoService.selectH5Page(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	
	/**
	 * 获取定位城市信息
	 * @return
	 */
	@PostMapping("/changeCity")
	public ResultObject changeCity(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			resultObject= showInfoService.changeCity(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	
	/**
	 * 获取升级信息
	 * @return
	 */
	@PostMapping("/getUpgradeVersionInfo")
	public ResultObject getUpgradeVersionInfo(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			resultObject= showInfoService.getUpgradeVersionInfo(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	
	/**
	 * 获取官网品牌logo列表
	 * @return
	 */
	@PostMapping("/getBrandLogoList")
	public ResultObject getBrandLogoList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> parameters = ServletUtils.getParameters(request);
			parameters.put("picDomain", ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN));
			resultObject= showInfoService.getBrandLogoList(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	
	/**
	 * 资讯详情接口
	 * @param request
	 * 	id
	 * @return
	 */
	@PostMapping("/getInfoList")
	public ResultObject getInfoList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject.put("informationList", this.informationChannelService.selectInforList(parameters));
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    } 
	
}
