package com.up.sell.controller.api;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.up.sell.controller.AbstractController;
import com.up.sell.service.TeamService;
import com.up.sell.util.CheckParametersUtil;
import com.up.sell.util.Constant;
import com.up.sell.util.ResultException;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.ResultObject;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

@Api(value="TeamController",tags={"团队管理接口集"})
@RestController
@RequestMapping("team/auth")
public class TeamController extends AbstractController{

	private static final Logger logger = LoggerFactory.getLogger(TeamController.class);
	
	@Autowired
	TeamService teamService;

	/**
	 * 创建团队
	 * @param request
	 * memberId 
	 * teamType 1 个人 2 公司
	 * teamName 团队名称
	 * @return
	 */
	@PostMapping("/createTeam")
	public ResultObject createTeam(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = teamService.createTeam(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 解散
	 * @param request memberId
	 * @return
	 */
	@PostMapping("/disband")
	public ResultObject disband(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = teamService.disband(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 查询团队相关信息
	 * @param request memberId  type 1 推荐  2 团队 
	 * @return
	 */
	@PostMapping("/teamRelation")
	public ResultObject teamRelation(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = teamService.teamRelationList(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 是否可以解散
	 * @param request memberId
	 * @return
	 */
	@PostMapping("/isDisband")
	public ResultObject isDisband(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = teamService.isDisband(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	
	/**
	 * 查询团队等级
	 * @param request memberId
	 * @return
	 */
	@PostMapping("/selectTeamLevel")
	public ResultObject selectTeamLevel(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			resultObject = teamService.selectTeamLevel(parameters);
		} catch (ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
    }
	/**
	 * 获取团队列表
	 * @param request
	 * @return
	 */
	@PostMapping("selectTeamList")
	public ResultObject findTeamList(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String, Object> parameters = ServletUtils.getParameters(request);
			List<Map<String,Object>> teamList = teamService.selectTeamList(parameters.get("teamName")+"");
			resultObject.put("teamList", teamList);
		}catch(ResultException e) {
			resultObject.setCode(e.getCode());
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
		}catch(Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	/**
	 * 加入团队
	 * @param request
	 * @return
	 */
	@ApiOperation(value="加入团队接口",httpMethod = "POST")
	@ApiImplicitParams({@ApiImplicitParam(name = "teamName", value = "团队Id", required = true, paramType = "create", dataType = "String"),
					@ApiImplicitParam(name = "memberId", value = "用户Id", required = true, paramType = "create", dataType = "String")})
	@PostMapping("joinTeam")
	public ResultObject joinTeam(HttpServletRequest request) {
		ResultObject resultObject = new ResultObject();
		Map<String, Object> parameters = ServletUtils.getParameters(request);
		try {
			CheckParametersUtil.getInstance().put(parameters.get("teamName"), "teamName")
			.checkParameter();
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_PARAM_NULL);
			resultObject.setMessage(e.getMessage());
			logger.error(e.getMessage(), e);
			return resultObject;
		}
		Map<String,Object> map = teamService.joinTeam(parameters.get("teamName")+"", Integer.valueOf(parameters.get("loginIdentity")+""));
		resultObject.setCode(Integer.valueOf(map.get("code")+""));
		resultObject.setMessage(map.get("msg")+"");
		return resultObject;
	}
	
	
	
}
