package com.up.sell.interceptor;

import java.util.Enumeration;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import com.alibaba.fastjson.JSONObject;
import com.up.sell.redis.RedisBaseDao;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.Constant;
import com.up.sell.util.JWTUtil;
import com.up.sell.util.ServletUtils;
import com.up.sell.vo.Member;

@Component
public class LoginInteceptor extends HandlerInterceptorAdapter {

	private static final Logger logger = LoggerFactory.getLogger(LoginInteceptor.class);
	private static final ThreadLocal<Long> threadLocal = new ThreadLocal<Long>();
	@Autowired
	RedisBaseDao redisBaseDao;
	
	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
		threadLocal.set(System.currentTimeMillis());
		this.printParameter(request);
		String requestURI = request.getRequestURI().replace(request.getContextPath(), "");
		if (StringUtils.contains(requestURI, "/auth/")) { 
			String token = request.getHeader(Constant.APP_HEADER_ACCESS_TOKEN);
			if(StringUtils.isBlank(token)) {
				token = request.getParameter(Constant.APP_HEADER_ACCESS_TOKEN);
			}
			if("-1".equals(token)){
				return true;
			}
			if(StringUtils.isBlank(token)) {
				JSONObject jsonDate = new JSONObject();
			    jsonDate.put("code", Constant.CODE_NOT_LOGIN);
			    jsonDate.put("message", "未登录");
				ServletUtils.printWriter(response, jsonDate.toString());
				return false; 
			}
			Member member = JWTUtil.unsign(token, Member.class);
			if(member==null) {
				JSONObject jsonDate = new JSONObject();
			    jsonDate.put("code", Constant.CODE_TOKEN_EFFICACY);
			    jsonDate.put("message", "Token 失效");
				ServletUtils.printWriter(response, jsonDate.toString());
				return false; 
			}
		}
       return true;
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex)
			throws Exception {
		try {
			long time = System.currentTimeMillis() - threadLocal.get();
			if (time > ConfigurationUtils.getIntValue(Constant.METHOD_TIME)) {
				String user = "";
				StringBuffer logStr = new StringBuffer("access url path:")
						.append(ServletUtils.getCurrentURL(request))
						.append(", running time (millis) = ")
						.append(time)
						.append("	")
						.append(user)
						.append("	")
						.append(ServletUtils.getIpAddress(request))
						.append("	")
						.append(request.getHeader("User-Agent"));
				logger.debug(logStr.toString());
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}
	
	private void printParameter(HttpServletRequest request) {
		Enumeration<String> headerNames = request.getHeaderNames();
		String headerName = "";
		while (headerNames.hasMoreElements()) {
			headerName = headerNames.nextElement();
			logger.debug(headerName + ":" + request.getHeader(headerName));
		}
		Enumeration<String> parameterNames = request.getParameterNames();
		String parameterName = "";
		while (parameterNames.hasMoreElements()) {
			parameterName = parameterNames.nextElement();
			logger.debug(parameterName + ":" + request.getParameter(parameterName));
		}
	}
}
