/**
 * 
 */
package com.up.sell.pay.wx;

import java.security.MessageDigest;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;


/**
 * @author Administrator
 *
 */
public class WxAuth {
	private static final Logger logger = LoggerFactory.getLogger(WxAuth.class);
	
	/**
	 * 微信小程序取openid
	 * @param appId
	 * @param appSecret
	 * @param code
	 * @return
	 */
	public static Map<String,String> getOpenIDForMiniProgram(String appId, String appSecret, String code) {
		String url = "https://api.weixin.qq.com/sns/jscode2session?appid=" + appId + "&secret="
				+ appSecret + "&js_code=" + code + "&grant_type=authorization_code";
		String result = HttpUtil.sendHttpsGET(url);
		JSONObject obj = JSON.parseObject(result);
		Map<String,String> resultObject = new HashMap<String,String>();
		resultObject.put("session_key", obj.getString("session_key"));
		resultObject.put("openid", obj.getString("openid"));
		return resultObject;
	}
	
	/**
	 * 获取微信openID
	 * @param appId
	 * @param appSecret
	 * @param code
	 * @return
	 * {
		    "access_token": "oXbCZ7HzHfTte8oViZ3uIsXerN2uKY7B5JN0i6sjnNMMSHOag-z19AZlaCjzqK0Jz2JcvNeWDAAsSLbXUec_iw",
		    "expires_in": 7200,
		    "refresh_token": "s9DS1HoIkujnt940uS-fkoUQTEoscANdFwJC-yzjG8cEuYBh9gKdd4zWQvpRk8f1bZbj-R-IAn0QR2lTUNf3Ng",
		    "openid": "oI_3fwHT30V34NdS_r4TVXNt0300",
		    "scope": "snsapi_base"
		}
	 */
	public static Map<String,String> getOpenID(String appId, String appSecret, String code) {
		String url = "https://api.weixin.qq.com/sns/oauth2/access_token?appid=" + appId + "&secret="
				+ appSecret + "&code=" + code + "&grant_type=authorization_code";
		String result = HttpUtil.sendHttpsGET(url);
		JSONObject obj = JSON.parseObject(result);
		Map<String,String> resultObject = new HashMap<String,String>();
		resultObject.put("access_token", obj.getString("access_token"));
		resultObject.put("expires_in", obj.getString("expires_in"));
		resultObject.put("refresh_token", obj.getString("refresh_token"));
		resultObject.put("openid", obj.getString("openid"));
		resultObject.put("scope", obj.getString("scope"));
		return resultObject;
	}
	
	public static Map<String,String> getUserInfo(String accessToken, String openId) {
		String url = "https://api.weixin.qq.com/sns/userinfo?access_token=" + accessToken + "&openid="+openId+"&lang=zh_CN ";
		String result = HttpUtil.sendHttpsGET(url);
		JSONObject obj = JSON.parseObject(result);
		Map<String,String> resultObject = new HashMap<String,String>();
		resultObject.put("openId", obj.getString("openid"));
		resultObject.put("nickname", obj.getString("nickname"));
		resultObject.put("sex", obj.getString("sex"));
		resultObject.put("province", obj.getString("province"));
		resultObject.put("city", obj.getString("city"));
		resultObject.put("country", obj.getString("country"));
		resultObject.put("headimgurl", obj.getString("headimgurl"));
		resultObject.put("unionid", obj.getString("unionid"));
		return resultObject;
	}
	
	/**
     * 获取accessToken
     * 
     * @return
     */
    public static String getAccessToken(String appId, String appSecret) {
        String api = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid="+appId+"&secret="+appSecret;
        String result = HttpUtil.sendHttpsGET(api);
        logger.info("获取accessToken"+result);
		JSONObject obj = JSON.parseObject(result);
        return obj.getString("access_token");
    }
    
    /**
     * 
     * 获取jsapi_ticket
     * 
     * @return
     */
    public static String getJsapiTicket(String accessToken) {
        String api = "https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token="+accessToken+"&type=jsapi";
        String result = HttpUtil.sendHttpsGET(api);
        logger.info("获取jsapi_ticket"+result);
		JSONObject obj = JSON.parseObject(result);
        return obj.getString("ticket");
    }
    
    /**
     * 获得分享链接的签名。
     * @param ticket
     * @param nonceStr
     * @param timeStamp
     * @param url
     * @return
     * @throws Exception
     */
    public static String getSignature(String ticket, String nonceStr, String timeStamp, String url) throws Exception {
        String sKey = "jsapi_ticket=" + ticket
                + "&noncestr=" + nonceStr + "&timestamp=" + timeStamp
                + "&url=" + url;
        return getSignature(sKey);
    }


 /**
     * 验证签名。
     * 
     * @param signature
     * @param timestamp
     * @param nonce
     * @return
     */
    public static String getSignature(String sKey) throws Exception {
        String ciphertext = null;
        MessageDigest md = MessageDigest.getInstance("SHA-1");
        byte[] digest = md.digest(sKey.toString().getBytes());
        ciphertext = byteToStr(digest);
        return ciphertext.toLowerCase();
    }

 /** 
     * 将字节数组转换为十六进制字符串 
     *  
     * @param byteArray 
     * @return 
     */ 
    private static String byteToStr(byte[] byteArray) {  
        String strDigest = "";  
        for (int i = 0; i < byteArray.length; i++) {  
            strDigest += byteToHexStr(byteArray[i]);  
        }  
        return strDigest;  
    }  
  /** 
     * 将字节转换为十六进制字符串 
     *  
     * @param mByte 
     * @return 
     */ 
    private static String byteToHexStr(byte mByte) {  
        char[] Digit = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };  
        char[] tempArr = new char[2];  
        tempArr[0] = Digit[(mByte >>> 4) & 0X0F];  
        tempArr[1] = Digit[mByte & 0X0F];  

        String s = new String(tempArr);  
        return s;  
    }
    
    public static String getTimeStamp() {
		return String.valueOf(System.currentTimeMillis() / 1000);
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
