/**
 * 
 */
package com.up.sell.pay.wx;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.InetAddress;
import java.security.KeyStore;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.UUID;

import javax.net.ssl.SSLContext;

import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.conn.ssl.SSLContexts;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.tools.ant.filters.StringInputStream;
import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.up.sell.util.CodeGeneration;

/**
 * @author zhangmg
 *
 */
public class WxPay {
	
	private static final Logger logger = LoggerFactory.getLogger(WxPay.class);
	
	/**
	 * APP支付
	 * @param appId
	 * @param partnerKey
	 * @param partner
	 * @param out_trade_no
	 * @param total_amount
	 * @param notifyUrl
	 * @param userIP
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public static Map<String,Object> getAppWxPayInfo(String appId,String partnerKey,String partner,String out_trade_no,double total_amount,String notifyUrl,String userIP) throws UnsupportedEncodingException  {
		String totalFee = new BigDecimal(total_amount+"").multiply(new BigDecimal("100")).intValue() + "";//总金额，表示1分
		RequestHandler reqHandler = new RequestHandler("UTF-8");
		reqHandler.init(appId, partnerKey);   //初始化 
		//设置package订单参数
		String noncestr = Sha1Util.getNonceStr();  //随机字符串
		SortedMap<String, String> packageParams = new TreeMap<String, String>();
		packageParams.put("appid", appId);
		packageParams.put("mch_id", partner); //设置商户号
		packageParams.put("nonce_str", noncestr);  //随机数
		packageParams.put("out_trade_no", out_trade_no); //商户订单号  
		packageParams.put("body", "商品"); //商品描述   
		packageParams.put("total_fee", totalFee); //商品总金额,以分为单位
		packageParams.put("spbill_create_ip", userIP); //订单生成的机器IP，指用户浏览器端IP
		packageParams.put("notify_url", notifyUrl); //通知地址
		packageParams.put("trade_type", "APP"); //交易类型
		//获取package包
		String packageValue = reqHandler.genPackage(packageParams);
		//提交预支付 返回 预支付交易会话标识
		String prepayId = reqHandler.sendPrepay(packageValue);
		String timestamp = Sha1Util.getTimeStamp();  //时间戳
		//设置支付参数
		SortedMap<String, String> signParams = new TreeMap<String, String>();
		signParams.put("appid", appId);
		signParams.put("partnerid", partner);
		signParams.put("prepayid", prepayId);
		signParams.put("noncestr", noncestr);
		signParams.put("timestamp", timestamp);
		signParams.put("package", "Sign=WXPay");
		String sign = reqHandler.createSign(signParams); //创建MD5签名
		Map<String,Object> data = new HashMap<String, Object>();
		data.put("partnerid", partner);
		data.put("prepayid", prepayId);
		data.put("noncestr", noncestr);
		data.put("timestamp", timestamp);
		data.put("sign", sign);
		data.put("appid", appId);
		data.put("package", "Sign=WXPay");
		data.put("amount", total_amount);
		data.put("orderNo", out_trade_no);
		logger.debug("data="+data);
		return data;
	}
	/**
	 * 微信公众号支付
	 * @param appId 微信公众号appid
	 * @param partnerKey 私匙 微信支付key
	 * @param partner 微信支付商户号
	 * @param paymentBill
	 * @param notifyUrl
	 * @param ip
	 * @param openid
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public static Map<String,Object> getWxPayInfo(String appId,String partnerKey,String partner,String out_trade_no,double total_amount,String notifyUrl,String ip,String openid) throws UnsupportedEncodingException  {
		String totalFee = new BigDecimal(total_amount+"").multiply(new BigDecimal("100")).intValue() + "";//总金额，表示1分
		RequestHandler reqHandler = new RequestHandler("UTF-8");
		reqHandler.init(appId, partnerKey);   //初始化 
		//设置package订单参数
		String noncestr = Sha1Util.getNonceStr();  //随机字符串
		SortedMap<String, String> packageParams = new TreeMap<String, String>();
		packageParams.put("appid", appId);
		packageParams.put("mch_id", partner); //设置商户号
		packageParams.put("nonce_str", noncestr);  //随机数
		packageParams.put("out_trade_no", out_trade_no); //商户订单号  
		packageParams.put("body", "优铺会员服务费"); //商品描述   
		packageParams.put("total_fee", totalFee); //商品总金额,以分为单位
		packageParams.put("spbill_create_ip", ip); //订单生成的机器IP，指用户浏览器端IP
		packageParams.put("notify_url", notifyUrl); //通知地址
		packageParams.put("trade_type", "JSAPI"); //交易类型
		packageParams.put("openid", openid); 

		//获取package包
		String packageValue = reqHandler.genPackage(packageParams);
		//提交预支付 返回 预支付交易会话标识
		String prepayId = reqHandler.sendPrepay(packageValue);
		String timestamp = WxAuth.getTimeStamp();  //时间戳
		noncestr =  Sha1Util.getNonceStr(); 
		//设置支付参数
		SortedMap<String, String> signParams = new TreeMap<String, String>();
		signParams.put("appId", appId);
		signParams.put("timeStamp", timestamp);
		signParams.put("nonceStr", noncestr);
		signParams.put("package", "prepay_id="+prepayId);
		signParams.put("signType", "MD5");
		String sign = reqHandler.createSign(signParams); //创建MD5签名
		signParams.put("paySign", sign);
		
		Map<String,Object> data = new HashMap<String, Object>();
		data.put("appid", appId);
		data.put("noncestr", noncestr);
		data.put("timestamp", timestamp);
		data.put("sign", sign);
		data.put("signType", "MD5");
		data.put("package", "prepay_id="+prepayId);
		
		logger.debug("data="+data);
		return data;
	}
	
	/**
	 * 微信H5支付，用于非微信浏览器
	 * @param appId 微信公众号appid
	 * @param partnerKey 私匙 微信支付key
	 * @param partner 微信支付商户号
	 * @param paymentBill
	 * @param notifyUrl
	 * @param ip
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public static String getWxH5PayInfo(String appId,String partnerKey,String partner,String out_trade_no,double total_amount,String notifyUrl,String ip) throws UnsupportedEncodingException  {
		String totalFee = new BigDecimal(total_amount+"").multiply(new BigDecimal("100")).intValue() + "";//总金额，表示1分
		RequestHandler reqHandler = new RequestHandler("UTF-8");
		reqHandler.init(appId, partnerKey);   //初始化 
		//设置package订单参数
		String noncestr = Sha1Util.getNonceStr();  //随机字符串
		SortedMap<String, String> packageParams = new TreeMap<String, String>();
		packageParams.put("appid", appId);
		packageParams.put("mch_id", partner); //设置商户号
		packageParams.put("nonce_str", noncestr);  //随机数
		packageParams.put("out_trade_no", out_trade_no); //商户订单号  
		packageParams.put("body", "会员商品"); //商品描述   
		packageParams.put("total_fee", totalFee); //商品总金额,以分为单位
		packageParams.put("spbill_create_ip", ip); //订单生成的机器IP，指用户浏览器端IP
		packageParams.put("notify_url", notifyUrl); //通知地址
		packageParams.put("trade_type", "MWEB"); //交易类型
		packageParams.put("scene_info", "{\"h5_info\": {\"type\":\"Wap\",\"wap_url\": \"http://m.youpuglobal.com\",\"wap_name\": \"卖铺宝\"}}"); 
		//获取package包
		String packageValue = reqHandler.genPackage(packageParams);

		String mweb_url = reqHandler.getMwebUrl(packageValue);
		
		logger.debug("mweb_url="+mweb_url +"&redirect_url=http%3a%2f%2fm.youpuglobal.com%2fmyMember");
		return mweb_url+"&redirect_url=http%3a%2f%2fm.youpuglobal.com%2fmyMember";
	}
	/**
	 * 微信扫码支付
	 * @param appId
	 * @param partnerKey
	 * @param partner
	 * @param out_trade_no
	 * @param total_amount
	 * @param notifyUrl
	 * @param ip
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public static String wxSaoMaPay(String appId,String partnerKey,String partner,String out_trade_no,double total_amount,String notifyUrl,String ip) throws UnsupportedEncodingException {
		String totalFee = new BigDecimal(total_amount+"").multiply(new BigDecimal("100")).intValue() + "";//总金额，表示1分
		RequestHandler reqHandler = new RequestHandler("UTF-8");
		reqHandler.init(appId, partnerKey);   //初始化 
		//设置package订单参数
		String noncestr =  Sha1Util.getNonceStr();  //随机字符串
		SortedMap<String, String> packageParams = new TreeMap<String, String>();
		packageParams.put("appid", appId);
		packageParams.put("mch_id", partner); //设置商户号
		packageParams.put("nonce_str", noncestr);  //随机数
		packageParams.put("out_trade_no", out_trade_no); //商户订单号  
		packageParams.put("body", "万卉源商品"); //商品描述   
		packageParams.put("total_fee", totalFee); //商品总金额,以分为单位
		packageParams.put("spbill_create_ip", ip); //订单生成的机器IP，指用户浏览器端IP
		packageParams.put("notify_url", notifyUrl); //通知地址
		packageParams.put("trade_type", "NATIVE"); //交易类型

		//获取package包
		String packageValue = reqHandler.genPackage(packageParams);
		String code_url = reqHandler.getCodeUrl(packageValue);
		logger.debug("code_url="+code_url);
		return code_url;
	}
	
	/**
	 * 微信刷卡支付
	 * @param appId
	 * @param partnerKey
	 * @param partner
	 * @param out_trade_no
	 * @param total_amount
	 * @param notifyUrl
	 * @param ip 调用微信支付API的机器IP
	 * @param authCode
	 * @return
	 * @throws UnsupportedEncodingException
	 */
	public static Map<String,String> wxShuaKaPay(String appId,String partnerKey,String partner,String out_trade_no,double total_amount,String ip,String authCode) throws UnsupportedEncodingException {
		Map<String,String> resultMap = new HashMap<String,String>();
		String totalFee = new BigDecimal(total_amount+"").multiply(new BigDecimal("100")).intValue() + "";//总金额，表示1分
		RequestHandler reqHandler = new RequestHandler("UTF-8");
		reqHandler.init(appId, partnerKey);   //初始化 
		//设置package订单参数
		String noncestr = UUID.randomUUID().toString(); //随机字符串
		SortedMap<String, String> packageParams = new TreeMap<String, String>();
		packageParams.put("appid", appId);
		packageParams.put("mch_id", partner); //设置商户号
		packageParams.put("nonce_str", noncestr);  //随机数
		packageParams.put("out_trade_no", out_trade_no); //商户订单号  
		packageParams.put("body", "万卉源商品"); //商品描述   
		packageParams.put("total_fee", totalFee); //商品总金额,以分为单位
		packageParams.put("spbill_create_ip", ip); //订单生成的机器IP，指用户浏览器端IP
		packageParams.put("auth_code", authCode); //交易类型
		//获取package包
		String packageValue = reqHandler.genPackage(packageParams);
		
		Document document = XMLUtil.builder.newDocument();
    	Element root = document.createElement("xml");
    	String [] params = packageValue.split("&");
    	for (int i = 0; i < params.length; i++) {
			String [] p = params[i].split("=");
			Element contentElement = document.createElement(p[0]);
			contentElement.setTextContent(p[1]);
			root.appendChild(contentElement);
		}
    	document.appendChild(root);
    	String doms = XMLUtil.toStringFromDoc(document);
    	String url = "https://api.mch.weixin.qq.com/pay/micropay";
    	String result = HttpUtil.sendHttpsPOST(url, doms);
    	logger.debug(result);
    	try {
    		java.io.InputStream is = new StringInputStream(result);
    		SAXReader saxReader = new SAXReader();  
            org.dom4j.Document d = saxReader.read(is); 
            org.dom4j.Element rootElt = d.getRootElement(); 
    		
			resultMap.put("openid",rootElt.elementText("openid"));
			resultMap.put("is_subscribe",rootElt.elementText("is_subscribe"));
			resultMap.put("trade_type",rootElt.elementText("trade_type"));
			resultMap.put("bank_type",rootElt.elementText("bank_type"));
			resultMap.put("fee_type",rootElt.elementText("fee_type"));
			resultMap.put("total_fee",rootElt.elementText("total_fee"));
			resultMap.put("cash_fee_type",rootElt.elementText("cash_fee_type"));
			resultMap.put("transaction_id",rootElt.elementText("transaction_id"));
			resultMap.put("out_trade_no",rootElt.elementText("out_trade_no"));
			resultMap.put("time_end",rootElt.elementText("time_end"));
			resultMap.put("return_code",rootElt.elementText("return_code"));
			resultMap.put("return_msg",rootElt.elementText("return_msg"));
			resultMap.put("result_code",rootElt.elementText("result_code"));
			resultMap.put("err_code",rootElt.elementText("err_code"));
			resultMap.put("err_code_des",rootElt.elementText("err_code_des"));
			is.close();
		} catch (DocumentException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return resultMap;
	}
	
	/**
	 * 微信退款
	 * @param appId
	 * @param partnerKey
	 * @param partner
	 * @param cert 证书路径	WxPay.class.getClassLoader().getResource("configs/cert/wanhuiyuan/").getPath() + "apiclient_cert.p12";
	 * @param refundOrderNo	退款单号
	 * @param orderNo	被退款订单号
	 * @param amount	订单金额
	 * @param refundAmount	退款金额
	 * @return return_code = SUCCESS and result_code = SUCCESS
	 */
	public static Map<String,String> refund (String refundOrderNo, String orderNo,double amount,double refundAmount,String appId,String partnerKey,String partner,InputStream cert) {
		CloseableHttpClient httpClient = null;
		CloseableHttpResponse responseEntry = null;
		Map<String,String> result = new HashMap<String, String>();
		try {
			String url = "https://api.mch.weixin.qq.com/secapi/pay/refund";
			String refundFee = new BigDecimal(refundAmount+"").multiply(new BigDecimal("100")).intValue() + "";//总金额，表示1分
			String totalFee = new BigDecimal(amount+"").multiply(new BigDecimal("100")).intValue() + "";//总金额，表示1分
			
			//设置package订单参数
			String noncestr = Sha1Util.getNonceStr();  //随机字符串
			SortedMap<String, String> packageParams = new TreeMap<String, String>();
			packageParams.put("appid", appId);
			packageParams.put("mch_id", partner); //设置商户号
			packageParams.put("nonce_str", noncestr);  //随机数
			packageParams.put("op_user_id", partner);  //
			packageParams.put("out_trade_no", orderNo); //商户订单号
			packageParams.put("out_refund_no", refundOrderNo); //退款单号
			packageParams.put("total_fee", totalFee); //订单金额,以分为单位
			packageParams.put("refund_fee", refundFee); //退款金额,以分为单位
			
			RequestHandler reqHandler = new RequestHandler("UTF-8");
			reqHandler.init(appId, partnerKey);   //初始化 
			String sign = reqHandler.createSign(packageParams); //创建MD5签名
			
			String bizContent = "<xml>"+
			"<appid>"+appId+"</appid>"+
			"<mch_id>"+partner+"</mch_id>"+
			"<nonce_str>"+noncestr+"</nonce_str>"+
			"<op_user_id>"+partner+"</op_user_id>"+
			"<out_refund_no>"+refundOrderNo+"</out_refund_no>"+
			"<out_trade_no>"+orderNo+"</out_trade_no>"+
			"<refund_fee>"+refundFee+"</refund_fee>"+
			"<total_fee>"+totalFee+"</total_fee>"+
			"<transaction_id></transaction_id>"+
			"<sign>"+sign+"</sign>"+
			"</xml>";
			logger.debug(bizContent);
//			String path = RefundUtil.class.getClassLoader().getResource("configs/cert/zx/").getPath() + "apiclient_cert.p12";
			KeyStore keyStore  = KeyStore.getInstance("PKCS12");
//            FileInputStream instream = new FileInputStream(new File(cert));
            try {
                keyStore.load(cert, partner.toCharArray());
            } finally {
                cert.close();
            }
            SSLContext sslcontext = SSLContexts.custom().loadKeyMaterial(keyStore, partner.toCharArray()).build(); 
            SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslcontext,new String[] { "TLSv1" },null,SSLConnectionSocketFactory.BROWSER_COMPATIBLE_HOSTNAME_VERIFIER);
            httpClient = HttpClients.custom().setSSLSocketFactory(sslsf).build();
            HttpPost httpost = new HttpPost(url);
            httpost.setEntity(new StringEntity(bizContent, "UTF-8"));
            responseEntry = httpClient.execute(httpost);
            org.dom4j.io.SAXReader saxReader = new SAXReader();  
            org.dom4j.Document document = saxReader.read(responseEntry.getEntity().getContent()); 
            org.dom4j.Element rootElt = document.getRootElement(); 
            
            result.put("return_code", rootElt.elementText("return_code"));
            result.put("return_msg", rootElt.elementText("return_msg"));
            result.put("result_code", rootElt.elementText("result_code"));
            result.put("err_code", rootElt.elementText("err_code"));
            result.put("err_code_des", rootElt.elementText("err_code_des"));
            result.put("appid", rootElt.elementText("appid"));
            result.put("mch_id", rootElt.elementText("mch_id"));
            result.put("device_info", rootElt.elementText("device_info"));
            result.put("nonce_str", rootElt.elementText("nonce_str"));
            result.put("sign", rootElt.elementText("sign"));
            result.put("transaction_id", rootElt.elementText("transaction_id"));
            result.put("out_trade_no", rootElt.elementText("out_trade_no"));
            result.put("out_refund_no", rootElt.elementText("out_refund_no"));
            result.put("refund_id", rootElt.elementText("refund_id"));
            result.put("refund_fee", rootElt.elementText("refund_fee"));
            result.put("settlement_refund_fee", rootElt.elementText("settlement_refund_fee"));
            result.put("total_fee", rootElt.elementText("total_fee"));
            result.put("settlement_total_fee", rootElt.elementText("settlement_total_fee"));
            result.put("fee_type", rootElt.elementText("fee_type"));
            result.put("cash_fee", rootElt.elementText("cash_fee"));
            result.put("cash_fee_type", rootElt.elementText("cash_fee_type"));
            result.put("cash_refund_fee", rootElt.elementText("cash_refund_fee"));
            
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		} finally {
			try {
				responseEntry.close();
				httpClient.close();
			} catch (IOException e) {
				logger.error(e.getMessage(), e);
			}
		}
		logger.debug(result.toString());
		return result;
	}
	
	/**
	 * 给同一个实名用户付款，单笔单日限额2W/2W,
	 * 不支持给非实名用户打款,
	 * 一个商户同一日付款总额限额100W,
	 * 单笔最小金额默认为1元,
	 * 每个用户每天最多可付款10次，可以在商户平台--API安全进行设置,
	 * 当返回错误码为“SYSTEMERROR”时，一定要使用原单号重试，否则可能造成重复支付等资金风险。
	 * 
	 * @param taNo 转账单号
	 * @param reUserName 收款方微信实名认证的姓名
	 * @param amount 转账金额
	 * @param remark 转账描述
	 * @param openid 收款方 
	 * @return
	 * https://pay.weixin.qq.com/wiki/doc/api/tools/mch_pay.php?chapter=14_2
	 */
	public static Map<String,String> transferAccounts (String taNo,String reUserName,double amount,String remark,String openid,InputStream cert,String appId,String partnerKey,String partner) {
		CloseableHttpClient httpClient = null;
		CloseableHttpResponse responseEntry = null;
		Map<String,String> result = new HashMap<String, String>();
		try {
			String url = "https://api.mch.weixin.qq.com/mmpaymkttransfers/promotion/transfers";
			String taAmount = new BigDecimal(amount+"").multiply(new BigDecimal("100")).intValue() + "";//转账金额 单位分
			
			//设置package订单参数
			String noncestr = Sha1Util.getNonceStr();  //随机字符串
			String ip = InetAddress.getLocalHost().getHostAddress();//获得本机IP	
			SortedMap<String, String> packageParams = new TreeMap<String, String>();
			packageParams.put("mch_appid", appId);
			packageParams.put("mchid", partner); //设置商户号
			packageParams.put("nonce_str", noncestr);  //随机数
			packageParams.put("partner_trade_no", taNo); //商户订单号
			packageParams.put("openid", openid); 
			packageParams.put("check_name", "NO_CHECK"); 
			packageParams.put("re_user_name", reUserName); 
			packageParams.put("amount", taAmount); 
			packageParams.put("desc", remark); 
			packageParams.put("spbill_create_ip", ip); 
			
			RequestHandler reqHandler = new RequestHandler("UTF-8");
			reqHandler.init(appId, partnerKey);   //初始化 
			String sign = reqHandler.createSign(packageParams); //创建MD5签名
			
			String bizContent = "<xml>"+
			"<mch_appid>"+appId+"</mch_appid>"+
			"<mchid>"+partner+"</mchid>"+
			"<nonce_str>"+noncestr+"</nonce_str>"+
			"<partner_trade_no>"+taNo+"</partner_trade_no>"+
			"<openid>"+openid+"</openid>"+
			"<check_name>NO_CHECK</check_name>"+
			"<re_user_name>"+reUserName+"</re_user_name>"+
			"<amount>"+taAmount+"</amount>"+
			"<desc>"+remark+"</desc>"+
			"<spbill_create_ip>"+ip+"</spbill_create_ip>"+
			"<sign>"+sign+"</sign>"+
			"</xml>";
			logger.info(bizContent);
//			String path = TransferAccountsUtil.class.getClassLoader().getResource("config/cert/zonggongsi/").getPath() + "apiclient_cert.p12";
			KeyStore keyStore  = KeyStore.getInstance("PKCS12");
//            FileInputStream instream = new FileInputStream(new File(path));
            try {
                keyStore.load(cert, partner.toCharArray());
            } finally {
            	cert.close();
            }
            SSLContext sslcontext = SSLContexts.custom().loadKeyMaterial(keyStore, partner.toCharArray()).build(); 
            SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslcontext,new String[] { "TLSv1" },null,SSLConnectionSocketFactory.BROWSER_COMPATIBLE_HOSTNAME_VERIFIER);
            httpClient = HttpClients.custom().setSSLSocketFactory(sslsf).build();
            HttpPost httpost = new HttpPost(url);
            httpost.setEntity(new StringEntity(bizContent, "UTF-8"));
            responseEntry = httpClient.execute(httpost);
			SAXReader saxReader = new SAXReader();  
			org.dom4j.Document document = saxReader.read(responseEntry.getEntity().getContent()); 
			org.dom4j.Element rootElt = document.getRootElement(); 
            
            result.put("return_code", rootElt.elementText("return_code"));//SUCCESS/FAIL 此字段是通信标识，非交易标识，交易是否成功需要查看result_code来判断
            result.put("return_msg", rootElt.elementText("return_msg"));//返回信息，如非空，为错误原因  签名失败 参数格式校验错误
            result.put("result_code", rootElt.elementText("result_code"));//SUCCESS/FAIL
            result.put("err_code", rootElt.elementText("err_code"));//错误码信息
            result.put("err_code_des", rootElt.elementText("err_code_des"));//结果信息描述
            result.put("mch_appid", rootElt.elementText("mch_appid"));
            result.put("mchid", rootElt.elementText("mchid"));
            result.put("device_info", rootElt.elementText("device_info"));//微信支付分配的终端设备号
            result.put("nonce_str", rootElt.elementText("nonce_str"));
            result.put("partner_trade_no", rootElt.elementText("partner_trade_no"));//商户订单号，需保持唯一性(只能是字母或者数字，不能包含有符号)
            result.put("payment_no", rootElt.elementText("payment_no"));//企业付款成功，返回的微信订单号
            result.put("payment_time", rootElt.elementText("payment_time"));//企业付款成功，返回的微信订单号2015-05-19 15：26：59
            
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		} finally {
			try {
				responseEntry.close();
				httpClient.close();
			} catch (IOException e) {
				logger.error(e.getMessage(), e);
			}
		}
		logger.info(result.toString());
		return result;
	}
	/**
	 * @param args
	 * @throws UnsupportedEncodingException 
	 */
//	 public final static String APPID = "wx7fe223bf9f6e4829";//服务号的应用号
//	 public final static String MCH_ID = "1488781502";//商户号
//	 public final static String API_KEY = "3e84b6613b6b35c2a6190ca5c15690c8";//API密钥
//	 public final static String NOTIFY_URL = "http://qptt8g.natappfree.cc/WXPayCallBackApi/callBack"; 
	//微信支付统一接口(POST)
	public final static String UNIFIED_ORDER_URL = "https://api.mch.weixin.qq.com/pay/unifiedorder";
	public static void main(String[] args) throws UnsupportedEncodingException {
		String appId = "wx7fe223bf9f6e4829";
		String partnerKey = "UoHcOi9L94MGAK3CsgAvhTUdJqesEbwo";
		String partner = "1491974392";
		String out_trade_no = CodeGeneration.getOrderNo();
		double total_amount = 0.01;
		String notifyUrl = "http://3asf9j.natappfree.cc/WXPayCallBackApi/callBack";
		String ip = "114.248.127.186";
		//String openid = "o407CwkTiAmDeU8gdJA43VI0J8gQ";
		//System.out.println(WxPay.getAppWxPayInfo(appId, partnerKey, partner, out_trade_no, total_amount, notifyUrl, ip));
		//System.out.println(WxPay.getWxPayInfo(appId, partnerKey, partner, out_trade_no, total_amount, notifyUrl, ip, openid));
		System.out.println(WxPay.getWxH5PayInfo(appId, partnerKey, partner, out_trade_no, total_amount, notifyUrl, ip));
	}

}
