package com.up.sell.service;

import java.util.List;
import java.util.Map;

import com.up.sell.vo.Member;
import com.up.sell.vo.ResultObject;

public interface MemberService {

	/**
	 * 登录
	 * @param mobile
	 * @param validCode
	 * @param provinceId
	 * @param cityId
	 * @param type：1、登录 2、推荐 3、加入团队
	 * @param referrerId 推荐人ID
	 * @param teamId 加入团队ID
	 * @return
	 */
	public ResultObject login(String mobile,String validCode,String provinceId,String cityId,Integer type,Integer referrerId,Integer teamId);
	
	/**
	 * 发送短信
	 * @param phone 手机号
	 * @return
	 */
	Map<String,Object> sendSms(String phone,String type);
	
	
	/**
	 * 查看关注列表
	 * @param parameters
	 * @return
	 */
	List<Map<String,Object>> selectFavoriteList(Map<String,Object> parameters);
	
	/**
	 * 根据id查询个人信息
	 * @param id
	 * @return
	 */
	ResultObject getMemberByPrimaryKey(Integer id);
	/**
	 * 根据id查询个人手机号、姓名（不需要登录）
	 * @param id
	 * @return
	 */
	ResultObject getMemberSubInfoByPrimaryKey(Integer id);
	
	/**
	 * 修改个人信息
	 * @param parameters
	 * @return 
	 * @return
	 */
	void updateByPrimaryKey(Map<String,Object> parameters);

	/**
	 * 团队推广二维码
	 * @param parameters
	 * @return
	 */
	public ResultObject createQRCode(Map<String, Object> parameters);

	/**
	 * 微信分享
	 * @param parameters
	 * @return
	 */
	public ResultObject weChatShare(Map<String, Object> parameters);

	/**
	 * 获取购买商品列表
	 * 
	 * @param parameters
	 * @return
	 */
	public ResultObject selectGoodsList(Map<String, Object> parameters);

	/**
	 * 获取会员信息
	 * @param parameters
	 * @return
	 */
	public ResultObject selectMemberInfo(Map<String, Object> parameters);

	/**
	 * 获取订单列表
	 * @param parameters
	 * @return
	 */
	public ResultObject selectOrderList(Map<String, Object> parameters);

	/**
	 * 我的模块取消关注(可多选)
	 * @param parameters
	 * @return
	 */
	public ResultObject cancelFavorites(Map<String, Object> parameters);

	/**
	 * 查询当前登录用户是否有未读消息
	 * @param parameters
	 * @return
	 */
	public ResultObject checkPushCount(Map<String, Object> parameters);

	/**
	 * 获取消息列表
	 * @param parameters
	 * @return
	 */
	public ResultObject selectPushList(Map<String, Object> parameters);

	/**
	 * 更改消息状态
	 * @param parameters
	 * @return
	 */
	public ResultObject updatePushStatus(Map<String, Object> parameters);

	/**
	 * 删除消息
	 * @param parameters
	 * @return
	 */
	public ResultObject delPush(Map<String, Object> parameters);

	/**
	 * 查询是否有未读消息
	 * @param parameters
	 * @return
	 */
	public ResultObject checkPushStatus(Map<String, Object> parameters);

	Member selectStatus(String mobile);

	Member selectStatusById(String id);

	int findPushStatus(Integer memberId);
}
