package com.up.sell.service;

import java.util.List;
import java.util.Map;

import com.up.sell.vo.ProjectOrder;
import com.up.sell.vo.ProjectReleased;
import com.up.sell.vo.ResultObject;

public interface ProjectService {

	/**
	 * 查询项目列表
	 * @param parameters
	 * cityId、projectType、houseType、beginPrice、endPrice、acreageRangeValue、districtId、projectName
	 * @return
	 */
	public List<Map<String,Object>> selectProjectList(Map<String,Object> parameters);
	/**
	 * 查询项目详情
	 * @param projectId
	 * @param houseType
	 * @param cityId 
	 * @return
	 */
	public ResultObject getProjectDetail(Integer projectId,Integer projectType,Integer houseType,Integer memberId, Integer cityId);
	/**
	 * 会员关注项目
	 * @param projectId
	 * @param memberId
	 * @return
	 */
	public int memberFavoriteProject(Integer projectId,Integer memberId);
	/**
	 * 会员取消关注
	 * @param projectId
	 * @param memberId
	 * @return
	 */
	public int memberCancelFavoriteProject(Integer projectId,Integer memberId);
	/**
	 * 发布项目
	 * @param projectReleased
	 * @return
	 */
	public int projectReleased(ProjectReleased projectReleased);
	/**
	 * 地图找房
	 * @param parameters
	 * @return
	 */
	public List<Map<String,Object>> projectOnMap(Map parameters);
	/**
	 * 项目报备申请
	 * @param projectOrder
	 */
	public void projectReported(ProjectOrder projectOrder);
	/**
	 * 我的发布
	 * @param memberId
	 * @return
	 */
	public List<Map<String,Object>> myPush(Integer memberId);
	/**
	 * 业务首页项目列表
	 * @param parameters
	 * @return
	 */
	public List<Map<String,Object>> projectManagerProjectList(Map parameters);
	/**
	 * 项目经理某个项目的项目详情
	 * @param projectId
	 * @param projectManagerId
	 * @return
	 */
	public ResultObject getProjectManagerProjectDetail(Map parameters);
	/**
	 * 
	 * @param projectType
	 * @param orderNo
	 * @return
	 */
	public Map<String,Object> getOrderStatuse(Integer projectType,String orderNo);
	/**
	 * 查询订单详情
	 * @param orderNo
	 * @return
	 */
	public ResultObject selectOrderDetail(String orderNo);
	/**
	 * 审核项目交易订单
	 * @param parameters
	 * orderNo订单号
	 * orderStatus（0结束交易、2审批、3录到访、4录认购/意向、5录签约）
	 * picPath上传的图片
	 * description备注
	 * endCause拒绝原因
	 * customerName客户姓名
	 * mobile客户手机号
	 * identityCard身份证号
	 * shopNumber商铺编号
	 * price签约价格
	 * signTime签约日期
	 * acreage签约面积
	 * @return 
	 */
	public ResultObject approve(Map<String, Object> parameters);
	
	public List<Map> picManage(String orderNo);
	
	public int updatePic(String picPath,Integer id);
	
	public ResultObject orderStatusCount(Map parameters);
	
	/**
	 * 获取项目列表总数(分页使用)
	 * @param parameters
	 * @return
	 */
	public int getProjectListCount(Map<String, Object> parameters);
	
	/**
	 * 获取热门项目
	 * @param parameters
	 * @return
	 */
    List<Map<String,Object>> getRecommendProjectList(Map<String, Object> parameters);
    
	ResultObject findSignDetail(String orderNo);
	
	ResultObject findOrderEntity(String orderNo);
	ResultObject findManyUser(String orderNo);
	String jsonPriceU(String projectType, String priceType);
	
	public int updateDescrAndPic(String picPath,String description,String orderTrackId,String orderNo,String customerName,String mobile);
}