package com.up.sell.service.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alipay.api.AlipayApiException;
import com.google.common.collect.Maps;
import com.up.sell.mapper.MemberOrderMapper;
import com.up.sell.mapper.MemberServiceMapper;
import com.up.sell.pay.ali.AliPay;
import com.up.sell.service.AliPayService;
import com.up.sell.util.ArithUtils;
import com.up.sell.util.CodeGeneration;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.DateUtils;
import com.up.sell.vo.MemberOrder;
import com.up.sell.vo.MemberService;
import com.up.sell.vo.ResultObject;

@Service
public class AliPayServiceImpl implements AliPayService{
	

	@Autowired
	MemberServiceMapper memberServiceMapper;
	
	@Autowired
	MemberOrderMapper memberOrderMapper;
	
	public final static String notifyUrl = ConfigurationUtils.getStringValue("ALI_NOTIFY_URL");
	public final static String appID = ConfigurationUtils.getStringValue("ALI_APP_ID");
	public final static String privateKey = ConfigurationUtils.getStringValue("ALI_APP_PRIVATE_KEY");
	public final static String publicKey = ConfigurationUtils.getStringValue("ALI_PAY_PUBLIC_KEY");
	public final static String returnUrl = ConfigurationUtils.getStringValue("ALI_H5_RETURN_URL");

	@Override
	public ResultObject aliPay(Map<String, Object> parameters) throws AlipayApiException {
		ResultObject resultObject = new ResultObject();
		Map<String, Object> map = dealparameters(parameters);
		String orderNo = (String) map.get("orderNo");
		String subject = (String) map.get("subject");
		//String amount="0.01";
		//Double payAmount = Double.parseDouble(amount);
		Double payAmount = (Double) map.get("payAmount");
		String appPay = AliPay.appPay(orderNo, subject, null, payAmount, notifyUrl, appID, privateKey, publicKey);
		resultObject.put("data", appPay);
		return resultObject;
	}

	//1、创建订单
	public synchronized MemberOrder createOrder(Integer memberId,Integer orderSource,Integer id,Double payAmount) {
		MemberOrder mo = new MemberOrder();
		mo.setOrderNo(CodeGeneration.getOrderNo());//订单号
		mo.setMemberId(memberId);
		mo.setOrderSource(orderSource);
		mo.setOrderStatus(1);
		mo.setOrderType(1);
		mo.setPaymentType(2);//支付宝
		MemberService ms = memberServiceMapper.selectByPrimaryKey(id);
		mo.setGoodsId(id);
		mo.setViewCount(ms.getViewCount());
		mo.setCreateTime(new Date());
		mo.setPayAmount(payAmount);
		mo.setGoodsName(ms.getServiceName());
		memberOrderMapper.insertSelective(mo);
		return mo;
	}

	@Override
	public ResultObject h5Pay(Map<String, Object> parameters) throws AlipayApiException {
		ResultObject resultObject = new ResultObject();		
		Map<String, Object> map = dealparameters(parameters);
		String orderNo = (String) map.get("orderNo");
		String subject = (String) map.get("subject");
		//String amount="0.01";
		//Double payAmount = Double.parseDouble(amount);
		Double payAmount = (Double) map.get("payAmount");
		String h5Pay = AliPay.pageExecute(appID,privateKey,publicKey, orderNo,payAmount,subject,notifyUrl, returnUrl);
		resultObject.put("data", h5Pay);
		return resultObject;
	}
	
	/**
	 * 处理参数
	 * @param parameters
	 * @return
	 */
	public Map<String, Object> dealparameters(Map<String, Object> parameters) {
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		Integer orderSource = Integer.valueOf(parameters.get("orderSource").toString());
		String orderNo =StringUtils.EMPTY;
		String subject = StringUtils.EMPTY;
		Double sub =null;
		MemberOrder order =null;
		Map<String, Object> mapParams = Maps.newHashMap();
		if(Objects.nonNull(parameters.get("orderNo"))) {//已有订单
			mapParams.put("orderNo", parameters.get("orderNo").toString());
		}
		mapParams.put("memberId", memberId);
		mapParams.put("startTime", DateUtils.now("yyyy-MM-dd"));
		//查询当前会员等级
		MemberOrder memberOrder = memberOrderMapper.selectMemberOrder(mapParams);
		Integer id =null;
		if(Objects.nonNull(parameters.get("id"))) {
			id = Integer.valueOf(parameters.get("id").toString());
		}
		MemberService ms = memberServiceMapper.selectByPrimaryKey(id);//要购买的产品信息
		Map<String, Object> map = Maps.newHashMap();
		if(Objects.nonNull(parameters.get("orderNo"))) {//不为空,不是首次购买
			orderNo = parameters.get("orderNo").toString();
			order = memberOrderMapper.selectByPrimaryKey(orderNo);
		}else{//首次购买会员
		    order = createOrder(memberId,orderSource,id,ms.getPayAmount());
			orderNo = order.getOrderNo();
		}
		if(StringUtils.equals("普通会员", order.getGoodsName())) {
			subject="普通会员服务费";
		}else if(StringUtils.equals("vip会员", order.getGoodsName())) {
			subject="vip会员服务费";
		}else if(StringUtils.equals("白金会员", order.getGoodsName())) {
			subject="白金会员服务费";
		}
		sub =order.getPayAmount();
		if(Objects.nonNull(parameters.get("type"))) {
			if(StringUtils.equals("1", parameters.get("type").toString())) {//会员升级
				Integer goodsId =order.getGoodsId();
				MemberService member = memberServiceMapper.selectByPrimaryKey(goodsId);
				//升级等级金额 - 会员剩余天数 * （当前会员金额/相对应的时间）= 升级该付款的金额
				 int day = Integer.valueOf(DateUtils.getDatePoor(memberOrder.getEndTime(),new Date()));//会员剩余天数
				 Double payAmount = order.getPayAmount();//当前会员金额
				 Integer usefulLife = member.getUsefulLife();
				 Double life = parseDouble(usefulLife);//相对应的时间
				 Double div =null;
				 try {
					 div = ArithUtils.div(payAmount, life, 2);//（当前会员金额/相对应的时间）
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
				double mul = ArithUtils.mul(parseDouble(day), div);//会员剩余天数 * （当前会员金额/相对应的时间）
				Double payAmount2 = ms.getPayAmount();
				sub = ArithUtils.sub(payAmount2, mul);//升级等级金额 - 会员剩余天数 * （当前会员金额/相对应的时间）
				order = createOrder(memberId,orderSource,id,sub);
				orderNo = order.getOrderNo();
			}	
		}
		map.put("orderNo", orderNo);
		map.put("subject", subject);
		map.put("payAmount", sub);
		return map;
	}
	
	
	public Double parseDouble(int val) {
		BigDecimal b = new BigDecimal(val);
		Double d = b.doubleValue();
		return d;
	}

	@Override
	public ResultObject getPayAmount(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		String orderNo =StringUtils.EMPTY;
		String subject = StringUtils.EMPTY;
		Double sub =null;
		MemberOrder order =null;
		 MemberService ms = null; 
		 Integer id = null;
		if(Objects.nonNull(parameters.get("id"))) {
			 id= Integer.valueOf(parameters.get("id").toString());
			 ms = memberServiceMapper.selectByPrimaryKey(id);//要购买的产品信息
			 sub=ms.getPayAmount();
		}
		if(Objects.nonNull(parameters.get("orderNo"))) {//不为空,不是首次购买
			orderNo = parameters.get("orderNo").toString();
			order = memberOrderMapper.selectByPrimaryKey(orderNo);
			sub=order.getPayAmount();
		}
		if(Objects.nonNull(parameters.get("type"))) {
			if(StringUtils.equals("1", parameters.get("type").toString())) {//会员升级
				Map<String,Object> map =Maps.newHashMap();
				map.put("memberId", memberId);
				MemberOrder memberOrder = memberOrderMapper.selectMemberOrder(map);
				Integer goodsId =order.getGoodsId();
				MemberService member = memberServiceMapper.selectByPrimaryKey(goodsId);
				//升级等级金额 - 会员剩余天数 * （当前会员金额/相对应的时间）= 升级该付款的金额
				 int day = Integer.valueOf(DateUtils.getDatePoor(memberOrder.getEndTime(),new Date()));//会员剩余天数
				 Double payAmount = order.getPayAmount();//当前会员金额
				 Integer usefulLife = member.getUsefulLife();
				 Double life = parseDouble(usefulLife);//相对应的时间
				 Double div =null;
				 try {
					 div = ArithUtils.div(payAmount, life, 2);//（当前会员金额/相对应的时间）
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
				double mul = ArithUtils.mul(parseDouble(day), div);//会员剩余天数 * （当前会员金额/相对应的时间）
				Double payAmount2 = ms.getPayAmount();
				sub = ArithUtils.sub(payAmount2, mul);//升级等级金额 - 会员剩余天数 * （当前会员金额/相对应的时间）
//				order = createOrder(memberId,orderSource,id,sub);
//				orderNo = order.getOrderNo();
			}	
		}
		resultObject.put("orderNo", orderNo);
		resultObject.put("subject", subject);  
		resultObject.put("payAmount", sub);
		return resultObject;
	}
}
