/**
 * 
 */
package com.up.sell.service.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Maps;
import com.up.sell.annotation.Cacheable;
import com.up.sell.mapper.BrandBusinessMapper;
import com.up.sell.mapper.BrandContactsAreaMapper;
import com.up.sell.mapper.BrandContactsErrorAreaMapper;
import com.up.sell.mapper.BrandContactsErrorMapper;
import com.up.sell.mapper.BrandContactsMapper;
import com.up.sell.mapper.BrandDatabaseCityMapper;
import com.up.sell.mapper.BrandDatabaseMapper;
import com.up.sell.mapper.BrandExpandAreaMapper;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.mapper.MemberOrderMapper;
import com.up.sell.mapper.MemberServiceUseMapper;
import com.up.sell.service.BrandService;
import com.up.sell.util.Constant;
import com.up.sell.util.DateUtils;
import com.up.sell.util.ResultException;
import com.up.sell.vo.BrandBusiness;
import com.up.sell.vo.BrandContacts;
import com.up.sell.vo.BrandContactsArea;
import com.up.sell.vo.BrandContactsError;
import com.up.sell.vo.BrandContactsErrorArea;
import com.up.sell.vo.BrandDatabase;
import com.up.sell.vo.BrandDatabaseCity;
import com.up.sell.vo.BrandExpandArea;
import com.up.sell.vo.MemberOrder;
import com.up.sell.vo.MemberServiceUse;
import com.up.sell.vo.ResultObject;

/**
 * @author zmg
 *
 */
@Service
@Transactional(propagation=Propagation.SUPPORTS)
public class BrandServiceImpl implements BrandService {
	
	@Autowired
	private BrandBusinessMapper brandBusinessMapper;
	@Autowired
	private BrandDatabaseMapper brandDatabaseMapper;
	@Autowired
	private BrandContactsMapper brandContactsMapper;
	@Autowired
	private BrandContactsAreaMapper brandContactsAreaMapper;
	@Autowired
	private BrandContactsErrorMapper brandContactsErrorMapper;
	@Autowired
	private BrandContactsErrorAreaMapper brandContactsErrorAreaMapper;
	@Autowired
	private BrandExpandAreaMapper brandExpandAreaMapper;
	@Autowired
	private BrandDatabaseCityMapper brandDatabaseCityMapper;
	@Autowired
	private MemberMapper memberMapper;
	@Autowired
	private MemberServiceUseMapper memberServiceUseMapper;
	@Autowired
	private MemberOrderMapper memberOrderMapper;
	/* (non-Javadoc)
	 * @see com.up.sell.service.impl.BrandService#getBrandBusinessList()
	 */
	@Override
	@Cacheable(expireTime=60*60*24)
	public List<Map> getBrandBusinessList (boolean isAll) {
		List<Map> levelOneMapList = new ArrayList<Map>();
		
		List<BrandBusiness> levelOneList = this.brandBusinessMapper.selectLevelOne();
		List<BrandBusiness> levelTwoList = this.brandBusinessMapper.selectLevelTwo();
		for(BrandBusiness levelOne : levelOneList) {
			Map levelOneMap = new HashMap();
			levelOneMap.put("id", levelOne.getId());
			levelOneMap.put("businessName", levelOne.getBusinessName());
			for(int i=0;i<levelTwoList.size();i++) {
				BrandBusiness levelTwo = levelTwoList.get(i);
				if(levelOne.getId() == levelTwo.getParentId()) {
					Map levelTwoMap = new HashMap();
					levelTwoMap.put("id", levelTwo.getId());
					levelTwoMap.put("businessName", levelTwo.getBusinessName());
					levelTwoMap.put("p_id", levelOne.getId());
					List<Map> levelTwoMapList = (List<Map>) levelOneMap.get("levelTwoList");
					if(levelTwoMapList == null) {
						levelTwoMapList = new ArrayList<Map>();
						if(isAll) {
							Map brandBusiness2 = new HashMap<>();
							brandBusiness2.put("businessName", "全部");
							brandBusiness2.put("id", "");
							brandBusiness2.put("p_id", levelOne.getId());
							levelTwoMapList.add(brandBusiness2);
						}
					}
					levelTwoMapList.add(levelTwoMap);
					levelOneMap.put("levelTwoList", levelTwoMapList);
					levelTwoList.remove(i);
					i--;
				}
			}
			levelOneMapList.add(levelOneMap);
		}
		return levelOneMapList;
	}
	/**
	 * 查询品牌列表
	 * @param parameters
	 * 	cityId、brandName、areasId、regionsId、provinceId、brandBusiness
	 */
	public List<Map> selectBrandList(Map parameters) {
		return this.brandDatabaseMapper.selectBrandList(parameters);
	}
	/**
	 * 查询置顶品牌列表
	 * @param parameters
	 * @return
	 */
	public List<Map> selectBrandTopList(Map parameters) {
		return this.brandDatabaseMapper.selectBrandTopList(parameters);
	}
	/**
	 * 查询品牌详情
	 * @param id
	 * @return
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public Map selectBrandDetail(Integer id) {
		this.brandDatabaseMapper.updateViewCount(id);
		return this.brandDatabaseMapper.selectBrandDetail(id);
	}
	/**
	 * 查询品牌联系人
	 * @param id
	 * @return
	 */
	public List<Map> selectBrandContacts(Integer id) {
		return this.brandContactsMapper.selectByBrandDatabaseId(id);
	}
	/**
	 * 我要成为品牌联系人
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public void createContact(BrandContacts brandContacts) {
		brandContacts.setCreateTime(new Date());
		brandContacts.setFlag(1);
		this.brandContactsMapper.insertSelective(brandContacts);
		for(BrandContactsArea brandContactsArea : brandContacts.getBrandContactsAreaList()) {
			brandContactsArea.setBrandContactsId(brandContacts.getId());
			this.brandContactsAreaMapper.insertSelective(brandContactsArea);
		}
	}
	/**
	 * 品牌联系人纠错
	 * @param brandContactsError
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public void createContactError(BrandContactsError brandContactsError) {
		brandContactsError.setCreateTime(new Date());
		brandContactsError.setFlag(1);
		this.brandContactsErrorMapper.insertSelective(brandContactsError);
		for(BrandContactsErrorArea brandContactsErrorArea : brandContactsError.getBrandContactsErrorAreaList()) {
			brandContactsErrorArea.setBrandContactsErrorId(brandContactsError.getId());
			this.brandContactsErrorAreaMapper.insertSelective(brandContactsErrorArea);
		}
	}

	/**
	 * 发布品牌
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public void saveBrandDatabase(BrandDatabase brandDatabase) {
		//前端不好操作，这里兼容前端，全选传3，数据库里保存1,2
		if (brandDatabase.getBrandNature().equals("3")){
			brandDatabase.setBrandNature("1,2");
		}else{
			brandDatabase.setBrandNature(brandDatabase.getBrandNature());
		}
		brandDatabase.setCreateTime(new Date());
		brandDatabase.setFlag(1);
		brandDatabase.setBrandStatus(1);
		brandDatabase.setSourceType(1);
		this.brandDatabaseMapper.insertSelective(brandDatabase);
		BrandDatabaseCity brandDatabaseCity = new BrandDatabaseCity();
		brandDatabaseCity.setBrandDatabaseId(brandDatabase.getId());
		brandDatabaseCity.setCityId(brandDatabase.getCityId());
		brandDatabaseCity.setCreateTime(new Date());
		brandDatabaseCity.setProvinceId(brandDatabase.getProvinceId());
		brandDatabaseCity.setType("1");
		this.brandDatabaseCityMapper.insertSelective(brandDatabaseCity);
		for(BrandContacts brandContacts : brandDatabase.getBrandContactsList()) {
			brandContacts.setBrandDatabaseId(brandDatabase.getId());
			brandContacts.setCreateTime(new Date());
			brandContacts.setFlag(1);
			brandContacts.setMemberId(brandDatabase.getMemberId());
			this.brandContactsMapper.insertSelective(brandContacts);
			for(BrandContactsArea brandContactsArea : brandContacts.getBrandContactsAreaList()) {
				brandContactsArea.setBrandContactsId(brandContacts.getId());
				this.brandContactsAreaMapper.insertSelective(brandContactsArea);
			}
		}
		for(BrandExpandArea brandExpandArea : brandDatabase.getBrandExpandAreaList()) {
			brandExpandArea.setBrandDatabaseId(brandDatabase.getId());
			this.brandExpandAreaMapper.insertSelective(brandExpandArea);
		}
	}
	
	/**
	 * 查询品牌联系人的联系电话
	 * @return
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public ResultObject getBrandContact(Integer brandContactsId,Integer memberId) {
		ResultObject resultObject = new ResultObject();
		resultObject.put("mobile", "");
		Map<String,Object> map = Maps.newHashMap();
		map.put("memberId", memberId);
		MemberOrder selectMemberOrder = memberOrderMapper.selectMemberOrder(map); 
		if(Objects.isNull(selectMemberOrder)) {
			resultObject.put("memberLevel", 0);
		}else {
			resultObject.put("memberLevel", selectMemberOrder.getGoodsId());
			String endTime = selectMemberOrder.getEndTime().toString();
		  	SimpleDateFormat sfEnd = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	      	SimpleDateFormat sfStart = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy",java.util.Locale.ENGLISH) ;
	     	try {
				endTime = sfEnd.format(sfStart.parse(endTime));
			} catch (ParseException e1) {
				e1.printStackTrace();
			}
			Date format = DateUtils.format(endTime, "yyyy-MM-dd");
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			String format1 = sdf.format(new Date());
			Date nowDate = null;
		    try {
		    	nowDate=sdf.parse(format1);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			int datePoor = DateUtils.getDatePoor(format,nowDate);
			if(datePoor<0) {
				resultObject.put("memberLevel", 4);//购买的会员已过期
				//member.setMemberLevel("4");
			}
		}
		String mobile = this.brandContactsMapper.selectMobileFromHistory(memberId, brandContactsId);
		if(StringUtils.isBlank(mobile)) {
			int row = this.memberMapper.updateViewCountByPrimaryKey(memberId);
			if(row>0) {
				BrandContacts brandContacts = this.brandContactsMapper.selectByPrimaryKey(brandContactsId);
				if(brandContacts!=null) {
					MemberServiceUse memberServiceUse = new MemberServiceUse();
					memberServiceUse.setBrandContactsId(brandContactsId);
					memberServiceUse.setCreateTime(new Date());
					memberServiceUse.setMemberId(memberId);
					this.memberServiceUseMapper.insertSelective(memberServiceUse);
					resultObject.put("mobile", brandContacts.getMobile());
					resultObject.put("flag", "1");
				} else {
					throw new ResultException(Constant.CODE_EXCEPTION, "联系人不存在");
				}
			} else {
				resultObject.setCode(Constant.CODE_SUCCESS);
				resultObject.put("flag", "0");
				resultObject.setMessage("今日已达查看次数上限，请明日再查");
			}
		} else {
			resultObject.put("flag", "1");
			resultObject.put("mobile", mobile);
		}
		return resultObject;
	}
	@Override
	public ResultObject valideBrandName(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		int count = brandDatabaseMapper.valideBrandName(parameters);
		if(count!=0) {
			resultObject.put("status", false);
		}else {
			resultObject.put("status", true);
		}
		return resultObject;
	}
	
	 public static Date parse(String str, String pattern, Locale locale) { 
	        if(str == null || pattern == null) { 
	            return null; 
	        } 
	        try { 
	            return new SimpleDateFormat(pattern, locale).parse(str); 
	        } catch (ParseException e) { 
	            e.printStackTrace(); 
	        } 
	        return null; 
	    } 
	 
	    public static String format(Date date, String pattern, Locale locale) { 
	        if(date == null || pattern == null) { 
	            return null; 
	        } 
	        return new SimpleDateFormat(pattern, locale).format(date); 
	    } 
	    public static void main(String[] args) throws ParseException {
	    	String dateString  = "Tue Apr 15 11:23:55 CST 2014";
	    	SimpleDateFormat sfEnd = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
	    	SimpleDateFormat sfStart = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy",java.util.Locale.ENGLISH) ;
	    	System.out.println(sfEnd.format(sfStart.parse(dateString)));
		}
		@Override
		public List<Map> getHotBrandList(Map<String, Object> parameters) {
			// TODO Auto-generated method stub
			return brandDatabaseMapper.getHotBrandList(parameters);
		}
		@Override
		public List<Map<String, Object>> getBrandRecommended(Map<String, Object> parameters) {
			// TODO Auto-generated method stub
			return brandDatabaseMapper.getBrandRecommended(parameters);
		}
		
		@Override
		public ResultObject getContactError(Integer brandContactsId, Integer memberId) {
			Boolean b =false;//默认没有查看过
			//1.查询当前用户是否查看过品牌
			ResultObject resultObject = new ResultObject();
			String mobile = this.brandContactsMapper.selectMobileFromHistory(memberId, brandContactsId);
			if(StringUtils.isBlank(mobile)) {
				resultObject.put("flag", b);//没有查看 不回显值
				resultObject.put("post","");//职位为空字符串
				resultObject.put("post_text","");//职位为空字符串
				resultObject.put("brandContactsArea","");//区域为空字符串
				resultObject.put("brandContactsArea_text","");//区域为空字符串
				resultObject.put("mobile","");//手机号为空字符串
				resultObject.put("user_name","");//手机号为空字符串
			}else {
				b=true;
				resultObject.put("flag", b);//没有查看 不回显值
				Map<String, Object> map = this.brandContactsMapper.selectBrandContacts(brandContactsId);
				if(Objects.nonNull(map)) {
					resultObject.put("post",map.get("post"));//职位
					resultObject.put("brandContactsArea",map.get("brandContactsArea"));//区域
					resultObject.put("mobile",map.get("mobile"));//区域
					resultObject.put("post_text",map.get("post_text"));//职位为空字符串
					resultObject.put("brandContactsArea_text",map.get("brandContactsArea_text"));//区域为空字符串
					resultObject.put("user_name",map.get("user_name"));//手机号为空字符串
				}else {
					resultObject.put("post","");//职位为空字符串
					resultObject.put("post_text","");//职位为空字符串
					resultObject.put("brandContactsArea","");//区域为空字符串
					resultObject.put("brandContactsArea_text","");//区域为空字符串
					resultObject.put("mobile","");//手机号为空字符串
					resultObject.put("user_name","");//手机号为空字符串
				}
			}
			return resultObject;
		}
	
} 
