package com.up.sell.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.up.sell.mapper.BrowseLogMapper;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.service.BrowseLogService;
import com.up.sell.util.Constant;
import com.up.sell.vo.BrowseLog;
import com.up.sell.vo.Member;
import com.up.sell.vo.ResultObject;


@Service
@Transactional(propagation=Propagation.SUPPORTS)
public class BrowseLogServiceImpl implements BrowseLogService {
	private static final Logger logger = LoggerFactory.getLogger(BrowseLogServiceImpl.class);
	
	@Autowired
	private BrowseLogMapper browseLogMapper;
	@Autowired
	private MemberMapper memberMapper;

	@Override
	public ResultObject selectPage(Map<String, Object> paraMap) {
		ResultObject resultObject = new ResultObject();
		try {
			Integer pageNum = Integer.valueOf((String)paraMap.get("pageNum"));
			Integer pageSize = Integer.valueOf((String)paraMap.get("pageSize"));
			Page<Object> startPage = PageHelper.startPage(pageNum, pageSize, true);
			List<BrowseLog> list= this.browseLogMapper.findList(paraMap);
			resultObject.put("list", list);
			resultObject.put("totalCount", startPage.getTotal());
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	
	
	@Override
	public ResultObject updateEntity(Map<String,Object> map){
		ResultObject resultObject = new ResultObject();
		try {
			Integer userId=Integer.valueOf(map.get("userId")+"") ;
			BrowseLog entity=this.browseLogMapper.findEntityByMap(map);
			if(entity==null){
				entity=new BrowseLog();
				entity.setNumber(1);
				entity.setUserId(userId);
				Member member=this.memberMapper.selectByPrimaryKey(userId);
				entity.setUserName(member==null?"张三":member.getUserName());
				entity.setPhone(member==null?"15210000000":member.getMobile());
				entity.setProjectId(map.get("projectId")+"");
				this.browseLogMapper.saveEntity(entity);
			}else{
				entity.setNumber(entity.getNumber()+1);
				entity.setUpdateTime(new Date());
				this.browseLogMapper.updateEntity(entity);
			}
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	
}
