package com.up.sell.service.impl;

import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.up.sell.fourhundred.FourCallClient;
import com.up.sell.mapper.FourHundredMapper;
import com.up.sell.service.FourHundredService;
import com.up.sell.vo.fourhundred.DeptDetail;
import com.up.sell.vo.fourhundred.FourHundred;
import com.up.sell.vo.fourhundred.IvrKeyShow;
import com.up.sell.vo.fourhundred.ResultFour;
import com.up.sell.vo.fourhundred.UserA;


/**
 * 商客通400小号对接业务逻辑类
 * @author PJJ
 *
 */
@Service
@Transactional(propagation=Propagation.SUPPORTS)
public class FourHundredServiceImpl implements FourHundredService {
	
	private static final Logger logger = LoggerFactory.getLogger(FourHundredServiceImpl.class);
	@Autowired
	private FourHundredMapper fourHundredMapper;
	
	//绑定某一个手机号
	@Override
	public Boolean bangDingOne(String linkPhone){
		FourCallClient fc=new FourCallClient();
		String ivrKey = this.fourHundredMapper.findIvrKeyMax();//导航键自定义6位数字;卖铺宝从30001开始
		UserA userA=fc.findUserEntityByPhone(linkPhone);
		if(userA !=null && userA.getUserName().contains("mls")){
			//【mls】
			//1)创建导航部门
			ResultFour resultF=fc.saveDept(ivrKey);
			if("false".equals(resultF.getResult()) || StringUtils.isEmpty(resultF.getDeptId())){
				return false;
			}
			//2）绑定坐席。
			String agentId= fc.getAgentIdByUserId(userA.getUserId());
			if(StringUtils.isEmpty(agentId)){
				logger.info("【根据用户ID获取坐席ID失败！】");
				return false;
			}
			Boolean flag=fc.saveSelectedAgentIds(resultF.getDeptId(),agentId);//这个手机号已经绑定了mls桌席，直接拿过来挂上
			logger.info("【将电话（"+linkPhone+"）绑定到（"+FourCallClient.deptPre+"）的结果是：（"+flag+"）】");
			if(flag){
				this.insertFourInfo(resultF.getDeptId(), resultF.getDeptName(), ivrKey, linkPhone, resultF);
			}
			return flag;
		}
		if(userA !=null && userA.getUserName().contains(FourCallClient.deptPre)){
			//【租铺宝】
			fc.delUserByUserId(userA.getUserId());//已经绑定了，先删掉
		}
		return this.toCreate(ivrKey.equals("0")?"100001":ivrKey, linkPhone);
	}
	
	
	//创建导航+创建坐席
	public Boolean toCreate(String ivrKey,String linkPhone){
		Boolean flag=true;
		FourCallClient fc=new FourCallClient();
		try {
			ResultFour resultF=fc.saveDept(ivrKey);
			if("false".equals(resultF.getResult()) || StringUtils.isEmpty(resultF.getDeptId())){
				return false;
			}
			String deptId=resultF.getDeptId();
			String deptName=resultF.getDeptName();
			resultF=fc.saveUserAgentBindIvr(linkPhone,deptId);
			if("false".equals(resultF.getResult())){
				return false;
			}
			this.insertFourInfo(deptId, deptName, ivrKey, linkPhone, resultF);
			logger.info("=========【成功绑定导航键（" + ivrKey + "）到手机号（" + linkPhone + "）】");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return flag;
	}
	
	//保存商客通400导航按键绑定手机号关系
	public void insertFourInfo(String deptId,String deptName,String ivrKey,String linkPhone,
			ResultFour resultF){
		FourHundred  fhEn=this.fourHundredMapper.findEntityByLinkPhone(linkPhone);
		if(fhEn !=null){
			this.fourHundredMapper.deleteByLinkPhone(linkPhone);
		}
		FourHundred  entity=new FourHundred();
		entity.setDeptId(deptId);
		entity.setDeptName(deptName);
		entity.setIvrKey(ivrKey);
		entity.setLinkPhone(linkPhone);
		entity.setPlatform("1");
		entity.setSktAgentId(resultF.getAgentId()==null?"-1":resultF.getAgentId());
		entity.setFourPhone(resultF.getFourPhone()==null?"-1":resultF.getFourPhone());
		entity.setGongHao(resultF.getGongHao()==null?"-1":resultF.getGongHao());
		entity.setGongHaoName(resultF.getGongHaoName()==null?"-1":resultF.getGongHaoName());
		entity.setGongHaoPwd(resultF.getGongHaoPwd()==null?"-1":resultF.getGongHaoPwd());
		entity.setCreateTime(new Date());
		this.fourHundredMapper.saveFourHundred(entity);
	}
	
	
	
	
	
	/**
	 * 查找外显商客通导航按键
	 * @param linkPhone
	 * @return
	 */
	public IvrKeyShow findIvrKeyShow(String linkPhone){
		IvrKeyShow show=new IvrKeyShow();
		show.setIvrType("1");
		show.setIvrPhone(FourCallClient.guPhone);
		show.setIvrAll(FourCallClient.guPhone);
		FourHundred  fh=this.fourHundredMapper.findEntityByLinkPhone(linkPhone);
		if(fh !=null && StringUtils.isNotEmpty(fh.getIvrKey())){
			String ivrAll=fh.getFourPhone()+"转"+fh.getIvrKey();
			show.setIvrType("2");
			show.setIvrPhone(fh.getFourPhone());
			show.setIvrKey(fh.getIvrKey());
			show.setIvrAll(ivrAll);
			return show;
		}
		return show;
	}
	
	/*public static void main(String[] args) {
		FourHundredServiceImpl fh=new FourHundredServiceImpl();
		fh.toCreate("1", "01059002466");
	}*/
	
	
	
}
