package com.up.sell.service.impl;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.up.sell.mapper.DictionaryMapper;
import com.up.sell.mapper.MemberAccountBillMapper;
import com.up.sell.mapper.MemberBankMapper;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.redis.RedisBaseDao;
import com.up.sell.service.MemberBankService;
import com.up.sell.util.ArithUtils;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.Constant;
import com.up.sell.vo.Member;
import com.up.sell.vo.MemberAccountBill;
import com.up.sell.vo.MemberBank;
import com.up.sell.vo.ResultObject;

@Service
public class MemberBankServiceImpl implements MemberBankService{
	
	private static final Logger logger = LoggerFactory.getLogger(MemberBankServiceImpl.class);
	public final static String bank_img = ConfigurationUtils.getStringValue("bank_img");
	
	@Autowired
	MemberBankMapper memberBankMapper;
	
	@Autowired
	RedisBaseDao redisBaseDao;
	
	@Autowired
	MemberAccountBillMapper memberAccountBillMapper;
	
	@Autowired
	DictionaryMapper dictionaryMapper;
	
	@Autowired
	MemberMapper memberMapper;

	@Transactional(propagation=Propagation.REQUIRED)
	@Override
	public ResultObject bindBankCard(Map<String,Object> map) {
		ResultObject resultObject = new ResultObject();
		Integer memberId = Integer.valueOf(map.get("loginIdentity").toString());
		int cardCount = memberBankMapper.selectCount(memberId);
		Member m = memberMapper.selectByPrimaryKey(memberId);
		String mobile = (String) map.get("mobile");
		String validCode = (String) map.get("validCode");
		resultObject.put("account",m.getAccount());
		resultObject.put("memberType", m.getMemberType());
		//是否首次绑卡标识 0 首次 
		if(cardCount==0 && StringUtils.isBlank(m.getIdCardNo())) {
			resultObject.put("flag", 0);
			return resultObject;
		}else if(cardCount>=5) {
			resultObject.put("flag", 3);
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("亲，您绑定的银行卡已达到上限，如需更改银行卡，请先解绑银行卡再绑定其他银行卡!");
			return resultObject;
		}else if(cardCount==0 && StringUtils.isNotBlank(m.getIdCardNo()) && StringUtils.isBlank(mobile)) {
			resultObject.put("flag", 1);
			return resultObject;
		}
		resultObject.put("flag", 2);
		if(StringUtils.isNotBlank(mobile)) {
			//验证验证码
			Object object = redisBaseDao.get(Constant.CERTIFICATION_VALIDCODE_KEY+mobile);
			//判空
			if(Objects.isNull(object)) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("验证码过期");
				return resultObject;
			}
			String code =String.valueOf(object);
			if(StringUtils.equals(validCode, code)) {
				 MemberBank mb = new MemberBank();
				 mb.setAccountBank(map.get("accountBank").toString());//开户行名称
				 mb.setAccountName(map.get("accountName").toString());//账户名称
				 mb.setCardNo(map.get("cardNo").toString());//银行账号
				 mb.setBankId(Integer.valueOf(map.get("bankId").toString()));//银行id
				 mb.setMemberId(memberId);
				 mb.setCreateTime(new Date());
				 memberBankMapper.insertSelective(mb);
				 resultObject.setMessage("提交成功");
				 this.redisBaseDao.del(Constant.CERTIFICATION_VALIDCODE_KEY + mobile);
			 }else {
				Long count = this.redisBaseDao.getIncr(Constant.CERTIFICATION_VALIDCODE_KEY_ERROR + mobile);
				if (count == 5) {
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("验证码5次输入错误，请重新获取。");
					this.redisBaseDao.del(Constant.CERTIFICATION_VALIDCODE_KEY + mobile);
					this.redisBaseDao.del(Constant.CERTIFICATION_VALIDCODE_KEY_ERROR + mobile);
				} else {
					this.redisBaseDao.incr(Constant.CERTIFICATION_VALIDCODE_KEY_ERROR + mobile);
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("验证码输入错误，请确认后输入。");
				}
		}
			return resultObject;
		}
		 return resultObject;
	}

	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public List<Map<String, Object>> bindBankCardList(Map<String, Object> parameters) {
		parameters.put("img", bank_img);
		return memberBankMapper.bindBankCardList(parameters);
	}

	@Transactional(propagation=Propagation.REQUIRED)
	@Override
	public ResultObject unbindBankCard(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
//		String mobile = (String) parameters.get("mobile");
//		String validCode = (String) parameters.get("validCode");
		//验证验证码
//		Object object = redisBaseDao.get(Constant.CERTIFICATION_VALIDCODE_KEY+mobile);
		//判空
//		if(Objects.isNull(object)) {
//			resultObject.setCode(Constant.CODE_EXCEPTION);
//			resultObject.setMessage("验证码过期");
//			return resultObject;
//		}
		//String code =String.valueOf(object);
		//if(StringUtils.equals(validCode, code)) {
		try {
			String[] ids = parameters.get("id").toString().split(",");
			//循环删除
			for(int i=0;i<ids.length;i++) {
				memberBankMapper.deleteByPrimaryKey(Integer.valueOf(ids[i]));
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error(e.getMessage(), e);
			resultObject.setMessage("解绑失败!");
			return resultObject;
		}
		resultObject.setMessage("解绑成功!"); 
//		this.redisBaseDao.del(Constant.CERTIFICATION_VALIDCODE_KEY + mobile);
//		}else {
//			Long count = this.redisBaseDao.getIncr(Constant.CERTIFICATION_VALIDCODE_KEY_ERROR + mobile);
//			if (count == 5) {
//				resultObject.setCode(Constant.CODE_EXCEPTION);
//				resultObject.setMessage("验证码5次输入错误，请重新获取。");
//				this.redisBaseDao.del(Constant.CERTIFICATION_VALIDCODE_KEY + mobile);
//				this.redisBaseDao.del(Constant.CERTIFICATION_VALIDCODE_KEY_ERROR + mobile);
//			} else {
//				this.redisBaseDao.incr(Constant.CERTIFICATION_VALIDCODE_KEY_ERROR + mobile);
//				resultObject.setCode(Constant.CODE_EXCEPTION);
//				resultObject.setMessage("验证码输入错误，请确认后输入。");
//			}
//			return resultObject;
//		}
		return resultObject;
	}

	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public ResultObject getEncashmentList(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		parameters.put("img", bank_img);
		resultObject.put("bankList", memberBankMapper.bindBankCardList(parameters));
		List<Map<String, Object>> selectMapByParentKey = dictionaryMapper.selectMapByParentKey("100");
		List<Map<String, Object>> encashmentList = memberAccountBillMapper.encashmentList(parameters);
		List<Map<String, Object>> list = new ArrayList<>();
		for(int i=0;i<encashmentList.size();i++) {
			Map<String, Object> map = encashmentList.get(i);
			for(int j=0;j<selectMapByParentKey.size();j++) {
				Map<String, Object> map2 = selectMapByParentKey.get(j);
				map.put("rate", map2.get("d_value"));
			}
			map.put("isSelected", false);
			list.add(map);
		}
		resultObject.put("encashmentList",list);
		return resultObject;
	}

	@Transactional(propagation=Propagation.REQUIRED)
	@Override
	public ResultObject encashment(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		String[] ids = parameters.get("id").toString().split(",");
		List<Double> doubleList = new ArrayList<>();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
			for(int i=0;i<ids.length;i++) {
				MemberAccountBill memberAccountBill = new MemberAccountBill();
				String id = ids[i];
				MemberAccountBill mab = null;
				if(id.contains("-")) {//提供发票
					String[] str = id.split("-");
					id = str[0];
					String invoice = str[1];
					mab = memberAccountBillMapper.selectByPrimaryKey(Integer.valueOf(id));
					if(StringUtils.equals("false", invoice)) {
						memberAccountBill.setProvideInvoice(0);
						List<Map<String, Object>> list = dictionaryMapper.selectMapByParentKey("100");
						String rate = list.get(0).get("d_value").toString();
						Double rateValue= Double.valueOf(rate);
						Double approveAmount = multi(mab.getAmount(), rateValue);
					    memberAccountBill.setAmount(-subtract(mab.getAmount(),approveAmount));//实际金额
					    memberAccountBill.setApproveAmount(-mab.getApproveAmount());//申请金额
					    doubleList.add(mab.getApproveAmount());
					}else {
						mab = memberAccountBillMapper.selectByPrimaryKey(Integer.valueOf(id));
						doubleList.add(mab.getApproveAmount());
					    memberAccountBill.setProvideInvoice(1);
						memberAccountBill.setAmount(-mab.getAmount());//实际金额
						memberAccountBill.setApproveAmount(-mab.getApproveAmount());//申请金额
					}
				}
				MemberBank mb = memberBankMapper.selectByPrimaryKey(Integer.valueOf(parameters.get("bankId").toString()));
				if(Objects.nonNull(mb)) {
					memberAccountBill.setAccountBank(mb.getAccountBank());
					memberAccountBill.setAccountName(mb.getAccountName());
					memberAccountBill.setCardNo(mb.getCardNo());
				}
				memberAccountBill.setProjectName(mab.getProjectName());
				if(1==mab.getBillType()) {
					memberAccountBill.setBillType(4);
				}else if(2==mab.getBillType()) {
					memberAccountBill.setBillType(5);
				}else if(3==mab.getBillType()) {
					memberAccountBill.setBillType(6);
				}
				memberAccountBill.setBillStatus(1);//处理中
				memberAccountBill.setCreateTime(new Date());
				memberAccountBill.setProjectId(mab.getProjectId());
				memberAccountBill.setMemberId(memberId);
				memberAccountBill.setOrderNo(mab.getOrderNo());
				memberAccountBillMapper.insertSelective(memberAccountBill);
				mab.setFlag("1");
				memberAccountBillMapper.updateByPrimaryKey(mab);
			}
			Member m = memberMapper.selectByPrimaryKey(memberId);
			double add = ArithUtils.add(doubleList);
			m.setId(memberId);
			m.setAccount(ArithUtils.sub(m.getAccount(), add));
			memberMapper.updateByPrimaryKey(m);
		resultObject.setMessage("提现申请已提交，请耐心等待");
		return resultObject;
	}


   public Double multi(Double a, Double b) {
	  BigDecimal a1 = new BigDecimal(Double.toString(a));
	  BigDecimal b1 = new BigDecimal(Double.toString(b)); 
	  BigDecimal result = a1.multiply(b1);
	  return result.doubleValue();
	}

  
   private double subtract(double v1,double v2){
       BigDecimal b1 = new BigDecimal(Double.toString(v1));  
       BigDecimal b2 = new BigDecimal(Double.toString(v2));  
       return b1.subtract(b2).doubleValue();  
   }

	@Override
	public ResultObject validateCardNo(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		if(Objects.isNull(parameters.get("cardNo"))) {
			resultObject.setCode(Constant.CODE_PARAM_NULL);
			resultObject.setMessage("参数为空");
			return resultObject;
		}
		int count = memberBankMapper.validateCardNo(parameters);
		if(0==count) {
		   resultObject.put("status", true);
		}else {
		   resultObject.put("status", false);
		}
		return resultObject;
	}

	@Override
	public ResultObject selectBillDetail(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		List<Map<String, Object>> selectBillDetail = memberAccountBillMapper.selectBillDetail(parameters);
		List<Map<String, Object>> list = new ArrayList<>();
		if(Objects.nonNull(selectBillDetail) && selectBillDetail.size()>0 ){
			  for(int i=0;i<selectBillDetail.size();i++){
				  Map<String, Object> map = selectBillDetail.get(i);
				  if(Objects.nonNull(map)) {
					  String billType = map.get("bill_type").toString();
						if(StringUtils.equals("2", billType) || StringUtils.equals("3", billType)) {
							String user_name = map.get("user_name").toString();
							String project = map.get("project_name").toString();
							map.put("title","["+user_name+"成交"+project+"项目]");
						}else if(StringUtils.equals("1", billType)) {
							String project = map.get("project_name").toString();
							map.put("title","["+project+"]");
						}else {
							map.put("title","");
						}
						if(StringUtils.equals("5", billType) || StringUtils.equals("6", billType)) {
							map.put("bill_type", "4");
						}
						DecimalFormat df   = new DecimalFormat("######0.00"); 
						String amount = map.get("amount").toString();
						Double d = Double.valueOf(amount);
						amount = df.format(d);
						map.put("amount",amount);
						if(StringUtils.equals("1", billType) || StringUtils.equals("2", billType) || StringUtils.equals("3", billType)) {
							String str = map.get("amount").toString();
							map.put("amount", "+"+str);
						}
				  }
				  list.add(map);
			    }
		}
		if(Objects.nonNull(list) && list.size()!=0) {
			selectBillDetail=list;
		}
		resultObject.put("detail", selectBillDetail);
		resultObject.put("income",memberAccountBillMapper.getTotalIncome(parameters));
		int expenditure= 0;
		String totalExpenditure =String.valueOf(memberAccountBillMapper.getTotalExpenditure(parameters)) ;
		if(!StringUtils.equals("0", totalExpenditure)) {
			totalExpenditure =totalExpenditure.substring(1, totalExpenditure.length());
			expenditure=Integer.valueOf(totalExpenditure);
		}
		resultObject.put("expenditure",expenditure);
		return resultObject;
	}

}
