/**
 * 
 */
package com.up.sell.service.impl;

import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.up.sell.mapper.MemberCustomerMapper;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.mapper.ProjectOrderMapper;
import com.up.sell.mapper.TeamMapper;
import com.up.sell.mapper.TeamMembersMapper;
import com.up.sell.service.MemberCustomerService;
import com.up.sell.service.ProjectService;
import com.up.sell.util.Constant;
import com.up.sell.util.ResultException;
import com.up.sell.vo.Member;
import com.up.sell.vo.MemberCustomer;
import com.up.sell.vo.ResultObject;
import com.up.sell.vo.TeamMembers;

/**
 * @author zmg
 *
 */
@Service
@Transactional(propagation=Propagation.SUPPORTS)
public class MemberCustomerServiceImpl implements MemberCustomerService {
	private static final Logger logger = LoggerFactory.getLogger(MemberCustomerServiceImpl.class);
	
	@Autowired
	private MemberCustomerMapper memberCustomerMapper;
	@Autowired
	private ProjectOrderMapper projectOrderMapper;
	@Autowired
	private ProjectService projectService;
	@Autowired
	MemberMapper memberMapper;
	@Autowired
	TeamMapper teamMapper;
	@Autowired
	TeamMembersMapper teamMembersMapper;
	
	/* (non-Javadoc)
	 * @see com.up.sell.service.impl.transaction#getMyCustomers(java.lang.Integer)
	 */
	@Override
	public ResultObject getMyCustomers(Integer memberId,String customerName) {
		ResultObject resultObject = new ResultObject();
		resultObject.put("customerList", this.memberCustomerMapper.getMyCustomers(memberId,customerName));
		resultObject.put("customer_num", this.memberCustomerMapper.countMyCustomers(memberId));
		Map transaction = this.memberCustomerMapper.getMyTransaction(memberId);

//		Member m = memberMapper.selectByPrimaryKey(memberId);
//		int count = teamMapper.isTeamLeader(memberId);
//		TeamMembers tms = teamMembersMapper.selectTeamMembers(memberId);
//		//只有个人类型和公司类型且是公司领导的经纪人可以看到项目交易总金额
//		if(m.getMemberType() == 2 && count > 0) {
//			resultObject.put("customerList", this.memberCustomerMapper.getMyCustomers(memberId,customerName));
//		}else if(m.getMemberType() == 1 && tms == null) {
//			resultObject.put("customerList", this.memberCustomerMapper.getMyCustomers(memberId,customerName));
//		}else {
//			List<Map<String,Object>> list = this.memberCustomerMapper.getMyCustomers(memberId,customerName);
//			for(Map<String,Object> mp:list) {
//				mp.put("projectAmount", 0);
//			}
//			resultObject.put("customerList", list);
//		}
		if(transaction!=null) {
			resultObject.putAll(transaction);
		} else {
			resultObject.put("projectCount", 0);
			resultObject.put("projectAmount", 0);
		}
		return resultObject;
	}
	@Transactional(propagation=Propagation.REQUIRED)
	public int saveMemberCustomer(MemberCustomer memberCustomer) {
		//根据手机号判断用户是否已存在
		String mobile = memberCustomer.getMobile();
		String leftSub = mobile.substring(0, 3);
		String rightSub = mobile.substring(7, 11);
		String memberId = memberCustomer.getMemberId().toString();
		if(memberCustomerMapper.CountByMobileSub(memberId, leftSub, rightSub) == 0) {
			this.memberCustomerMapper.insertSelective(memberCustomer);
		}else {
			throw new ResultException(Constant.CODE_EXCEPTION, "该客户已存在");
		}
		return 1;
	}
	/**
	 * 客户详情页
	 * @param customerId
	 * @return
	 */
	public ResultObject getCustomerDetail(Integer customerId) {
		ResultObject resultObject = new ResultObject();
		resultObject.put("customerDetail", this.memberCustomerMapper.selectByPrimaryKey(customerId));
		List<Map> customerOrders = this.projectOrderMapper.selectByCustomerId(customerId);
		for(Map order : customerOrders) {
			order.putAll(this.projectService.getOrderStatuse(Integer.valueOf(order.get("project_type").toString()), order.get("order_no").toString()));
		}
		resultObject.put("customerOrders", customerOrders);
		return resultObject;
	}
	
	
}
