package com.up.sell.service.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Maps;
import com.up.sell.mapper.AreasMapper;
import com.up.sell.mapper.MemberFavoriteProjectMapper;
import com.up.sell.mapper.MemberLoginLogMapper;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.mapper.MemberOrderMapper;
import com.up.sell.mapper.MemberRegisterLogMapper;
import com.up.sell.mapper.MemberServiceMapper;
import com.up.sell.mapper.PushSendMapper;
import com.up.sell.mapper.TeamMapper;
import com.up.sell.mapper.TeamMembersMapper;
import com.up.sell.redis.RedisBaseDao;
import com.up.sell.service.MemberService;
import com.up.sell.service.chuangnan.ChuangNanService;
import com.up.sell.util.AliSMS;
import com.up.sell.util.CodeGeneration;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.Constant;
import com.up.sell.util.DateUtils;
import com.up.sell.util.JWTUtil;
import com.up.sell.util.ProcessSMSUtils;
import com.up.sell.util.WxUtils;
import com.up.sell.vo.Areas;
import com.up.sell.vo.Member;
import com.up.sell.vo.ResultObject;
import com.up.sell.vo.Team;
import com.up.sell.vo.TeamMembers;

@Service
public class MemberServiceImpl implements MemberService{

	private static final Logger logger = LoggerFactory.getLogger(MemberServiceImpl.class);
	public static final Integer MAXAGE = 3600*24*15;
	private static final String icon = "/upload/2018/10/29/1540791989940.png";
	public final static String appId = ConfigurationUtils.getStringValue("appId");
	public final static String img_head = ConfigurationUtils.getStringValue("img_head");
	
	@Autowired
	RedisBaseDao redisBaseDao;
	
	@Autowired
	MemberMapper memberMapper;
	
	@Autowired
	MemberLoginLogMapper memberLoginLogMapper;
	
	@Autowired
	MemberRegisterLogMapper memberRegisterLogMapper;
	
	@Autowired
	TeamMapper teamMapper;
	
	@Autowired
	TeamMembersMapper teamMembersMapper;
	
	@Autowired 
	MemberFavoriteProjectMapper memberFavoriteProjectMapper;
	
	@Autowired
	MemberServiceMapper memberServiceMapper;
	
	@Autowired
	MemberOrderMapper memberOrderMapper;
	
	@Autowired
	PushSendMapper pushSendMapper;
	
	@Autowired
	AreasMapper areasMapper;
	@Autowired
	private ChuangNanService chuangNanService;
	
	/**
	 * 登录
	 * @param mobile
	 * @param validCode
	 * @param provinceId
	 * @param cityId
	 * @param type：1、登录 2、推荐 3、加入团队
	 * @param referrerId 推荐人ID
	 * @param teamId 加入团队ID
	 * 
	 * @return
	 */
	@Override
	public ResultObject login(String mobile,String validCode,String provinceId,String cityId,Integer type,Integer referrerId,Integer teamId) {
		ResultObject resultObject = new ResultObject();
		String code =StringUtils.EMPTY;
		List<String> whiteList = memberMapper.selectMemberWhiteList();
		logger.info("手机号白名单 : [{}]",whiteList);
		if(!whiteList.contains(mobile)){
			Object object = redisBaseDao.get(Constant.LOGIN_VALIDCODE_KEY + mobile);
			//判空
			if(Objects.isNull(object)) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("验证码过期，登录失败");
				return resultObject;
			}
			code =String.valueOf(object);
		}else {
			code="000000";
		}
		if(StringUtils.equals(validCode, code)) {
			Member member = memberMapper.selectByMobile(mobile);
			if(member == null) {
				member = new Member();
				member.setMobile(mobile);
				member.setCreateTime(new Date());
				if(StringUtils.isBlank(provinceId) && StringUtils.isNotBlank(cityId)) {
					Areas areas = areasMapper.selectProvinceIdByCityId(cityId);
					if(Objects.nonNull(areas)) {
						member.setProvinceId(areas.getParentid());		
					}else {
						member.setProvinceId(provinceId);
					}
				}else {
					member.setProvinceId(provinceId);	
				}
				member.setCityId(cityId);
				member.setMemberType(1);
				member.setFlag(1);
				member.setIconPath(icon);
				member.setViewCount(3);
				if(type == 2) {//推荐
					if(referrerId==null) {
						resultObject.setCode(Constant.CODE_EXCEPTION);
						resultObject.setMessage("请传referrerId参数");
						return resultObject;
					}
					if(memberMapper.selectByPrimaryKey(referrerId)!=null) {
						member.setReferrerId(referrerId);
						resultObject.setMessage("您已成功被推荐为优铺会员");
						Member me = memberMapper.selectByPrimaryKey(referrerId);
						me.setReferrerCount(me.getReferrerCount()+1);
						memberMapper.updateByPrimaryKey(me);
						member.setProvinceId(me.getProvinceId());
					}
				}
				memberMapper.insertSelective(member);
				resultObject.put("loginFlag", false);
			} else if (type == 2){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("您已注册为本平台的用户");
				return resultObject;
			}else {
				resultObject.put("loginFlag", true);
				if(member.getFlag() == 2) {
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("您的账户已被冻结，请联系客服");
					return resultObject;
				}
			}
			if(type == 3) {//加入团队
				if(teamId==null || referrerId==null) {
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("请传referrerId、teamId参数");
					return resultObject;
				}
				if(this.teamMapper.checkTeamMember(member.getId())>0) {
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("您已经是其它团队的成员，不能再加入此团队");
					return resultObject;
				} else {
					Team team = this.teamMapper.selectByPrimaryKey(teamId);
					if(team==null) {
						resultObject.setCode(Constant.CODE_EXCEPTION);
						resultObject.setMessage("团队不存在，teamId参数不正确");
						return resultObject;
					}
					TeamMembers pPeamMembers = this.teamMembersMapper.selectByPrimaryKey(teamId, referrerId);
					if(pPeamMembers == null) {
						resultObject.setCode(Constant.CODE_EXCEPTION);
						resultObject.setMessage("邀请人不存在，referrerId参数不正确");
						return resultObject;
					}
					TeamMembers teamMembers = new TeamMembers();
					teamMembers.setCreateTime(new Date());
					teamMembers.setMemberId(member.getId());
					teamMembers.setParentId(referrerId);
					teamMembers.setTeamId(teamId);
					Integer teamLevel = pPeamMembers.getTeamLevel();
					if(1==teamLevel) {
						teamMembers.setTeamLevel(2);
					}else if(2==teamLevel) {
						teamMembers.setTeamLevel(3);
					}
					this.teamMembersMapper.insertSelective(teamMembers);
					resultObject.setMessage("您已成功加入了"+team.getTeamName()+"团队");
					member.setMemberType(2);
					memberMapper.updateByPrimaryKeySelective(member);
					//向团队创建者发生短信，提醒成员加入成功
					Member m = memberMapper.selectByPrimaryKey(team.getMemberId());
					
					ProcessSMSUtils processSMSUtils = new ProcessSMSUtils();
					Map<String,Object> map = new HashMap<String,Object>();
					String userName = member.getUserName() == null ? member.getMobile():member.getUserName();
					map.put("userPhone", userName);
					map.put("teamName", team.getTeamName());
					map.put("sendPhone", m.getMobile());
					processSMSUtils.joinTeamSuccessSend(map, chuangNanService);
				}
			}
			member = this.memberMapper.selectByMobile(mobile);
			member.setMemberLevel("0");//未购买会员
			resultObject.put("memberInfo", member);
			//正确，生成token
			String token=JWTUtil.sign(member, MAXAGE);
			resultObject.put(Constant.APP_HEADER_ACCESS_TOKEN, token);
			//this.redisBaseDao.del(Constant.LOGIN_VALIDCODE_KEY + mobile);
		}else {
			Long count = this.redisBaseDao.incr(Constant.LOGIN_VALIDCODE_KEY_ERROR + mobile);
			if (5 == count ) {
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("验证码5次输入错误，请重新获取。");
				this.redisBaseDao.del(Constant.LOGIN_VALIDCODE_KEY + mobile);
				this.redisBaseDao.del(Constant.LOGIN_VALIDCODE_KEY_ERROR + mobile);
			} else {
				this.redisBaseDao.incr(Constant.LOGIN_VALIDCODE_KEY_ERROR + mobile);
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("验证码输入错误，请确认后输入。");
			}
			return resultObject;
		}
		return resultObject;
	}
	
	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public Map<String,Object> sendSms(String mobile,String type) {
		logger.info(" MemberServiceImpl sendSms mobile:{},type:{}",mobile,type);
		String validCode = CodeGeneration.getCode(6);
		logger.info("validCode:{}",validCode);
		Map<String,String> result = AliSMS.sendVerificationCode(validCode, mobile,type);
		Map<String,Object> map = Maps.newHashMap();
		map.put("code", validCode);
		if(result.get("Code") == null || !StringUtils.equalsIgnoreCase("OK", result.get("Code").toString())) {
			map.put("flag", false);
			return map;
		}
		if(StringUtils.equals("1", type)) {
			redisBaseDao.set(Constant.LOGIN_VALIDCODE_KEY+mobile, validCode);
		}else {
			redisBaseDao.set(Constant.CERTIFICATION_VALIDCODE_KEY+mobile, validCode);
		}
		//指定过期时间
		redisBaseDao.expire(mobile, 60 * 30);
		map.put("flag", true);
		return map;
	}
	@Override
	public List<Map<String,Object>> selectFavoriteList(Map<String,Object> parameters) {
		parameters.put("img", img_head);
		List<Map<String,Object>> list = memberFavoriteProjectMapper.selectFavoriteList(parameters);
		for(Map<String, Object> map : list) {
			String tag = (String) map.get("tag");
			if(StringUtils.isNotBlank(tag)) {
				map.put("tag", StringUtils.split(tag,";"));
			}
		}
		return list;
	}

	@Transactional(propagation=Propagation.REQUIRED)
	@Override
	public void updateByPrimaryKey(Map<String,Object> parameters) {
		Member m = new Member();
		String name = parameters.get("name") == null ? null : parameters.get("name").toString();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		m.setId(memberId);//ID
		if(StringUtils.isNotBlank(name)) {
			m.setUserName(name);//姓名	
		}
		m.setUpdateTime(new Date());
		if(StringUtils.isNotBlank((String)parameters.get("type")) && StringUtils.equals("1", (String)parameters.get("type"))) {
			Integer sex = parameters.get("sex") == null ? null : Integer.valueOf(parameters.get("sex").toString());
			if(Objects.nonNull(sex)) {
				m.setSex(sex);//性别
			}
			String iconPath= parameters.get("iconPath") == null ? null : parameters.get("iconPath").toString().replace(img_head, "");
			if(StringUtils.isNotBlank(iconPath)) {
				m.setIconPath(iconPath);//头像	
			}
		}else {
			String idCardPic= parameters.get("idCardPic") == null ? null : parameters.get("idCardPic").toString().replace(img_head, "");
			String idCardNo= parameters.get("idCardNo") == null ? null : parameters.get("idCardNo").toString();
			m.setIdCardNo(idCardNo);
			m.setIdCardPic(idCardPic);
		}
		m.setCompany(parameters.get("company")+"");
		m.setPosition(parameters.get("position")+"");
		memberMapper.updateByPrimaryKeySelective(m);
	}

	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public ResultObject getMemberByPrimaryKey(Integer id) {
		ResultObject resultObject = new ResultObject();
		Member member = memberMapper.selectByPrimaryKey(id);
		String path =img_head + member.getIconPath();
		member.setIconPath(path);
		Map<String,Object> map = Maps.newHashMap();
		map.put("img", img_head);
		map.put("loginIdentity", id);
		Map<String, Object> selectMemberInfo = memberMapper.selectMemberInfo(map);
		int checkCount = pushSendMapper.checkCount(member.getMobile());
		if(checkCount==0){
			member.setPushStatus(0);	
		}else {
			member.setPushStatus(1);	
		}
		if(Objects.isNull(selectMemberInfo)) {
			member.setMemberLevel("0");//未购买会员
		}else {
			member.setMemberLevel(selectMemberInfo.get("goods_id").toString());
			String endTime = selectMemberInfo.get("end_time").toString();
			Date format = DateUtils.format(endTime, "yyyy-MM-dd");
			SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
			String format1 = sdf.format(new Date());
			Date nowDate = null;
		    try {
		    	nowDate=sdf.parse(format1);
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			int datePoor = DateUtils.getDatePoor(format,nowDate);
			if(datePoor<0) {
				member.setMemberLevel("4");//购买的会员已过期
			}
		}
		resultObject.put("memberInfo", member);
		return resultObject;
	}
	
	
	//1-有未读消息；0-没有
	@Override
	public int findPushStatus(Integer memberId){
		int pushStatus=0;
		try {
			Member member = memberMapper.selectByPrimaryKey(memberId);
			if(member ==null){
				return pushStatus;
			}
			int checkCount = pushSendMapper.checkCount(member.getMobile());
			if(checkCount!=0){
				pushStatus=1;
			}
		} catch (NumberFormatException e) {
			e.printStackTrace();
		}
		return pushStatus;
	}
	
	
	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public ResultObject getMemberSubInfoByPrimaryKey(Integer id) {
		ResultObject resultObject = new ResultObject();
		Member member = memberMapper.selectByPrimaryKey(id);
		Map<String,Object> map = Maps.newHashMap();
		map.put("mobile", member.getMobile());
		map.put("userName", member.getUserName());
		String iconPath = StringUtils.EMPTY;
		if(StringUtils.isNotBlank(member.getIconPath())) {
			iconPath = img_head + member.getIconPath();
		}
		map.put("iconPath", iconPath);
		resultObject.put("memberInfo", map);
		return resultObject;
	}

	
	@Override
	public ResultObject createQRCode(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		Member member = memberMapper.selectByPrimaryKey(memberId);
		String type =StringUtils.EMPTY;
		if(Objects.isNull(parameters.get("type"))) {
			resultObject.setCode(Constant.CODE_PARAM_NULL);
			resultObject.setMessage("参数为空");
			return resultObject;
		}else {
			type = parameters.get("type").toString();
		}
		if(StringUtils.equals("1", type)) {
			resultObject.put("referrerId", memberId);
			resultObject.put("cityId", member.getCityId());
		}else {
			TeamMembers tm = teamMembersMapper.selectMemberLevel(memberId);
			if(Objects.nonNull(tm)) {
				resultObject.put("teamId", tm.getTeamId());		
			}else {
				resultObject.put("teamId", "");	
			}
			resultObject.put("referrerId", memberId);
			resultObject.put("cityId", member.getCityId());
			}
		return resultObject;
	}

	@Override
	public ResultObject weChatShare(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		String url = parameters.get("url").toString();//获取分享url
		String access_token =StringUtils.EMPTY;
		if(Objects.nonNull(redisBaseDao.get("access_token"))) {
			access_token = redisBaseDao.get("access_token").toString();
		}
		if(StringUtils.isBlank(access_token)) {
			access_token =  WxUtils.getToken(url,redisBaseDao);
		}
		String nonceStr = WxUtils.uuid();
		int timestamp = WxUtils.getSecondTimestamp(new Date());
		String jsapi_ticket = WxUtils.getjsapi_ticket(access_token);
		String sign = "jsapi_ticket=" + jsapi_ticket + "&noncestr=" + nonceStr + "&timestamp=" + timestamp + "&url=" + url + "";
		// 加密
		String signature = WxUtils.SHA1(sign);
		resultObject.put("appId", appId);
		resultObject.put("timestamp", timestamp);
		resultObject.put("nonceStr", nonceStr);
		resultObject.put("signature", signature);
		return resultObject;
	}

	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public ResultObject selectGoodsList(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		resultObject.put("list", memberServiceMapper.selectMemberServiceList());
		return resultObject;
	}

	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public ResultObject selectMemberInfo(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		parameters.put("img", img_head);
		resultObject.put("list", memberMapper.selectMemberInfo(parameters));
		return resultObject;
	}

	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public ResultObject selectOrderList(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		String type =StringUtils.EMPTY;;
		if(Objects.isNull(parameters.get("type"))) {
			resultObject.setCode(Constant.CODE_PARAM_NULL);
			resultObject.setMessage("参数为空");
			return resultObject;
		}else {
			type = parameters.get("type").toString();
		}
		Integer status =null;
		if(StringUtils.equals("0", type)) {
			parameters.put("status", status);//查询全部
		}else if(StringUtils.equals("1", type)) {
			parameters.put("status", 1);//查询未支付
		}else if(StringUtils.equals("2", type)) {
			parameters.put("status", 2);//查询已支付
		}else {
			parameters.put("status", status);//查询全部
		}
		resultObject.put("orderList", memberOrderMapper.selectOrdersByStatus(parameters));
		return resultObject;
	}

	
	
	@Transactional(propagation=Propagation.REQUIRED)
	@Override
	public ResultObject cancelFavorites(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		String projectId = parameters.get("projectId").toString();
		String[] ids = projectId.split(",");
		try {
			//循环删除
			for(int i=0;i<ids.length;i++) {
				memberFavoriteProjectMapper.deleteByPrimaryKey(memberId, Integer.valueOf(ids[i]));
			}
		} catch (NumberFormatException e) {
			resultObject.put("status", "0");
			resultObject.setMessage("取消失败!"); 
			e.printStackTrace();
			return resultObject;
		}
		resultObject.put("status", "1");
		resultObject.setMessage("取消成功!"); 
		return resultObject;
	}

	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public ResultObject checkPushCount(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		String mobile = parameters.get("mobile").toString();
		int checkCount = pushSendMapper.checkCount(mobile);
		resultObject.put("count", checkCount);
		return resultObject;
	}

	@Transactional(propagation=Propagation.SUPPORTS)
	@Override
	public ResultObject selectPushList(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		List<Map<String, Object>> selectPushList = pushSendMapper.selectPushList(parameters);
		resultObject.put("selectPushList", selectPushList);
		return resultObject;
	}

	@Transactional(propagation=Propagation.REQUIRED)
	@Override
	public ResultObject updatePushStatus(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
	   try {
			pushSendMapper.updatePushStatus(parameters);
		} catch (Exception e) {
			e.printStackTrace();
			resultObject.put("status", false);
			return resultObject;
		}
		resultObject.put("status", true);
		return resultObject;
	}

	@Transactional(propagation=Propagation.REQUIRED)
	@Override
	public ResultObject delPush(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		String str = parameters.get("id").toString();
		String[] ids = str.split(",");
		   try {
			 //循环删除
				for(int i=0;i<ids.length;i++) {
					parameters.put("id", Integer.valueOf(ids[i]));
					pushSendMapper.delPush(parameters);
				}
			} catch (Exception e) {
				e.printStackTrace();
				resultObject.put("status", false);
				return resultObject;
			}
			resultObject.put("status", true);
			return resultObject;
		}

	@Override
	public ResultObject checkPushStatus(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		int checkCount = pushSendMapper.checkCount(parameters.get("mobile").toString());
		if(checkCount==0){
			resultObject.put("pushStatus", 0);
		}else {
			resultObject.put("pushStatus", 1);
		}
		return resultObject;
	}

	@Override
	public Member selectStatus(String mobile) {
		return memberMapper.selectStatus(mobile);
	}
	
	@Override
	public Member selectStatusById(String id) {
		return memberMapper.selectStatusById(id);
	}
	public static void main(String[] args) {
		 String mobile="18600000138";
		 String replaceAll = mobile.substring(7, mobile.length());
		 System.out.println("replaceAll------>"+ replaceAll);
	}
	
}
