/**
 * 
 */
package com.up.sell.service.impl;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.imageio.ImageIO;

import org.apache.commons.lang.StringUtils;
import org.apache.ibatis.javassist.bytecode.annotation.IntegerMemberValue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Maps;
import com.up.sell.mapper.FourHundredMapper;
import com.up.sell.mapper.MemberCustomerMapper;
import com.up.sell.mapper.MemberFavoriteProjectMapper;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.mapper.MemberSignMapper;
import com.up.sell.mapper.ProjectLicenseMapper;
import com.up.sell.mapper.ProjectMapper;
import com.up.sell.mapper.ProjectOrderMapper;
import com.up.sell.mapper.ProjectOrderTrackMapper;
import com.up.sell.mapper.ProjectPictureMapper;
import com.up.sell.mapper.ProjectReleasedMapper;
import com.up.sell.mapper.PushMapper;
import com.up.sell.mapper.PushSendMapper;
import com.up.sell.mapper.TeamMapper;
import com.up.sell.mapper.TeamMembersMapper;
import com.up.sell.service.BrowseLogService;
import com.up.sell.service.ProjectService;
import com.up.sell.service.WxPublicAccountService;
import com.up.sell.service.chuangnan.ChuangNanService;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.Constant;
import com.up.sell.util.DateUtils;
import com.up.sell.util.ProcessSMSUtils;
import com.up.sell.util.ResultException;
import com.up.sell.vo.ManyUser;
import com.up.sell.vo.Member;
import com.up.sell.vo.MemberCustomer;
import com.up.sell.vo.MemberFavoriteProject;
import com.up.sell.vo.Project;
import com.up.sell.vo.ProjectManyUser;
import com.up.sell.vo.ProjectOrder;
import com.up.sell.vo.ProjectOrderTrack;
import com.up.sell.vo.ProjectReleased;
import com.up.sell.vo.Push;
import com.up.sell.vo.ResultObject;
import com.up.sell.vo.Team;
import com.up.sell.vo.TeamMembers;
import com.up.sell.vo.fourhundred.FourHundred;

/**
 * @author zmg
 *
 */
@Service
@Transactional(propagation=Propagation.SUPPORTS)
public class ProjectServiceImpl implements ProjectService {

	Logger logger = LoggerFactory.getLogger(ProjectServiceImpl.class);
	private final String imgPre="http://up-sell.oss-cn-beijing.aliyuncs.com";

	@Autowired
	private ProjectMapper projectMapper;
	@Autowired
	private ProjectPictureMapper projectPictureMapper;
	@Autowired
	private ProjectLicenseMapper projectLicenseMapper;
	@Autowired
	private MemberFavoriteProjectMapper memberFavoriteProjectMapper;
	@Autowired
	private ProjectReleasedMapper projectReleasedMapper;
	@Autowired
	private MemberCustomerMapper memberCustomerMapper;
	@Autowired
	private ProjectOrderMapper projectOrderMapper;
	@Autowired
	private ProjectOrderTrackMapper projectOrderTrackMapper;
	@Autowired
	private MemberMapper memberMapper;
	@Autowired
	private BrowseLogService browseLogService;
	@Autowired
	private MemberSignMapper memberSignMapper;
	@Autowired
	private FourHundredMapper  fourHundredMapper;
	@Autowired
	private ChuangNanService chuangNanService;
	@Autowired
	private TeamMembersMapper teamMemberMapper;
	@Autowired
	private WxPublicAccountService wxPublicAccountService;
	@Autowired
	private TeamMapper teamMapper;

	DateTimeFormatter sdf = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm");
	SimpleDateFormat datefm = new SimpleDateFormat("yyyy-MM-dd HH:mm");
	
	/**
	 * 项目分页列表
	 */
	@Override
	public List<Map<String,Object>> selectProjectList(Map<String,Object> parameters) {
		parameters.put("picDomain", ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN));
		List<Map<String, Object>> list = this.projectMapper.selectProjectList(parameters);
		for(Map<String, Object> map : list) {
			String tag = (String) map.get("tag");
			if(StringUtils.isNotBlank(tag)) {
				map.put("tag", StringUtils.split(tag,";"));
			} else {
				map.put("tag", new String[] {});
			}
			map.put("price_u", this.jsonPriceU(map.get("projectType")+"",map.get("priceType")+""));
		}
		return list;
	}
	
	/**
	 * 格式化单位
	 * 销售：
		（1）:30000/元/㎡
		（2）：待定“”
	         招商：
		（1）:30000元/天/㎡
		（2）：待定“”
	 * @param projectType
	 * @param priceType
	 * @return
	 */
	@Override
	public String jsonPriceU(String projectType,String priceType){
		String priceU="";
		if("1".equals(priceType)){//价格类型：1-数字；2-文本
			if("1".equals(projectType)){//项目类型：1-销售；2-招商
				priceU="元/㎡";//销售
			}else{
				priceU="元/天/㎡";//招商
			}
		}else{
			priceU="";
		}
		return priceU;
	}
	
	
	@Transactional(propagation=Propagation.REQUIRED)
	public ResultObject getProjectDetail(Integer projectId,Integer projectType,Integer houseType,Integer memberId, Integer cityId) {
		ResultObject resultObject = new ResultObject();
		try {
			Map<String,Object> mapEntity=new HashMap<String, Object>();
			if(houseType == 1) {//商业
				mapEntity=this.projectMapper.getProjectDetail1(projectId, projectType, houseType,memberId);
			} else if(houseType == 2) {//办公
				mapEntity=this.projectMapper.getProjectDetail2(projectId, projectType, houseType,memberId);
			} else if(houseType == 3) {//住宅
				mapEntity= this.projectMapper.getProjectDetail3(projectId, projectType, houseType,memberId);
				resultObject.put("projectLicense", this.projectLicenseMapper.selectByProjectId(projectId));//预售许可证
			}
			mapEntity.put("price_u", this.jsonPriceU(projectType+"",mapEntity.get("price_type")+""));
			resultObject.put("projectDetail",mapEntity);
			Map<String, Object> projectDetail = (Map<String, Object>) resultObject.get("projectDetail");
			if(projectDetail!=null) {
				String tag = (String) projectDetail.get("tag");
				if(StringUtils.isNotBlank(tag)) {
					projectDetail.put("tag", StringUtils.split(tag,";"));
				} else {
					projectDetail.put("tag", new String[] {});
				}
			}
			//↓↓↓↓↓↓↓↓↓↓
//			List<Map<String,Object>> picList = this.projectPictureMapper.selectProjectPic(projectId, 1,ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN));
//			if(picList != null && picList.size()>0) {
//				picList.stream().forEach(p ->{
//					String picPath = p.get("pic_path").toString();
//					if(picPath.contains("58-sy")) {
//						BufferedImage sourceImg;
//						try {
//							picPath = picPath.replace("?x-oss-process=image/resize,m_lfit,w_600", "");
//							sourceImg = ImageIO.read(new URL(picPath).openStream());
//							int width = sourceImg.getWidth();
//		            			int height =  sourceImg.getHeight();
//		            			int copWidth = width - 130;
//		            			String style = "?x-oss-process=image/crop,x_0,y_0,w_"+copWidth+",h_"+height;
//							String pic = picPath + style;
//							System.out.println("pic : "+pic);
//							p.put("pic_path", pic);
//						} catch (MalformedURLException e) {
//							// TODO Auto-generated catch block
//							e.printStackTrace();
//						} catch (IOException e) {
//							// TODO Auto-generated catch block
//							e.printStackTrace();
//						}
//					}
//				});
//			}
//			resultObject.put("projectPic", picList);//项目banner图
			//↑↑↑↑↑↑↑↑↑↑↑↑↑
			resultObject.put("projectPic", this.projectPictureMapper.selectProjectPic(projectId, 1,ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN)));//项目banner图
			if(houseType == 1 || houseType == 2) {
				resultObject.put("projectFloorPic", this.projectPictureMapper.selectProjectPic(projectId, 2,ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN)));//楼层图
			} else {
				resultObject.put("projectFloorPic", this.projectPictureMapper.selectProjectPic(projectId, 3,ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN)));//户型图图
			}
			//全景地址
			List<Map<String,Object>> listFull=this.projectPictureMapper.selectProjectPic(projectId, 4,ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN));
			if(listFull!=null && listFull.size()>0){
				resultObject.put("fullViewPic",listFull.get(0).get("pic_path"));//全景地址
			}else{
				resultObject.put("fullViewPic","");//全景地址
			}
			//暂时屏蔽项目推荐
//			List<Map<String, Object>> projectRecommended = this.projectMapper.projectRecommended(projectId, projectType, houseType,cityId);
//			for(Map<String, Object> tempReco : projectRecommended) {
//				tempReco.put("price_u", this.jsonPriceU(projectType+"",tempReco.get("price_type")+""));
//				String tag = (String) tempReco.get("tag");
//				if(StringUtils.isNotBlank(tag)) {
//					tempReco.put("tag", StringUtils.split(tag,";"));
//				}
//				String pictures = (String) tempReco.get("pictures");
//				if(StringUtils.isNotBlank(pictures)) {
//					tempReco.put("pictures", ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN)+StringUtils.split(pictures,",")[0]);
//				}
//			}
			List<Map<String, Object>> projectRecommended = new ArrayList<Map<String,Object>>();
			resultObject.put("projectRecommended", projectRecommended);//项目推荐
			this.projectMapper.updateViewCount(projectId);
			String project_manager_id=projectDetail.get("project_manager_id")+"";
			resultObject.put("agentName", "优铺");
			resultObject.put("agentMobile", "-1");
			resultObject.put("agentIconPath","");
			if(StringUtils.isNotEmpty(project_manager_id) && !project_manager_id.equals("null")){
				Member member=this.memberMapper.selectByPrimaryKey(Integer.valueOf(project_manager_id));
				if(member!=null){
					resultObject.put("agentName", member.getUserName());
					resultObject.put("agentMobile", member.getMobile());
					resultObject.put("agentIconPath",member.getIconPath()==null?"":imgPre+member.getIconPath());
				}
			}
			FourHundred fourHundredEntity=this.fourHundredMapper.findEntityByLinkPhone(resultObject.get("agentMobile")+"");
			if(fourHundredEntity !=null){
				resultObject.put("ivrType","2");
				resultObject.put("ivrKey",fourHundredEntity==null?"0":fourHundredEntity.getIvrKey());
				resultObject.put("ivrPhone",fourHundredEntity==null?"4001981188": fourHundredEntity.getFourPhone());
			}else{
				resultObject.put("ivrType","1");
				resultObject.put("ivrKey","");
				resultObject.put("ivrPhone",resultObject.get("agentMobile")=="-1"?Constant.MPB_KEFU_MOBILE:resultObject.get("agentMobile"));
			}
			resultObject.put("loginUserId",memberId);
			if(StringUtils.isEmpty(project_manager_id) || "null".equals(project_manager_id)){
				if(memberId==null || memberId==0){
					Member hxxEn=this.memberMapper.selectByMobile(Constant.PHONE_HXX);
					if(hxxEn !=null){
						memberId=hxxEn.getId();//默认胡晓旭
					}
				}
				resultObject.put("projectManagerId",memberId);
			}else{ 
				resultObject.put("projectManagerId",project_manager_id);
			}
			//添加浏览痕迹
			Map<String,Object> map=new HashMap<String,Object>();
			map.put("userId", memberId==null?"1":memberId);
			map.put("projectId", projectId);
			this.browseLogService.updateEntity(map);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_SUCCESS);
			resultObject.setMessage(e.getMessage());
			e.printStackTrace();
		}
		return resultObject;
	}
	
	
	
	/**
	 * 会员关注项目
	 * @param projectId
	 * @param memberId
	 * @return
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public int memberFavoriteProject(Integer projectId,Integer memberId) {
		MemberFavoriteProject memberFavoriteProject = new MemberFavoriteProject();
		memberFavoriteProject.setCreateTime(new Date());
		memberFavoriteProject.setMemberId(memberId);
		memberFavoriteProject.setProjectId(projectId);
		Map<String,Object> map = Maps.newHashMap();
		map.put("type", "1");
		map.put("id", projectId);
		projectMapper.updateFavoriteCount(map);
		return this.memberFavoriteProjectMapper.insert(memberFavoriteProject);
		
	}
	/**
	 * 会员取消关注
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public int memberCancelFavoriteProject(Integer projectId,Integer memberId) {
		Map<String,Object> map = Maps.newHashMap();
		map.put("type", "2");
		map.put("id", projectId);
		projectMapper.updateFavoriteCount(map);
		return this.memberFavoriteProjectMapper.deleteByPrimaryKey(memberId, projectId);
	}
	/**
	 * 发布项目
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public int projectReleased(ProjectReleased projectReleased) {
		return this.projectReleasedMapper.insertSelective(projectReleased);
	}
	/**
	 * 地图找房
	 */
	public List<Map<String,Object>> projectOnMap(Map parameters) {
		return this.projectMapper.projectOnMap(parameters);
	}
	/**
	 * 项目报备
	 * @param projectReleased
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public void projectReported(ProjectOrder projectOrder) {
		Project project = this.projectMapper.selectByPrimaryKey(projectOrder.getProjectId());
		if(project == null) {
			throw new ResultException(Constant.CODE_EXCEPTION, "项目不存在或者项目ID错误");
		}
		projectOrder.setProjectName(project.getProjectName());
		projectOrder.setProjectType(project.getProjectType());
		MemberCustomer memberCustomer = this.memberCustomerMapper.selectByPrimaryKey(projectOrder.getCustomerId());
		if(memberCustomer == null) {
			throw new ResultException(Constant.CODE_EXCEPTION, "客户不存在或者客户ID错误");
		}
		projectOrder.setCustomerName(memberCustomer.getCustomerName());
		projectOrder.setMobile(memberCustomer.getMobile());
		if(StringUtils.isNotBlank(projectOrder.getBrandName())) {
			memberCustomer.setBrandName(projectOrder.getBrandName());
			this.memberCustomerMapper.updateByPrimaryKeySelective(memberCustomer);
		}
		if(projectOrderMapper.selectCountByPIdAndCId(projectOrder) > 0) {
			throw new ResultException(Constant.CODE_EXCEPTION, "客户已在该项目报备过");
		};
		this.projectOrderMapper.insertSelective(projectOrder);
		ProjectOrderTrack projectOrderTrack = new ProjectOrderTrack();
		projectOrderTrack.setCreateTime(new Date());
		projectOrderTrack.setMemberId(projectOrder.getMemberId());
		projectOrderTrack.setOrderNo(projectOrder.getOrderNo());
		projectOrderTrack.setOrderStatus(projectOrder.getOrderStatus());
		this.projectOrderTrackMapper.insertSelective(projectOrderTrack);

		//  通过项目经理 查询项目经理姓名以及公司名称
		Member member = memberMapper.selectByPrimaryKey(projectOrder.getMemberId());
		Member manager = memberMapper.selectByPrimaryKey(project.getProjectManagerId());
		if (null != member && manager !=null){
			ProcessSMSUtils processSMSUtils = new ProcessSMSUtils();
			LocalDateTime time = LocalDateTime.now();
			logger.info("----获取项目报备保护期");
			LocalDateTime afterTime = time.minus(-project.getPeriod(), ChronoUnit.DAYS);////截止报备日期为必填项
			logger.info("----报备保护期：{}",afterTime);
			Map<String,Object> map = new HashMap<>();
			map.put("name",member.getUserName());//经纪人姓名
			map.put("startDate",sdf.format(time));//报备项目时间
			map.put("projectName",project.getProjectName());//项目名称
			map.put("customerName",memberCustomer.getCustomerName());//客户名称
			map.put("phone",memberCustomer.getMobile());//客户电话
			map.put("endDate",sdf.format(afterTime));//截止报备日期
			map.put("sendPhone",manager.getMobile());//项目经理电话
			logger.info("当前发送短信项目名称：{}，项目经理手机号码：{}",project.getProjectName(),manager.getMobile());
			String remark=processSMSUtils.baobeiSend(map,chuangNanService);
			logger.info("短信发送成功");
			
			//微信公众号-推送消息
			Boolean ff=this.sendMsgToGzh(manager.getMobile(), remark);
			if (!ff){
				logger.info("微信公众号-推送消息失败-"+manager.getMobile());
			}else{
				logger.info("微信公众号-推送消息失败-"+manager.getMobile());
			}
		}
	}
	
	//向微信公众号发消息
	public Boolean sendMsgToGzh(String sendPhone,String remark){
		Boolean flag=false;
		Map<String,Object> parameters = new HashMap<>();
		parameters.put("phone", sendPhone);
		parameters.put("remark", remark);
		ResultObject resultObject=this.wxPublicAccountService.sendMsgToGzh(parameters);
		if(resultObject.getCode()==Constant.CODE_SUCCESS){
			return true;
		}
		return flag;
	}

	public static void main(String[] args) {
		LocalDateTime time = LocalDateTime.now();
		time = time.minus(7, ChronoUnit.DAYS);
	}

	/**
	 * 我的发布
	 * @param memberId
	 * @return
	 */
	public List<Map<String,Object>> myPush(Integer memberId) {
		return this.projectMapper.myPush(memberId);
	}
	/**
	 * 业务首页项目列表
	 * @param memberId
	 * @return
	 */
	public List<Map<String,Object>> projectManagerProjectList(Map parameters) {
		return this.projectMapper.projectManagerProjectList(parameters);
	}
	/**
	 * 项目经理某个项目的项目详情
	 * @param projectId
	 * @param projectManagerId
	 * @return
	 */
	public ResultObject getProjectManagerProjectDetail(Map parameters) {
		ResultObject resultObject = new ResultObject();
		List<Map>  list=this.projectOrderMapper.selectOrderListByProjectId(parameters);
		if(list !=null && list.size()>0){
			for(Map temp:list){
				String customer_mobile=temp.get("customer_mobile")+"";
				String customer_name=temp.get("customer_name")+"";
				if(customer_mobile.contains(",")){
					customer_mobile=customer_mobile.substring(0,customer_mobile.indexOf(","));
					customer_name=customer_name.substring(0,customer_name.indexOf(","));
					temp.put("customer_mobile", customer_mobile);
					temp.put("customer_name", customer_name);
				}
			}
		}
		resultObject.put("orderList",list);
		return resultObject;
	}
	/**
	 * 项目经理某个项目的项目详情各个订单状态的订单数量统计
	 * @param parameters
	 * @return
	 */
	public ResultObject orderStatusCount(Map parameters) {
		ResultObject resultObject = new ResultObject();
		//各个交易状态的数量统计
		resultObject.put("orderStatusCount", this.projectOrderMapper.selectProjectManagerOrderStatus(parameters));
		return resultObject;
	}
	/**
	 * 查询订单状态
	 * @param projectType
	 * @param orderNo
	 * @return
	 */
	public Map<String,Object> getOrderStatuse(Integer projectType,String orderNo) {
		Map<String,Object> result = new HashMap<String,Object>();
		List<Map<String,Object>> potList = new ArrayList<Map<String,Object>>();
		List<Map<String,Object>> osList = new ArrayList<Map<String,Object>>();
		 Map<String,Object> mapTrack=new HashMap<String,Object>();
		 mapTrack.put("orderNo", orderNo);
		List<ProjectOrderTrack> projectOrderTrackList = this.projectOrderTrackMapper.selectByOrderNo(mapTrack);
		for(ProjectOrderTrack projectOrderTrack : projectOrderTrackList) {
			Map<String,Object> pot = new HashMap<String,Object>();
			Map<String,Object> os = new HashMap<String,Object>();
			pot.put("createTime", projectOrderTrack.getCreateTime());
			pot.put("orderStatus_id", projectOrderTrack.getOrderStatus());
			pot.put("year", DateUtils.format(projectOrderTrack.getCreateTime(), "yyyy"));
			pot.put("md", DateUtils.format(projectOrderTrack.getCreateTime(), "MM-dd"));
			pot.put("orderTrackId", projectOrderTrack.getId());
			if(projectOrderTrack.getOrderStatus() == 1) {
				pot.put("orderStatus", "报备提交");
				pot.put("description", "经纪人预约看房");
				os.put("orderStatus", "报备");
				os.put("picStatus", "1");
			} else if(projectOrderTrack.getOrderStatus() == 2) {
				pot.put("orderStatus", "报备成功");
				pot.put("description", "项目经理报备成功");
				if(osList.size()>0) osList.remove(0);
				os.put("orderStatus", "报备");
				os.put("picStatus", "2");
			} else if(projectOrderTrack.getOrderStatus() == 3) {
				pot.put("orderStatus", "到访成功");
				pot.put("description", "项目经理到访成功");
				os.put("orderStatus", "到访");
				os.put("picStatus", "2");
			} else if(projectOrderTrack.getOrderStatus() == 4) {
				if(projectType == 1) {
					pot.put("orderStatus", "认购成功");
					pot.put("description", "项目经理认购成功");
					os.put("orderStatus", "认购");
				} else {
					pot.put("orderStatus", "意向成功");
					pot.put("description", "项目经理意向成功");
					os.put("orderStatus", "意向");
				}
				os.put("picStatus", "2");
			} else if(projectOrderTrack.getOrderStatus() == 5) {
				pot.put("orderStatus", "签约成功");
				pot.put("description", "项目经理签约成功");
				os.put("orderStatus", "签约");
				os.put("picStatus", "2");
			} else if(projectOrderTrack.getOrderStatus() == 6) {
				pot.put("orderStatus", "佣金确认完成");
				pot.put("description", "财务确认佣金完成");
				os.put("orderStatus", "结佣");
				os.put("picStatus", "2");
			} else if(projectOrderTrack.getOrderStatus() == 7) {
				pot.put("orderStatus", "完成");
				pot.put("description", "交易完成");
				os.put("orderStatus", "完成");
				os.put("picStatus", "2");
			} else if(projectOrderTrack.getOrderStatus() == 0) {
				pot.put("orderStatus", "交易结束");
				pot.put("description", "交易完成");
				if(osList.size()>0) osList.get(osList.size()-1).put("picStatus", "3");
			}
			potList.add(pot);
			osList.add(os);
		}
		Integer lastOrderStatus,isEnd;
		if(projectOrderTrackList==null || projectOrderTrackList.size()==0){
			lastOrderStatus=1;
			isEnd=0;
		}else{
			if(projectOrderTrackList.get(0).getOrderStatus()==0) {
				lastOrderStatus=projectOrderTrackList.get(1).getOrderStatus();
				isEnd=1;
			} else {
				lastOrderStatus=projectOrderTrackList.get(0).getOrderStatus();
				isEnd=0;
			}
		}
		result.put("orderTrackList", potList);
		result.put("lastOrderStatus", lastOrderStatus);
		result.put("isEnd", isEnd);
		return result;
	}
	/**
	 * 查询订单详情
	 * @param orderNo
	 * @return
	 */
	public ResultObject selectOrderDetail(String orderNo) {
		ResultObject resultObject = new ResultObject();
		Map<String,Object> orderDetail = this.projectOrderMapper.selectOrderDetail(orderNo);
		if(orderDetail !=null){
			String customer_mobile=orderDetail.get("customer_mobile")+"";
			String customer_name=orderDetail.get("customer_name")+"";
			if(customer_mobile.contains(",")){
				customer_mobile=customer_mobile.substring(0,customer_mobile.indexOf(","));
				customer_name=customer_name.substring(0,customer_name.indexOf(","));
				orderDetail.put("customer_mobile", customer_mobile);
				orderDetail.put("customer_name", customer_name);
			}
			if(orderDetail.get("brand_name")==null || "".equals(orderDetail.get("brand_name"))){
				orderDetail.put("brand_name", "暂无");
			}
		}
		
		//获取渠道信息 1、加入团队的情况 2、被推荐的情况 3、独立注册的情况
		String member_id = orderDetail.get("member_id")+"";
		Member m = memberMapper.selectByPrimaryKey(Integer.valueOf(member_id));
		logger.debug("m referrerId : [{}]",m.getReferrerId());
		if(m.getReferrerId() == null) {
			//是否加入团队
			TeamMembers tm =  teamMemberMapper.selectTeamMembers(Integer.valueOf(member_id));
			logger.debug("tm : [[]]",tm);
			if(tm == null) {
				//3、独立经纪人情况
				orderDetail.put("p_mobile","暂无");
				orderDetail.put("p_user_name","暂无");
				orderDetail.put("p_team_name","暂无");
			}else {
				//1、加入团队情况 1、老板为推荐人 2、老板有推荐人
				Member bossM = memberMapper.selectByPrimaryKey(tm.getParentId());
				if( bossM ==null || bossM.getReferrerId() == null) {
					orderDetail.put("p_mobile","暂无");
					orderDetail.put("p_user_name","暂无");
					orderDetail.put("p_team_name","暂无");
				}else {
					Member bossReff = memberMapper.selectByPrimaryKey(bossM.getReferrerId());
					Team team = teamMapper.selectByMemberId(bossM.getReferrerId());
					String teamName = "暂无";
					if(team != null) {
						teamName = team.getTeamName();
					}
					orderDetail.put("p_mobile",bossReff.getMobile());
					orderDetail.put("p_user_name",bossReff.getUserName());
					orderDetail.put("p_team_name",teamName);
				}
			}
		}else {
			Member agentReff = memberMapper.selectByPrimaryKey(m.getReferrerId());
			TeamMembers tm =  teamMemberMapper.selectTeamMembers(agentReff.getId());
			String teamName = "暂无";
			if(tm != null) {
				if(tm.getParentId() != null && tm.getTeamLevel() == 2) {
					Team team = teamMapper.selectByMemberId(tm.getParentId());
					teamName = team.getTeamName();
				}
//				else {
//					Team team = teamMapper.selectByMemberId(tm.getMemberId());
//					teamName = team.getTeamName();
//				}
				
			}
			orderDetail.put("p_mobile",agentReff.getMobile());
			orderDetail.put("p_user_name",agentReff.getUserName());
			orderDetail.put("p_team_name",teamName);
		}
		
		orderDetail.putAll(this.getOrderStatuse(Integer.valueOf(orderDetail.get("project_type").toString()), orderNo));
		resultObject.put("orderDetail", orderDetail);
		return resultObject;
	}
	
	
	@Override
	public ResultObject findSignDetail(String orderNo) {
		ResultObject resultObject = new ResultObject();
		try {
			ProjectOrder projectOrder = this.projectOrderMapper.selectByPrimaryKey(orderNo);
			resultObject.put("projectOrder", projectOrder);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			e.printStackTrace();
		}
		return resultObject;
	}
	
	@Override
	public ResultObject findManyUser(String orderNo) {
		ResultObject resultObject = new ResultObject();
		try {
			ProjectOrder projectOrder = this.projectOrderMapper.selectByPrimaryKey(orderNo);
			ProjectManyUser manyEn=new ProjectManyUser();
			 BeanUtils.copyProperties(projectOrder,manyEn);
			 manyEn.setProjectTypeName(manyEn.getProjectType()==1?"销售":"招商");
			 
//			 List<ManyUser> listUser=new ArrayList<ManyUser>();
//			 String customerName=projectOrder.getCustomerName();
//			 String mobile= projectOrder.getMobile();
//			 String brandName = projectOrder.getBrandName();
//			 String customerPosition = projectOrder.getCustomerPosition();
//			 String identityCard= projectOrder.getIdentityCard();
//			 if(StringUtils.isBlank(identityCard)) {
//				 identityCard = "1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,";
//			 }
//			 if(StringUtils.isNotEmpty(customerName)){
//				 String[] nameArr=customerName.split(",");
//				 String[] mobileArr=mobile.split(",");
//				 String[] identityCardArr=identityCard.split(",");
//				 String[] brandNameArr = brandName.split(",");
//				 String[] positionArr = customerPosition.split(",");
//				 for(int i=0;i<nameArr.length;i++){
//					 ManyUser temp=new ManyUser();
//					 temp.setCustomerName(nameArr[i]);
//					 temp.setMobile(mobileArr[i]);
//					 temp.setBrandName(brandNameArr[i]);
//					 temp.setCustomerPosition(positionArr[i]);
//					 temp.setIdentityCard(identityCardArr[i]);
//					 listUser.add(temp);
//				 }
//			 }
			 List<ManyUser> listUser=new ArrayList<ManyUser>();
			 if(manyEn.getProjectType() == 1) {
				 listUser = getManyUserForX(projectOrder);
			 }else {
				 listUser = getManyUserForS(projectOrder);
			 }
			 manyEn.setListUser(listUser);
			 Map<String,Object> mapPic=new HashMap<String,Object>();
			 mapPic.put("orderNo", orderNo);
			 mapPic.put("orderStatus","5");
			 List<ProjectOrderTrack> picList= this.projectOrderTrackMapper.selectByOrderNo(mapPic);
			 if(picList !=null && picList.size()>0 ){
				 String picPath=picList.get(0).getPicPath();
				 if(StringUtils.isNotEmpty(picPath)){
					 String[] picArr=picPath.split(",");
					 if(picArr !=null){
						 List<String> attrs = new ArrayList<String>();
						 for(String temp:picArr){
							 attrs.add(imgPre+temp);
						 }
						 manyEn.setPicList(attrs);
					 }
				 }
				 manyEn.setDescription(picList.get(0).getDescription());
			 }
			resultObject.put("projectOrder", manyEn);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			e.printStackTrace();
		}
		return resultObject;
	}
	
	public List<ManyUser> getManyUserForX(ProjectOrder projectOrder){
		List<ManyUser> listUser=new ArrayList<ManyUser>();
		 String customerName=projectOrder.getCustomerName();
		 String mobile= projectOrder.getMobile();
		 String identityCard= projectOrder.getIdentityCard();
		 if(StringUtils.isNotEmpty(customerName)){
			 String[] nameArr=customerName.split(",");
			 String[] mobileArr=mobile.split(",");
			 String[] identityCardArr=identityCard.split(",");
			 for(int i=0;i<nameArr.length;i++){
				 ManyUser temp=new ManyUser();
				 temp.setCustomerName(nameArr[i]);
				 temp.setMobile(mobileArr[i]);
				 temp.setIdentityCard(identityCardArr[i]);
				 listUser.add(temp);
			 }
		 }
		 return listUser;
	}
	
	public List<ManyUser> getManyUserForS(ProjectOrder projectOrder){
		List<ManyUser> listUser=new ArrayList<ManyUser>();
		 String customerName=projectOrder.getCustomerName();
		 String mobile= projectOrder.getMobile();
		 String brandName = projectOrder.getBrandName();
		 String customerPosition = projectOrder.getCustomerPosition();
		 if(StringUtils.isNotEmpty(customerName)){
			 String[] nameArr=customerName.split(",");
			 String[] mobileArr=mobile.split(",");
			 String[] brandNameArr = brandName.split(",");
			 String[] positionArr = customerPosition.split(",");
			 for(int i=0;i<nameArr.length;i++){
				 ManyUser temp=new ManyUser();
				 temp.setCustomerName(nameArr[i]);
				 temp.setMobile(mobileArr[i]);
				 temp.setBrandName(brandNameArr[i]);
				 temp.setCustomerPosition(positionArr[i]);
				 listUser.add(temp);
			 }
		 }
		 return listUser;
	}
	
	
	/**
	 * 审核项目交易订单
	 * orderNo订单号
	 * orderStatus（0结束交易、2审批、3录到访、4录认购/意向、5录签约）
	 * picPath上传的图片
	 * description备注
	 * endCause拒绝原因
	 * customerName客户姓名
	 * mobile客户手机号
	 * identityCard身份证号
	 * shopNumber商铺编号
	 * price签约价格
	 * signTime签约日期
	 * acreage签约面积
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public ResultObject approve(Map<String, Object> parameters) {
		ResultObject resultObject=new ResultObject();
		try {
			ProjectOrderTrack projectOrderTrack = new ProjectOrderTrack();
			projectOrderTrack.setCreateTime(new Date());
			System.out.println(" descritption : "+parameters.get("description")+"   "+parameters.get("description")+"");
			projectOrderTrack.setDescription(parameters.get("description")+"");
			if(parameters.get("endCause") != null && StringUtils.isNotBlank(parameters.get("endCause")+"")) {
				projectOrderTrack.setEndCause(Integer.valueOf(parameters.get("endCause")+""));
			}
			projectOrderTrack.setMemberId(Integer.valueOf(parameters.get("memberId")+""));
			projectOrderTrack.setOrderNo(parameters.get("orderNo")+"");
			projectOrderTrack.setOrderStatus(Integer.valueOf(parameters.get("orderStatus")+""));
			projectOrderTrack.setPicPath((String)parameters.get("picPath"));
			this.projectOrderTrackMapper.insertSelective(projectOrderTrack);
			logger.info("1==================>:记录日志完事儿！");
			//0除外 7的时候就不能操作了其它的情况只能向上更新
			logger.info("2==================>:开始-将订单("+parameters.get("orderNo")+")的状态更改为（"+parameters.get("orderStatus")+"）");
			System.out.println("brandName : "+parameters.get("brandName"));
			System.out.println("customerPosition : "+parameters.get("customerPosition"));
			System.out.println("customerBusiness : "+parameters.get("customerBusiness"));
			System.out.println("acreage : "+parameters.get("acreage"));
			this.projectOrderMapper.updateApproveOrder(parameters);
			logger.info("3==================>:结束-更新订单("+parameters.get("orderNo")+")状态完事儿！");
			//this.projectOrderMapper.updateProjectOrderTotal(parameters);
			//logger.info("3==================>:更新OrderTotal完事儿！");
			// 审核通过或者拒绝 发送短信给项目经纪人
			this.sendMsg(projectOrderTrack,parameters);
			logger.info("4==================>:发送消息完事儿 ！");
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
			logger.info("approve异常了："+e.getMessage());
		}
		return resultObject;
	}
	
	//发短信
	public void sendMsg(ProjectOrderTrack projectOrderTrack,Map<String, Object> parameters){
		try {
			ProcessSMSUtils processSMSUtils = new ProcessSMSUtils();
			ProjectOrder projectOrder = projectOrderMapper.selectByPrimaryKey(projectOrderTrack.getOrderNo());//订单表信息
			if(projectOrder==null){
				logger.info("----订单编号("+projectOrderTrack.getOrderNo()+")有误，查无数据！");
				return ;
			}
			Project project = this.projectMapper.selectByPrimaryKey(projectOrder.getProjectId());
			if(project==null){
				logger.info("----项目id("+projectOrder.getProjectId()+")有误，查无数据！");
				return ;
			}
			LocalDateTime time = LocalDateTime.now();
			logger.info("----获取项目到访保护期");
			LocalDateTime afterTime = time.minus(-project.getDfPeriod(), ChronoUnit.DAYS);//设置到访保护期
			logger.info("项目到访保护期：{}",afterTime);
			Map<String,Object> map = new HashMap<>();
			Member manager = memberMapper.selectByPrimaryKey(projectOrderTrack.getMemberId());//项目经理信息
			if(manager==null){
				logger.info("----项目经理id("+projectOrderTrack.getMemberId()+")有误，查无数据！");
				return ;
			}
			Member jjrEntity = memberMapper.selectByPrimaryKey(projectOrder.getMemberId());//项目经纪人手机号码
			if(jjrEntity==null){
				logger.info("----项目经纪人id("+projectOrder.getMemberId()+")有误，查无数据！");
				return ;
			}
			map.put("startDate",datefm.format(projectOrder.getCreateTime()));// 提交时间 格式化时间
			map.put("customerName",projectOrder.getCustomerName().substring(0,1) + "先生/女士");//客户姓名
			map.put("projectName",projectOrder.getProjectName());//项目名称
			map.put("endDate",sdf.format(afterTime));// 保护截止日期  默认7天
			map.put("manager",manager.getUserName()==null?"**":manager.getUserName());//项目经理名称
			map.put("phone",manager.getMobile());//项目经理电话
			map.put("sendPhone",jjrEntity.getMobile());//经纪人手机号码
			String remark = "";
			logger.info("-----------当前OrderStatus为：{}",projectOrderTrack.getOrderStatus());

			if (projectOrderTrack.getOrderStatus() == 0){
				logger.info("-----------报备审核失败短信发送");
				remark = processSMSUtils.BaobeiFailureSend(map,chuangNanService);
				savePushBaobeiFail(map);
			}else if (projectOrderTrack.getOrderStatus() == 2){
				LocalDateTime after = time.minus(-project.getPeriod(), ChronoUnit.DAYS);//报备保护期
				map.put("endDate",sdf.format(after));
				logger.info("-----------报备审核成功短信发送");
				remark = processSMSUtils.BaobeiSuccessSend(map,chuangNanService);
				logger.info("-----------报备审核成功站内消息发送");
				savePushBaobeiSuccess(map);
			}else if (projectOrderTrack.getOrderStatus() == 3){
				logger.info("-----------到访成功短信发送");
				remark = processSMSUtils.DaofangSuccessSend(map,chuangNanService);
				savePushDF(map);
			}else if (projectOrderTrack.getOrderStatus() == 4){
				logger.info("-----------认购成功短信发送");
				map.put("endDate",parameters.get("subscribeTime")+"");//subscribeTime  认购时间
				remark = processSMSUtils.RengouSuccessSend(map,chuangNanService);
				savePushRG(map);
			}else if (projectOrderTrack.getOrderStatus() == 5){
				logger.info("-----------签约成功短信发送");
				map.put("endDate",parameters.get("signTime")+"");//signTime签约时间
				remark = processSMSUtils.QianyueSuccessSend(map,chuangNanService);
				savePushQY(map);
			}
			logger.info("短信发送成功");
			
			Boolean ff=this.sendMsgToGzh(map.get("sendPhone")+"", remark);//手机号+内容
			if (!ff){
				logger.info("微信公众号-推送消息失败-"+map.get("sendPhone")+"");
			}else{
				logger.info("微信公众号-推送消息失败-"+map.get("sendPhone")+"");
			}
		} catch (Exception e) {
			e.printStackTrace();
			logger.error(e.getMessage(), e);
		}
	}
	
	@Autowired
	PushMapper pushMapper;
	@Autowired
	PushSendMapper pushSendMapper;
	
	//报备成功
	public void savePushBaobeiSuccess(Map<String,Object> map) {
		String content = "您好！您于" + map.get("startDate") + " 提交的客户" + map.get("customerName") + "在" + map.get("projectName") + "的带看报备已审核成功；保护截至日期为" + map.get("endDate") + "。请您尽快带看以免客户流失，项目经理" + map.get("manager") + " 电话：" + map.get("phone") + "。";
		map.put("title", "报备审核成功");
		map.put("content", content);
		savePush(map);
	}
	//报备失败
	public void savePushBaobeiFail(Map<String,Object> map) {
		String content = "您好！您于" + map.get("startDate") + " 提交的客户" + map.get("customerName") + "在" + map.get("projectName") + "的带看报备审核失败；为避免客户流程，请尽快联系项目经理，项目经理" + map.get("manager") + " 电话：" + map.get("phone") + "。";
		map.put("title", "报备审核失败");
		map.put("content", content);
		savePush(map);
	}
	
	//到访成功
	public void savePushDF(Map<String,Object> map) {
		String content = "您好！恭喜您：您于" + map.get("startDate") + " 提交的客户" + map.get("customerName") + "在" + map.get("projectName") + "已确认到访；保护截至日期为" + map.get("endDate") + "。项目经理" + map.get("manager") + " 电话：" + map.get("phone") + "。";
		map.put("title", "到访成功");
		map.put("content", content);
		savePush(map);
	}
	
	//认购成功
	public void savePushRG(Map<String,Object> map) {
		String content = "您好！恭喜您：您于" + map.get("startDate") + " 提交的客户" + map.get("customerName") + "在" + map.get("projectName") + "已成功认购； 认购日期" + map.get("endDate") + ".认购详情请登录卖铺宝app进行查看。项目经理" + map.get("manager") + " 电话：" + map.get("phone") + "。";
		map.put("title", "认购成功");
		map.put("content", content);
		savePush(map);
	}
	//签约成功
	public void savePushQY(Map<String,Object> map) {
		String content = "您好！恭喜您：您于" + map.get("startDate") + " 提交的客户" + map.get("customerName") + "在" + map.get("projectName") + "楼盘已成功签约；签约日期" + map.get("endDate") + "，签约详情请登录卖铺宝app进行查看，项目经理" + map.get("manager") + " 电话：" + map.get("phone") + "。";
		map.put("title", "签约成功");
		map.put("content", content);
		savePush(map);
	}
	/**
	 * 新增站内消息
	 */
	public void savePush(Map<String,Object> map) {
		
		Push push = new Push();
		push.setTitle(map.get("title").toString());
		push.setContent(map.get("content").toString());
		push.setAuthor("优铺");
		//push.setPushAreas(map.get("pushAreas").toString());
		push.setOpenType(0);
		push.setPushType(1);
		push.setPublicTime(new Date());
		push.setCreateTime(new Date());
		pushMapper.insterInt(push);
		Map<String,Object> sendMap = new HashMap<String,Object>();
		sendMap.put("pushId", push.getId());
		sendMap.put("phone", map.get("sendPhone")); //给报备人发送站内消息（经纪人）
		sendMap.put("send", 0);
		sendMap.put("sendName", "优铺");
		sendMap.put("publicTime", new Date());
		sendMap.put("createTime", new Date());
		savePushSend(sendMap);
	}
	/**
	 * 新增站内消息发送记录
	 * void insterPushSend(List<Map<String,Object>> list);
	 */
	public void savePushSend(Map<String,Object> map) {
		List<Map<String,Object>> list = new ArrayList<Map<String,Object>>();
		list.add(map);
		pushSendMapper.insterPushSend(list);
	}
	/**
	 * 查询交易审批中的图片
	 * @param orderNo
	 * @return
	 */
	public List<Map> picManage(String orderNo) {
		String domain = ConfigurationUtils.getStringValue(Constant.PIC_DOMAIN);
		List<Map> list = this.projectOrderTrackMapper.selectPicByOrderNo(orderNo);
		String picPath = null;
		for(Map map : list) {
			picPath = (String) map.get("pic_path");
			String[] picPaths = StringUtils.split(picPath, ",");
			for(int i=0;i<picPaths.length;i++) {
				picPaths[i]=domain+picPaths[i];
			}
			map.put("pic_path", picPaths);
		}
		return list;
	}
	/**
	 * 增加项目交易审批的图片
	 */
	@Transactional(propagation=Propagation.REQUIRED)
	public int updatePic(String picPath,Integer id) {
		System.out.println("picPath : "+picPath);
		return this.projectOrderTrackMapper.updatePic(picPath, id);
	}
	@Override
	public int getProjectListCount(Map<String, Object> parameters) {
		// TODO Auto-generated method stub
		return projectMapper.getProjectListCount(parameters);
	}
	@Override
	public List<Map<String, Object>> getRecommendProjectList(Map<String, Object> parameters) {
		// TODO Auto-generated method stub
		return projectMapper.getRecommendProjectList(parameters);
	}
	
	
	@Override
	public ResultObject findOrderEntity(String orderNo) {
		ResultObject resultObject = new ResultObject();
		ProjectOrder orderEntity=this.projectOrderMapper.selectByPrimaryKey(orderNo);
		
		resultObject.put("orderEntity", orderEntity);
		return resultObject;
	}
	
	public int updateDescrAndPic(String picPath,String description,String orderTrackId
			,String orderNo,String customerName, String mobile) {
		ProjectOrderTrack pot = projectOrderTrackMapper.selectByPrimaryKey(Integer.valueOf(orderTrackId));
		projectOrderMapper.updateVisitTime(pot.getOrderNo());
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		String dateStr = sdf.format(new Date());
		description = description+","+dateStr;
		Map<String, Object> parameters = new HashMap<String,Object>();
		parameters.put("orderNo", orderNo);
		parameters.put("customerName", customerName);
		parameters.put("mobile", mobile);
		this.projectOrderMapper.updateApproveOrder(parameters);
		return projectOrderTrackMapper.updateDescrAndPic(picPath, description, orderTrackId);
	}
	
	
}
