package com.up.sell.service.impl;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Maps;
import com.up.sell.mapper.AdvertisementMapper;
import com.up.sell.mapper.AppDataMapper;
import com.up.sell.mapper.AppUpgradeMapper;
import com.up.sell.mapper.AreasMapper;
import com.up.sell.mapper.BrandDatabaseCityMapper;
import com.up.sell.mapper.BrandDatabaseMapper;
import com.up.sell.mapper.H5PageMapper;
import com.up.sell.mapper.InformationChannelMapper;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.mapper.ProjectMapper;
import com.up.sell.mapper.SysCompanyMapper;
import com.up.sell.service.ProjectService;
import com.up.sell.service.ShowInfoService;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.vo.ResultObject;

@Service
@Transactional(propagation=Propagation.SUPPORTS)
public class ShowInfoServiceImpl implements ShowInfoService{
	
	@Autowired
	AppDataMapper appDataMapper;
	
	@Autowired
	AdvertisementMapper advertisementMapper;
	
	@Autowired
	InformationChannelMapper infoMapper;
	
	@Autowired
	ProjectMapper projectMapper;
	
	@Autowired
	H5PageMapper h5PageMapper;
	
	@Autowired
	AreasMapper areasMapper;
	
	@Autowired
	SysCompanyMapper sysCompanyMapper;
	
	@Autowired
	AppUpgradeMapper appUpgradeMapper;
	
	@Autowired
	MemberMapper memberMapper;
	
	@Autowired
	BrandDatabaseCityMapper brandDatabaseCityMapper;
	
	@Autowired
	BrandDatabaseMapper brandDatabaseMapper;
	
	@Autowired
	ProjectService projectService;
	
	private static final Logger logger = LoggerFactory.getLogger(ShowInfoServiceImpl.class);

	public final static String img_head = ConfigurationUtils.getStringValue("img_head");

	@Override
	public Map<String,Object> getShowInfo(Map<String,Object> parameters) {
		logger.info("getShowInfo parameters:{}",parameters.get("cityId"));
		Map<String,Object> map = Maps.newHashMap();
		Integer cityId =null;
		if(Objects.nonNull(parameters.get("cityId"))) {
			cityId = Integer.valueOf(parameters.get("cityId").toString());
		}else {
			cityId=110100;
		}
		if(Objects.nonNull(parameters.get("place"))) {
			parameters.put("place", 2);
		}else {
			parameters.put("place", 1);
		}
		parameters.put("cityId", cityId);
		parameters.put("img", img_head);
		//项目数
		int selectProjectCount = projectMapper.selectProjectCount(parameters);
		//经纪人
		parameters.put("type", "1");
		int memberPerson = memberMapper.selectMemberCount(parameters);
		//经纪公司
		parameters.put("type", "2");
		int memberCom = memberMapper.selectMemberCount(parameters);
		//品牌数
		int selectBrandCount = brandDatabaseCityMapper.selectBrandCount(parameters);
		
		Map<String, Object> selectDataByCityId = appDataMapper.selectDataByCityId(cityId);
		if(Objects.isNull(selectDataByCityId)) {
			Map<String, Object> maps = Maps.newHashMap();
			maps.put("brands", selectBrandCount);
			maps.put("projects", selectProjectCount);
			maps.put("agents", memberPerson);
			maps.put("companys", memberCom);
			map.put("numberList",maps);//入驻相关数据
		}else {
			 Integer brand = Integer.valueOf(selectDataByCityId.get("brands").toString());
			 Integer projects = Integer.valueOf(selectDataByCityId.get("projects").toString());
			 Integer agents = Integer.valueOf(selectDataByCityId.get("agents").toString());
			 Integer companys = Integer.valueOf(selectDataByCityId.get("companys").toString());
			 selectDataByCityId.put("brands", selectBrandCount+brand);
			 selectDataByCityId.put("projects", selectProjectCount+projects);
			 selectDataByCityId.put("agents", memberPerson+agents);
			 selectDataByCityId.put("companys", memberCom+companys);
			map.put("numberList",selectDataByCityId);//入驻相关数据
		}
		map.put("bannerList",advertisementMapper.selectAdvList(parameters));//banner图
		map.put("infoList", infoMapper.selectInfoList(parameters));//滚动咨询
		List<Map<String,Object>>   listXiaoShou=projectMapper.selectProjects(cityId.toString(),1,img_head);
		if(listXiaoShou !=null && listXiaoShou.size()>0){
			for(Map<String,Object> temp:listXiaoShou){
				temp.put("price_u", this.projectService.jsonPriceU("1" , temp.get("price_type")+""));
			}
		}
		List<Map<String,Object>>   listZhaoShang=projectMapper.selectProjects(cityId.toString(),2,img_head);
		if(listZhaoShang !=null && listZhaoShang.size()>0){
			for(Map<String,Object> temp:listZhaoShang){
				temp.put("price_u", this.projectService.jsonPriceU("2" , temp.get("price_type")+""));
			}
		}
		map.put("selectProject1",listXiaoShou);//销售列表
		map.put("selectProject2",listZhaoShang);//招商列表   天
		List<String> selectByCityId = areasMapper.selectByCityId(cityId.toString());
		if(Objects.nonNull(selectByCityId) && selectByCityId.size()!=0) {
			map.put("cityName", selectByCityId.get(0));
		}else {
			map.put("cityName", "北京");
		}
		return map;
	}

	@Override
	public Map<String, Object> selectH5Page(Map<String, Object> map) {
		return h5PageMapper.selectH5Page(map);
	}

	@Override
	public ResultObject changeCity(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		List<Map<String, String>> list = sysCompanyMapper.changeCity(parameters.get("cityName").toString());
		logger.info("changeCity list-->" + list.size());
		if(Objects.isNull(list) || list.size()==0) {
			List<Map<String, String>> defaultList = sysCompanyMapper.changeCity("北京市");
			Map<String, String> map = defaultList.get(0);
			resultObject.put("status", false);
			resultObject.put("cityId", map.get("city_id"));
			resultObject.put("provinceId", map.get("province_id"));
			resultObject.put("url", map.get("web_url"));
			resultObject.put("m_url", map.get("m_url"));
		}else {
			Map<String, String> map = list.get(0);
			resultObject.put("status", true);
			resultObject.put("cityId", map.get("city_id"));
			resultObject.put("provinceId", map.get("province_id"));
			resultObject.put("url", map.get("web_url"));
			resultObject.put("m_url", map.get("m_url"));
		}
		return resultObject;
	}

	@Override
	public ResultObject getUpgradeVersionInfo(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		if(Objects.nonNull(parameters.get("platform"))) {
			String platform = parameters.get("platform").toString();
			resultObject.put("data", appUpgradeMapper.getUpgradeVersion(platform));
		}else{
			resultObject.setMessage("参数为空");
			resultObject.put("data", resultObject);
		}
		return resultObject;
	}

	@Override
	public ResultObject getBrandLogoList(Map<String, Object> parameters) {
		logger.info("getBrandLogoList parameters:{}",parameters.get("cityId"));
		 ResultObject resultObject = new ResultObject();
		 List<Map> brandLogoListPC = brandDatabaseMapper.getBrandLogoListPC(parameters);
		 resultObject.put("logoListPC", brandLogoListPC);
		 return resultObject;
	}

}
