package com.up.sell.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.codec.binary.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.up.sell.mapper.MemberMapper;
import com.up.sell.mapper.TeamMapper;
import com.up.sell.mapper.TeamMembersMapper;
import com.up.sell.service.TeamService;
import com.up.sell.service.chuangnan.ChuangNanService;
import com.up.sell.util.ConfigurationUtils;
import com.up.sell.util.DateUtils;
import com.up.sell.util.ProcessSMSUtils;
import com.up.sell.vo.Member;
import com.up.sell.vo.ResultObject;
import com.up.sell.vo.Team;
import com.up.sell.vo.TeamMembers;


@Service
@Transactional(propagation=Propagation.SUPPORTS)
public class TeamServiceImpl implements TeamService{
	public final static String img_head = ConfigurationUtils.getStringValue("img_head");
	@Autowired
	TeamMapper teamMapper;
	
	@Autowired
	TeamMembersMapper teamMembersMapper;
	
	@Autowired
	MemberMapper memberMapper;
	
	@Autowired
	private ChuangNanService chuangNanService;
	
	@Override
	@Transactional(propagation=Propagation.REQUIRED)
	public ResultObject createTeam(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		int count = teamMapper.isTeamLeader(memberId);//判断会员是否创建过团队
		TeamMembers t = teamMembersMapper.selectMemberLevel(memberId);
		if(Objects.nonNull(t) && 3==t.getTeamLevel()) {
			resultObject.put("status", "3");
			resultObject.setMessage("您当前的等级不能再进行发展下级人员！");
			return resultObject;
		}
		if((0 == count && Objects.nonNull(t) && 1==t.getTeamLevel()) || Objects.isNull(t) ) {
			resultObject.put("status", "0");
			if(Objects.nonNull(parameters.get("teamType"))) {
				String teamType = parameters.get("teamType").toString();
				Team team = new Team();
				team.setCreateTime(new Date());
				team.setMemberId(memberId);
				team.setTeamName(parameters.get("teamName").toString());
				team.setTeamType(Integer.valueOf(teamType));
				teamMapper.insertSelective(team);
				Member mem = memberMapper.selectByPrimaryKey(memberId);
				if(StringUtils.equals("2", teamType)) {//公司 修改会员表 会员类型
					mem.setMemberType(2);
					mem.setUpdateTime(new Date());
					String idCardNo = mem.getIdCardNo();
				if(StringUtils.equals("", idCardNo)) {//未实名认证
					resultObject.put("flag", "1");
				}else {//已实名认证
					resultObject.put("flag", "0");
					mem.setIdCardNo("");
					mem.setIdCardPic("");
				}
					memberMapper.updateByPrimaryKeySelective(mem);
				}else {
					resultObject.put("flag", "1");
				}
				resultObject.setMessage("团队创建成功!");
				Integer teamId = teamMapper.selectByMemberId(memberId).getId();
				resultObject.put("teamId", teamId);
				TeamMembers tm = new TeamMembers();
				tm.setCreateTime(new Date());
				tm.setMemberId(memberId);
				tm.setParentId(null);
				tm.setTeamLevel(1);
				tm.setTeamId(teamId);
				tm.setTeamLevel(1);
				teamMembersMapper.insertSelective(tm);
			}
		}else {
			resultObject.put("status", "1");
			resultObject.put("flag", "1");
			resultObject.setMessage("已有团队不能再次创建!");
			if(1==t.getTeamLevel()) {
				resultObject.put("teamId", teamMapper.selectByMemberId(memberId).getId());
			}else if(2==t.getTeamLevel()) {
				resultObject.put("teamId", teamMembersMapper.selectTeamMembers(memberId).getTeamId());
			}
		}
		return resultObject;
	}

	@Override
	@Transactional(propagation=Propagation.REQUIRED)
	public ResultObject disband(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		//Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		Integer memberId = Integer.valueOf(parameters.get("id").toString());
		int count = teamMembersMapper.isHasRelations(memberId);
		if(0 == count) {
			TeamMembers tm = teamMembersMapper.selectMemberLevel(memberId);
			if(1==tm.getTeamLevel()) {//一级
				teamMapper.deleteByMemberId(memberId);
			}
			teamMembersMapper.deleteByPrimaryKey(tm.getTeamId(), memberId);
			resultObject.put("status", "0");
			resultObject.setMessage("解散成功");
		}else {
			resultObject.put("status", "1");
			resultObject.setMessage("您当前还存在下级人员，不能进行解散!");
		}
		return resultObject;
	}

	
	@Override
	@Transactional(propagation=Propagation.SUPPORTS)
	public ResultObject teamRelationList(Map<String, Object> parameters) {
		parameters.put("img", img_head);
		ResultObject resultObject = new ResultObject();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		String type =null;
		if(Objects.nonNull(parameters.get("id"))) {
			memberId = Integer.valueOf(parameters.get("id").toString());
		}
		parameters.put("memberId", memberId);
		TeamMembers tm = teamMembersMapper.selectMemberLevel(memberId);
		if(Objects.nonNull(tm)) {
			resultObject.put("teamLevel", tm.getTeamLevel());	
		}else {
			resultObject.put("teamLevel", "");	
		}
		if(Objects.nonNull(parameters.get("type"))) {
			 type = parameters.get("type").toString();// 1 推荐  2 团队
				if(StringUtils.equals("1", type)) {
					Date startTime = DateUtils.beginTimeHelper(new Date());
					Date endTime = DateUtils.endTimeHelper(new Date());
					parameters.put("startTime", startTime);
					parameters.put("endTime", endTime);
					resultObject.put("referreInfo", memberMapper.selectReferreInfo(parameters));
					List<Map<String, Object>> referrerRelationList = teamMembersMapper.referrerRelationList(parameters);
					resultObject.put("referrerRelationList",referrerRelationList);
					resultObject.put("totalCount", referrerRelationList.size());
				}else if(StringUtils.equals("2", type)){//团队
					if(Objects.isNull(tm)) {
					    Map<String, Object> selectOneLevel = null;
						resultObject.put("teamInfo", selectOneLevel);
						List<Map<String, Object>> teamRelationList = null;
			    		    resultObject.put("teamRelationList",teamRelationList );
					    resultObject.put("totalCount", 0);
						return resultObject;
					   }
					 Integer teamLevel = tm.getTeamLevel();
					 List<Map<String, Object>> teamRelationList = teamMembersMapper.teamRelationList(parameters);
					 
				    	   if(1==teamLevel) {
				    		   resultObject.put("teamInfo", teamMapper.selectOneLevel(memberId));
//				    		   List<Map<String, Object>> teamRelationList = teamMembersMapper.teamRelationList(parameters);
				    		   resultObject.put("teamRelationList",teamRelationList);
						   resultObject.put("totalCount", teamRelationList.size());
						   resultObject.put("teamlevelStr", getTeamLeave(teamRelationList.size()));
						   
				    	   }else	 if(2==teamLevel) {//二级
				    		   		Map<String,Object> map = teamMembersMapper.selectTeamInfo(memberId);
						      	resultObject.put("teamInfo", map);
//						      	List<Map<String, Object>> teamRelationList = teamMembersMapper.teamRelationList(parameters);
						      	resultObject.put("teamRelationList",teamRelationList );
						      	resultObject.put("totalCount", teamRelationList.size());
						      	resultObject.put("teamlevelStr", getTeamLeave(Integer.valueOf(map.get("count").toString())));
						    	} else if(3==teamLevel) {//三级
						    			Map<String,Object> map = teamMembersMapper.selectTeamInfo(memberId);
							      	resultObject.put("teamInfo", map);
							        Integer teamId = tm.getTeamId();
							      	parameters.put("teamId", teamId);
							      	int selectTeamCount = teamMembersMapper.selectTeamCount(parameters);
							      	resultObject.put("totalCount", selectTeamCount);
							      	resultObject.put("teamlevelStr", getTeamLeave(Integer.valueOf(map.get("count").toString())));
					 	}
		            }
				}else {
				    Integer teamLevel = tm.getTeamLevel();
					if(2==teamLevel) {
						resultObject.put("teamRelationList", teamMembersMapper.teamRelationList(parameters));
					}else if(3==teamLevel) { 
						resultObject.put("thirdLevelInfo", teamMembersMapper.selectDetailInfo(parameters));
					}
		      }
		return resultObject;
	}

	@Override
	@Transactional(propagation=Propagation.REQUIRED)
	public ResultObject isDisband(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		int count = teamMembersMapper.isHasRelations(memberId);
		if(0 == count) {
			resultObject.put("status", true);
		}else {
			resultObject.put("status", false);
		}
		return resultObject;
	}

	@Override
	public ResultObject selectTeamLevel(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		Integer memberId = Integer.valueOf(parameters.get("loginIdentity").toString());
		TeamMembers selectMemberLevel = teamMembersMapper.selectMemberLevel(memberId);
		if(Objects.nonNull(selectMemberLevel)) {
			resultObject.put("teamLevel", selectMemberLevel.getTeamLevel());
		}else {
			resultObject.put("teamLevel", 0);
		}
		return resultObject;
	}
	
	@Override
	public List<Map<String,Object>> selectTeamList(String teamName){
		return teamMapper.selectTeamList(teamName);
	}
	
	@Override
	public Map<String,Object> joinTeam(String teamName,Integer memberId){
		Map<String,Object> map = new HashMap<String,Object>();
		if(teamMapper.checkTeamMember(memberId) > 0) {
			map.put("code", "1");
			map.put("msg", "您已加入其它团队，不能再加入此团队");
			return map;
		}
		Team team = teamMapper.selectTeamBYName(teamName);
		if( team == null) {
			map.put("code", "2");
			map.put("msg", "您要加入的团队不存在！");
			return map;
		}
		TeamMembers teamMembers = new TeamMembers();
		teamMembers.setCreateTime(new Date());
		teamMembers.setMemberId(memberId);
		teamMembers.setParentId(team.getMemberId());
		teamMembers.setTeamId(team.getId());
		teamMembers.setTeamLevel(2); //二级人员
		teamMembersMapper.insertSelective(teamMembers);
		Member member = memberMapper.selectByPrimaryKey(memberId);
		member.setMemberType(2);
		memberMapper.updateByPrimaryKeySelective(member);
		map.put("code", "101");
		map.put("msg", "加入团队成功");
		//向团队创建者发生短信，提醒成员加入成功
		Member m = memberMapper.selectByPrimaryKey(team.getMemberId());
		ProcessSMSUtils processSMSUtils = new ProcessSMSUtils();
		Map<String,Object> smsMap = new HashMap<String,Object>();
		String userName = member.getUserName() == null ? member.getMobile():member.getUserName();
		smsMap.put("userPhone", userName);
		smsMap.put("teamName", team.getTeamName());
		smsMap.put("sendPhone", m.getMobile());
		//processSMSUtils.joinTeamSuccessSend(smsMap, chuangNanService);
		return map;
	}
	
	/**
	 * 根据团队人数，获取团队级别
	 * @param list
	 * @return
	 */
	public String getTeamLeave(Integer n) {
		String leaveStr = "zw";
		if(n > 3 && n< 10) {
			leaveStr = "tongpai";
		}else if(n >= 10 && n<30) {
			leaveStr = "yinpai";
		}else if(n >=30 ) {
			leaveStr = "jinpai";
		}
		return leaveStr;
	}

}
