package com.up.sell.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.up.sell.mapper.MemberMapper;
import com.up.sell.mapper.MemberOpenidMapper;
import com.up.sell.mapper.VisitorsLogMapper;
import com.up.sell.redis.RedisBaseDao;
import com.up.sell.service.WxMiniService;
import com.up.sell.util.AESUtils;
import com.up.sell.util.Constant;
import com.up.sell.util.DateUtils;
import com.up.sell.util.ObjectMyUtils;
import com.up.sell.util.WxUtils;
import com.up.sell.vo.Member;
import com.up.sell.vo.MemberOpenid;
import com.up.sell.vo.ResultObject;
import com.up.sell.vo.VisitorsLog;
/**
 * 【小程序】
 * @author PJJ
 *
 */
@Service
public class WxMiniServiceImpl implements WxMiniService{
	private static final Logger logger = LoggerFactory.getLogger(WxMiniServiceImpl.class);
	@Autowired
	private MemberMapper memberMapper;
	@Autowired
	private VisitorsLogMapper visitorsLogMapper;
	@Autowired
	private MemberOpenidMapper memberOpenidMapper;
	@Autowired
	private RedisBaseDao redisBaseDao;
	//个人账号-红仁
	//public final static String AppID = "wxd4f5eb12990b7cb2";
	//public final static String AppSecret = "b7768d884c4407a109011c9b6e4dbaa4";
	//企业账号
	public final static String AppID = "wxb9ed182f7150b894";
	public final static String AppSecret = "f4153053a8ed0aa23ae10af23cfc130f";
	
	/**
	 * 【弹框输入电话】
	 * 往member_openid 表记录一条数据，可能是空phone
	 * @param parameters
	 * @return
	 */
	@Override
	public ResultObject findWxPhoneAlart(Map<String, Object> parameters ){
		ResultObject resultObject=new ResultObject();
		try {
			String openid=parameters.get("openid")+"";
			String phone=parameters.get("phone")+"";
			String validCode=parameters.get("validCode")+"";
			resultObject=this.checkValidCode(phone, validCode);
			if(resultObject.getCode() !=Constant.CODE_SUCCESS){
				return resultObject;
			}	
			//弹框输入电话和openid，存记录
			Map<String,Object> selecMap=new HashMap<String, Object>();
			selecMap.put("oppenid", openid);
			selecMap.put("ext1", "1");
			MemberOpenid entity=this.memberOpenidMapper.findEntityByMap(selecMap);
			if(entity ==null){
				entity=new MemberOpenid();
				entity.setOppenid(openid);
				entity.setChannel("1");
				entity.setFromBy("2");
				entity.setPhone(phone);
				entity.setExt1("1");//方法来源：1-top（弹框输入手机号）；2-bottom（微信授权手机号）
				this.memberOpenidMapper.insert(entity);
				logger.info("【成功绑定用户（空）与openid（"+openid+"）--废数据】");
			}
			resultObject=this.findUserFlagByOpenid(openid);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	
	
	/**
	 * 【底部授权获取电话】
	 * 获取用户在微信的唯一标识
	 * @param request
	 * @return
	 * 0  	 --请求成功
	 * -1	 --系统繁忙，此时请开发者稍候再试
	 * 40125 --appid或是appsecret中间有空格
	 * 40029 --jscode 无效
	 * 45011 --频率限制，每个用户每分钟100次
	 */
	@Override
	public ResultObject findWxPhone(Map<String, Object> parameters) {
		ResultObject resultObject = new ResultObject();
		try {
			String jsCode=parameters.get("jsCode")+"";
			String openid=parameters.get("openid")+"";
			resultObject=this.findOpenidByJsCode(jsCode);
			if(resultObject.getCode() !=Constant.CODE_SUCCESS ||resultObject.get("session_key")==null){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("【根据jsCode获取session_key失败，错误信息为："+resultObject.toString()+"】");
				return resultObject;
			}
			
			String sessionKey=resultObject.get("session_key")+"";
			String encrypted=parameters.get("encrypted")+"";
			String iv=parameters.get("iv")+"";
			logger.error("【(1)解密入参-encrypted："+encrypted+"】");
			logger.error("【(2)解密入参-sessionKey："+sessionKey+"】");
			logger.error("【(3)解密入参-iv："+iv+"】");
			JSONObject  jsonObj=AESUtils.decodePhone(encrypted, sessionKey, iv);
			logger.error("【(4)解密返回报文："+jsonObj+"】");
			if (jsonObj !=null && jsonObj.containsKey("phoneNumber")) {
				String wxPhone = jsonObj.getString("phoneNumber");
				parameters.put("wxPhone",wxPhone);
			}else{
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("【解密微信手机号失败,请检查入参是否正确：encrypted-sessionKey-iv】");
				return resultObject;
			}
			
			//授权的时候，如果手机号注册了则更新手机号和openid绑定
			this.updateMemberIcon(parameters);
			//不论是否注册了，都往openid池存一条数据
			String wxPhone=parameters.get("wxPhone")+"";
			Map<String,Object> selecMap=new HashMap<String, Object>();
			selecMap.put("oppenid", openid);
			selecMap.put("ext1", "2");
			MemberOpenid entity=this.memberOpenidMapper.findEntityByMap(selecMap);
			if(entity ==null){
				entity=new MemberOpenid();
				entity.setOppenid(openid);
				entity.setChannel("1");
				entity.setFromBy("2");
				entity.setPhone(wxPhone);
				entity.setExt1("2");//方法来源：1-top（弹框输入手机号）；2-bottom（微信授权手机号）
				this.memberOpenidMapper.insert(entity);
				logger.info("【成功绑定用户（"+wxPhone+"）与openid（"+openid+"）--授权电话影响弹框的电话】");
			}
			resultObject=this.findUserFlagByOpenid(openid);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	
	/**
	 * 校验验证码是否正确
	 * @param phone
	 * @param validCode
	 * @return
	 */
	public ResultObject checkValidCode(String phone,String validCode){
		ResultObject resultObject=new ResultObject();
		try {
			String code =StringUtils.EMPTY;
			List<String> whiteList = memberMapper.selectMemberWhiteList();
			logger.info("手机号白名单 : [{}]",whiteList);
			if(!whiteList.contains(phone)){
				Object object = redisBaseDao.get(Constant.LOGIN_VALIDCODE_KEY + phone);
				//判空
				if(Objects.isNull(object)) {
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("验证码过期，登录失败");
					return resultObject;
				}
				code =String.valueOf(object);
			}else {
				code="000000";
			}
			if( !StringUtils.equals(validCode, code)) {
				Long count = this.redisBaseDao.incr(Constant.LOGIN_VALIDCODE_KEY_ERROR + phone);
				if (5 == count ) {
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("验证码5次输入错误，请重新获取。");
					this.redisBaseDao.del(Constant.LOGIN_VALIDCODE_KEY + phone);
					this.redisBaseDao.del(Constant.LOGIN_VALIDCODE_KEY_ERROR + phone);
				} else {
					this.redisBaseDao.incr(Constant.LOGIN_VALIDCODE_KEY_ERROR + phone);
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("验证码输入错误，请确认后输入。");
				}
				return resultObject;
			}
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	
	
	
	
	@Override
	public ResultObject findUserFlagByOpenid(String openId) {
		ResultObject resultObject = new ResultObject();
		try {
			//-------------------------------------------
			//【弹框】上面要返回userId，入参openId
			Map<String,Object> selecMap=new HashMap<String, Object>();
			selecMap.put("oppenid", openId);
			selecMap.put("ext1", "1");
			MemberOpenid entity=this.memberOpenidMapper.findEntityByMap(selecMap);
			if(entity!=null){
				resultObject.put("bingFlagAlert", "2");
				//当member_openid有一条记录，但是userId为空的时候，用手机号去查一遍member表，返回userId
				Member member2=this.memberMapper.selectByMobile(entity.getPhone());
				resultObject.put("userId",member2==null?"":member2.getId());
			}else{
				resultObject.put("bingFlagAlert", "1");
				resultObject.put("userId","");
			}
			//-------------------------------------------		
			//【授权】下面不要userId,openid
			Map<String,Object> mapBottom=new HashMap<String, Object>();
			mapBottom.put("oppenid", openId);
			mapBottom.put("ext1", "2");
			MemberOpenid entityB=this.memberOpenidMapper.findEntityByMap(mapBottom);
			if(entityB!=null){
				resultObject.put("bingFlag", "2");
			}else{
				resultObject.put("bingFlag", "1");
			}
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
		
	
	/**
	 * 更新用户头像和名称
	 * @param mobile
	 * @param wxImg
	 * @param wxName
	 * flag 2-绑定了
	 */
	public String updateMemberIcon(Map<String,Object> parameters) throws Exception{
		String userId=null;//没绑定
		String wxPhone=parameters.get("wxPhone")+"";
		String wxImg=parameters.get("wxImg")+"";
		String wxName=parameters.get("wxName")+"";
		String openid=parameters.get("openid")+"";
		//String unionid=parameters.get("unionid")+"";
		if(StringUtils.isEmpty(wxPhone)){
			return userId;
		}
		Member member=this.memberMapper.selectByMobile(wxPhone);
		if(member !=null){
			userId=member.getId()+"";
			if(StringUtils.isEmpty(member.getIconPath()) && ObjectMyUtils.isNotEmpty(wxImg)){
				member.setIconPath(wxImg);
			}
			if(StringUtils.isEmpty(member.getUserName())&& ObjectMyUtils.isNotEmpty(wxName)){
				member.setUserName(wxName);
			}
			if(StringUtils.isEmpty(member.getOpenid())&& ObjectMyUtils.isNotEmpty(openid)){
				member.setOpenid(openid);
			}
			/*if(StringUtils.isEmpty(member.getUnionid())&& ObjectMyUtils.isNotEmpty(unionid)){
				member.setUnionid(unionid);
			}*/
			if(StringUtils.isEmpty(member.getWxImg()) && ObjectMyUtils.isNotEmpty(wxImg) ){
				member.setWxImg(wxImg);
			}
			if(StringUtils.isEmpty(member.getWxName()) && ObjectMyUtils.isNotEmpty(wxName)){
				member.setWxName(wxName);
			}
			this.memberMapper.updateByPrimaryKey(member);
		}else{
			logger.info("【此微信用户手机号("+wxPhone+")未在卖铺宝APP注册,故无法匹配member表！】");
		}
		return userId;
	}
	
	
	/**
	 * 添加访客记录
	 */
	@Override
	public ResultObject addVisitorsLog(Map<String, Object> parameters,VisitorsLog entity) {
		ResultObject resultObject = new ResultObject();
		try {
			String visitorsOpenId=entity.getVisitorsOpenId();
			entity.setVisitorsTime(DateUtils.format(new Date(), "yyyy-MM-dd"));
			entity.setUpdateTime(new Date());
			Map<String,Object> logMap=new HashMap<String, Object>();
			logMap.put("projectId", entity.getProjectId());
			logMap.put("visitorsOpenId", entity.getVisitorsOpenId());
			logMap.put("visitorsTime", entity.getVisitorsTime());
			logMap.put("type", "2");
			VisitorsLog  logEn=this.visitorsLogMapper.findEntityByMap(logMap);
			if(logEn !=null){
				entity.setType("2");//如果已经存在type=2的数据，又传入type=1的数据，则保持type仍然等于2
				//【有】删掉旧数据
				Map<String, Object> mapDel=new HashMap<String, Object>();
				mapDel.put("visitorsTime", entity.getVisitorsTime());
				mapDel.put("visitorsOpenId", visitorsOpenId);
				this.visitorsLogMapper.deleteByOpenId(mapDel);
			}
			if("2".equals(entity.getType())){
				Member memEn=this.memberMapper.findUserByOpenid(visitorsOpenId);
				entity.setPhoneAni(memEn==null?"保密":memEn.getMobile());
				entity.setOwner(memEn==null?"保密":memEn.getMobile());
			}
			//生成新的数据
			this.visitorsLogMapper.saveEntity(entity);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	
	@Override
	public ResultObject findVisitorsLogPage(Map<String, Object> paraMap) {
		ResultObject resultObject = new ResultObject();
		try {
			Integer pageNum = Integer.valueOf((String)paraMap.get("pageNum"));
			Integer pageSize = Integer.valueOf((String)paraMap.get("pageSize"));
			Page<Object> startPage = PageHelper.startPage(pageNum, pageSize, true);
			List<Integer> maxIdList=this.visitorsLogMapper.findMaxIdList(paraMap.get("projectId")+"");
			List<VisitorsLog> list= this.visitorsLogMapper.findList(paraMap);
			if(list!=null && list.size()>0){
				for(VisitorsLog temp:list){
					for(Integer tempMaxId:maxIdList){
						String myMaxId=tempMaxId+"";
						String myTempId=temp.getId()+"";
						if(myTempId.equals(myMaxId)){
							temp.setFirstFlag("-1");//第一条
							continue;
						}
					}
				}
			}
			resultObject.put("list", list);
			resultObject.put("totalCount", startPage.getTotal());
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("系统异常");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}

	
	
	@Override
	public ResultObject findOpenidByJsCode(String jsCode) {
		ResultObject resultObject = new ResultObject();
		try {
			//String jsCode=parameters.get("jsCode")+"";
			String resultStr="";
			if("-1".equals(jsCode)){
				resultStr="{\"openid\":\"oidfF5Lx43EF5jn4FKhxtQLQXZy8\",\"session_key\":\"XNUXjbPGIUuMGEQQIx3mIw==\"}";
				//{"errcode":40163,"errmsg":"code been used, hints: [ req_id: AHGcY24ce-iYXAka ]"}
			}else{
				resultStr=WxUtils.get("https://api.weixin.qq.com/sns/jscode2session?appid="+AppID+"&secret="+AppSecret+"&js_code="+jsCode+"&grant_type=authorization_code");
			}
			JSONObject  data=JSONObject.parseObject(resultStr);
			logger.error("【微信获取用户oppenId返回报文："+resultStr+"】");
			if(data==null || data.get("openid")==null||"".equals(data.get("openid"))){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				if("40125".equals(data.get("errcode"))){
					resultObject.setMessage("获取微信unionId失败:appid或是appsecret账号不正确！");
				}else if ("45011".equals(data.get("errcode"))){
					resultObject.setMessage("获取微信unionId失败:-频率限制，每个用户每分钟100次！");
				}else if ("40029".equals(data.get("errcode")) || "40163".equals(data.get("errcode"))){
					resultObject.setMessage("获取微信unionId失败:小程序里传到后台的jscode有误（只能使用一次）");
				}else{
					resultObject.setMessage("获取微信unionId失败:"+resultStr);
				}
				return resultObject;
			}
			resultObject.put("openid", data.get("openid")+"");
			resultObject.put("session_key", data.get("session_key")+"");
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}
	
	

}
