package com.up.sell.service.impl;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.aliyun.oss.common.utils.StringUtils;
import com.up.sell.mapper.DictionaryMapper;
import com.up.sell.mapper.MemberOpenidMapper;
import com.up.sell.service.WxPublicAccountService;
import com.up.sell.util.Constant;
import com.up.sell.util.DateUtils;
import com.up.sell.util.ObjectMyUtils;
import com.up.sell.util.WxPublicAccountSMSUtil;
import com.up.sell.vo.Dictionary;
import com.up.sell.vo.MemberOpenid;
import com.up.sell.vo.ResultObject;
/**
 * 【小程序】
 * @author PJJ
 *
 */
@Service
public class WxPublicAccountServiceImpl implements WxPublicAccountService{
	private static final Logger logger = LoggerFactory.getLogger(WxPublicAccountServiceImpl.class);

	@Autowired
	private MemberOpenidMapper memberOpenidMapper;
	@Autowired
	private DictionaryMapper dictionaryMapper;
	
	/**
	 * 微信公众号绑定客户电话
	 * @param parameters
	 * @return
	 */
	@Override
	public ResultObject bindingPhone(Map<String,Object> parameters) {
		ResultObject resultObject = new ResultObject();
		try {
			String code=parameters.get("code")+"";
			String phone=parameters.get("phone")+"";
			WxPublicAccountSMSUtil  wa=new WxPublicAccountSMSUtil();
			resultObject=wa.findOpenIdGzh(code);
			if(resultObject.getCode() !=Constant.CODE_SUCCESS|| resultObject.get("openId")==null ||"".equals(resultObject.get("openId"))){
				return resultObject;
			}
			String oppenid=resultObject.get("openId")+"";
			//同一个openid，同一个手机号，是否已经绑定过了
			Map<String,Object> selecMap=new HashMap<String, Object>();
			selecMap.put("oppenid", oppenid);
			selecMap.put("phone", phone);
			MemberOpenid entity=this.memberOpenidMapper.findEntityByMap(selecMap);
			if(entity ==null){
				entity=new MemberOpenid();
				entity.setOppenid(oppenid);
				entity.setPhone(phone);
				entity.setChannel("1");
				entity.setFromBy("1");
				this.memberOpenidMapper.insert(entity);
				logger.info("【成功绑定用户（"+phone+"）与openid（"+oppenid+"）】");
			}else{
				logger.info("【用户（"+phone+"）与openid（"+oppenid+"）已存在绑定关系】");
			}
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}

	@Override
	public ResultObject sendMsgToGzh(Map<String,Object> parameters) {
		ResultObject resultObject = new ResultObject();
		try {
			//【判断配置开关】
			Dictionary	openEntity=this.dictionaryMapper.findValueByDkey("216");//216-open开；close关；
			if(openEntity ==null || "close".equals(openEntity.getdValue())){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("请修改字典表d_key=216的数据d_key=open");//
				logger.info("请修改字典表d_key=216的数据d_key=open");
				return resultObject;
			}
			String phone=parameters.get("phone")+"";
			String remark=parameters.get("remark")+"";
			//【1、根据phone获取openid】
			String openId=this.getOpenIdFromTable(phone);
			if(StringUtils.isNullOrEmpty(openId)){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("手机号（"+phone+"）必须先关注公众号[优铺网络]且绑定");
				logger.info("手机号（"+phone+"）必须先关注公众号[优铺网络]且绑定了，生成openid存入表member_openid，fromBy=1");
				return resultObject;
			}
			WxPublicAccountSMSUtil  wa=new WxPublicAccountSMSUtil();
			String accessToken="";
			//【获取access-token】
			Dictionary	dicMap=this.dictionaryMapper.findValueByDkey("215");
			if(dicMap ==null){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("请前往字典表dictionary配置一条数据d_key=215");
				return resultObject;
			}
			Date updateTime=dicMap.getUpdateTime();
			int  sub=DateUtils.subFenZhong(updateTime);
			if(sub>60 || dicMap.getdValue()==null || "".equals(dicMap.getdValue()) ){//7200秒=2h*60m*60s;
				//大于1小时(及60分钟)则重新请求，且更新dictionary表
				accessToken=wa.getAccessToken();
				if(accessToken !=null && !"".equals(accessToken) && !"null".equals(accessToken)){
					Dictionary dicEntity=new Dictionary();
					dicEntity.setdKey("215");
					dicEntity.setdValue(accessToken);
					dicEntity.setUpdateTime(new Date());
					this.dictionaryMapper.updateEntity(dicEntity);
					logger.error("成功更新accessToken到字典表d_key=215");
				}else{
					resultObject.setCode(Constant.CODE_EXCEPTION);
					resultObject.setMessage("获取最新accessToken失败，返回："+accessToken);
					logger.info("【调用微信平台获取最新 access_token方法失败，请检查（1）访问ip是否在公众号[安全中心]配置了白名单；(2)appid和secret秘钥是否正确，返回报文："+accessToken+"】");
					return resultObject;
				}
			}else{
				//小于等于1小时就直接用库里面的
				accessToken=dicMap.getdValue();
			}
			if(org.apache.commons.lang.StringUtils.isEmpty(accessToken)){
				resultObject.setCode(Constant.CODE_EXCEPTION);
				resultObject.setMessage("入参accessToken有误，请检查"+accessToken);
				logger.error("入参accessToken有误，请检查"+accessToken);
				return resultObject;
			}
			parameters.put("accessToken",accessToken);
			parameters.put("openId",openId);
			parameters.put("remark",remark);
			resultObject=wa.sendMsgToGzhCommon(parameters);
		} catch (Exception e) {
			resultObject.setCode(Constant.CODE_EXCEPTION);
			resultObject.setMessage("异常了");
			logger.error(e.getMessage(), e);
		}
		return resultObject;
	}

	//获取公众号绑定的openid去发送消息
	public String getOpenIdFromTable(String phone){
		String openid="";
		Map<String,Object> selecMap=new HashMap<String, Object>();
		selecMap.put("phone", phone);
		selecMap.put("fromBy","1");
		MemberOpenid entity=this.memberOpenidMapper.findEntityByMap(selecMap);
		if(entity !=null){
			openid=entity.getOppenid();
		}
		return openid;
	}

}
