/**
 * 
 */
package com.up.sell.util;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Maps;

/**
 * @author zhangmg
 *
 */
public class IpAddressUtils {
	
	private static final Logger logger = LoggerFactory.getLogger(IpAddressUtils.class);
	/**
	 * 获取地址
	 * 
	 * @param params
	 * @param encoding
	 * @return
	 * @throws Exception
	 */
	public static Map<String,String> getAddress(String params, String encoding) throws Exception {
		 Map<String,String> map = Maps.newHashMap();
		if (StringUtils.isNotEmpty(params)) {
			String path = "http://ip.taobao.com/service/getIpInfo.php?ip=";
			String returnStr = getRs(path + params, encoding);
			logger.info("getAddress returnStr-->"+ returnStr);
			// {"code":0,"data":{"ip":"221.221.8.234","country":"中国","area":"","region":"北京","city":"北京","county":"XX","isp":"联通","country_id":"CN","area_id":"","region_id":"110000","city_id":"110100","county_id":"xx","isp_id":"100026"}}
			com.alibaba.fastjson.JSONObject jsonObject = JSON.parseObject(returnStr);
			if (returnStr != null) {
				if ("0".equals(jsonObject.get("code").toString())) {
					 com.alibaba.fastjson.JSONObject json = jsonObject.getJSONObject("data");
					 
					 map.put("country", json.getString("country"));// 国家
					 map.put("countryId", json.getString("country_id"));// 国家
				    
					 map.put("area", json.getString("area"));// 国家
					 map.put("areaId", json.getString("area_id"));// 国家

				     map.put("region", json.getString("region"));// 国家
					 map.put("regionId", json.getString("region_id"));// 国家
					
					 map.put("city", json.getString("city"));// 国家
				     map.put("cityId", json.getString("city_id"));// 国家
					
				     map.put("county1", json.getString("county"));// 国家
				     map.put("countyId1", json.getString("county_id"));// 国家
					
					 map.put("isp", json.getString("isp"));// 国家
				     map.put("ispId", json.getString("isp_id"));// 国家

					//info.setIpAddress(params);
				}
			}
		}
		return map;
	}

	/**
	 * 从url获取结果
	 * 
	 * @param path
	 * @param params
	 * @param encoding
	 * @return
	 */
	public static String getRs(String path, String encoding) {

		URL url = null;

		HttpURLConnection connection = null;

		try {

			url = new URL(path);

			connection = (HttpURLConnection) url.openConnection();// 新建连接实例

			connection.setConnectTimeout(2000);// 设置连接超时时间，单位毫�?

			connection.setReadTimeout(2000);// 设置读取数据超时时间，单位毫�?

			connection.setDoInput(true);// 是否打开输出�? true|false

			connection.setRequestMethod("GET");// 提交方法POST|GET

			connection.setUseCaches(false);// 是否缓存true|false

			connection.connect();// 打开连接端口

			BufferedReader reader = new BufferedReader(new InputStreamReader(
					connection.getInputStream(), encoding));

			StringBuffer buffer = new StringBuffer();

			String line = "";

			while ((line = reader.readLine()) != null) {

				buffer.append(line);

			}

			reader.close();

			return buffer.toString();

		} catch (Exception e) {

			e.printStackTrace();

		} finally {

			connection.disconnect();// 关闭连接

		}

		return null;
	}

	/**
	 * 字符转码
	 * 
	 * @param theString
	 * @return
	 */
	public static String decodeUnicode(String theString) {

		char aChar;

		int len = theString.length();

		StringBuffer buffer = new StringBuffer(len);

		for (int i = 0; i < len;) {

			aChar = theString.charAt(i++);

			if (aChar == '\\') {

				aChar = theString.charAt(i++);

				if (aChar == 'u') {

					int val = 0;

					for (int j = 0; j < 4; j++) {

						aChar = theString.charAt(i++);

						switch (aChar) {

						case '0':

						case '1':

						case '2':

						case '3':

						case '4':

						case '5':

						case '6':

						case '7':

						case '8':

						case '9':

							val = (val << 4) + aChar - '0';

							break;

						case 'a':

						case 'b':

						case 'c':

						case 'd':

						case 'e':

						case 'f':

							val = (val << 4) + 10 + aChar - 'a';

							break;

						case 'A':

						case 'B':

						case 'C':

						case 'D':

						case 'E':

						case 'F':

							val = (val << 4) + 10 + aChar - 'A';

							break;

						default:

							throw new IllegalArgumentException(

							"Malformed      encoding.");
						}

					}

					buffer.append((char) val);

				} else {

					if (aChar == 't') {

						aChar = '\t';
					}

					if (aChar == 'r') {

						aChar = '\r';
					}

					if (aChar == 'n') {

						aChar = '\n';
					}

					if (aChar == 'f') {

						aChar = '\f';

					}

					buffer.append(aChar);
				}

			} else {

				buffer.append(aChar);

			}

		}

		return buffer.toString();

	}
	
	/**
	 * 获得访问者的IP
	 * @param request
	 * @return
	 * @throws Exception
	 */
	public static String getIpAddress(HttpServletRequest request) {
		try {
			String ipString = request.getHeader("x-forwarded-for");
			if (StringUtils.isBlank(ipString) || "unknown".equalsIgnoreCase(ipString)) {
				ipString = request.getHeader("Proxy-Client-IP");
			}
			if (StringUtils.isBlank(ipString) || "unknown".equalsIgnoreCase(ipString)) {
				ipString = request.getHeader("WL-Proxy-Client-IP");
			}
			if (StringUtils.isBlank(ipString) || "unknown".equalsIgnoreCase(ipString)) {
				ipString = request.getRemoteAddr();
			}
			// 多个路由时，取第一个非unknown的ip
			String[] arr = ipString.split(",");
			for (String str : arr) {
				if (!"unknown".equalsIgnoreCase(str)){
					ipString = str;
					break;
				}
			}
			return ipString;
		} catch (Exception e) {
			e.printStackTrace();
		}
		return "";
	}
	 public static void main(String[] args) {  
         
	          String ip = "221.221.8.234";  
	          // String ip = "192.168.1.155";
	          try {  
	                
	        	 Map<String, String> address = IpAddressUtils.getAddress(ip, "utf-8");  
	        	 System.out.println(address);  
	             
	          } catch (Exception e) {  
	                
	              e.printStackTrace();  
	          }  
	            
	         
	  
	    }  
}
