/**
 * 
 */
package com.up.sell.util;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.TokenExpiredException;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * @author Administrator
 *
 */
public class JWTUtil {
	private static final Logger logger = LoggerFactory.getLogger(JWTUtil.class);
	private static final String SECRET = "rds2#!G9Fds%^&Gg4>aV0@s]E56*Gh^3<Ud8(Rf3}Mxs1$mq5~d7JClR";  
    private static final String PAYLOAD = "payload";
    public static final Integer MAXAGE = 60*24*7;
    /**
     * 签名每次访问放到header中
     * request.getHeader("Authorization")
     * @param object
     * @param maxAge (Minutes)
     * @return
     */
	public static <T> String sign(T object, Integer maxAge) {
        try {
        	if(object != null) {
        		if(maxAge == null) {
        			maxAge = MAXAGE;
        		}
        		Map<String, Object> headerClaims = new HashMap<String, Object>();
        		headerClaims.put("alg", "HS256");
        		headerClaims.put("type", "JWT");
        		ObjectMapper mapper = new ObjectMapper();  
        		String jsonString = mapper.writeValueAsString(object);
        		String token = JWT.create()
        				.withHeader(headerClaims)
        				.withClaim(PAYLOAD, jsonString)// payload
        				.withExpiresAt(DateUtils.addMinutes(new Date(), maxAge))
        				.sign(Algorithm.HMAC256(SECRET));// 加密
        		return token;
        	}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
        return null;
	}

	public static <T> T unsign(String token, Class<T> classT) {
		try {
			if(StringUtils.isBlank(token)) {
				return null;
			}
			JWTVerifier verifier = JWT.require(Algorithm.HMAC256(SECRET)).build();
			DecodedJWT jwt = verifier.verify(token);
			Map<String, Claim> claims = jwt.getClaims();
			if (claims.containsKey(PAYLOAD)) {
				String json = claims.get(PAYLOAD).asString();  
				ObjectMapper objectMapper = new ObjectMapper();  
				return objectMapper.readValue(json, classT);  
			}
		} catch (TokenExpiredException e) {
			
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}

	/**
	 * @param args
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {
		Map<String, Object> map = new HashMap<String, Object>();
    	map.put("name", "张三");
    	map.put("年龄", "20");
    	String token = JWTUtil.sign(map, 60);
		System.out.println(token);
		System.out.println(JWTUtil.unsign(token, Map.class));
	}

}
