package com.up.sell.util;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.module.SimpleModule;

public class SpringObjectMapper extends ObjectMapper {

	private static final long serialVersionUID = 983517725154258150L;
	
	public SpringObjectMapper() {
		SimpleModule newModule = new SimpleModule();
		newModule.addSerializer(Date.class, new JsonSerializer<Date>() {
			@Override
			public void serialize(Date value, JsonGenerator jsonGenerator, SerializerProvider provider)
					throws IOException, JsonProcessingException {
				String val = "";
				if (value != null) {
					val = DateUtils.format(value, "yyyy-MM-dd HH:mm:ss");
					//val = DateUtils.format(value, "yyyy-MM-dd");
				}
				jsonGenerator.writeString(val);
			}
		});
		newModule.addSerializer(Double.class, new JsonSerializer<Double>() {
			@Override
			public void serialize(Double value, JsonGenerator jsonGenerator, SerializerProvider provider)
					throws IOException, JsonProcessingException {
				String val = "";
				if (value != null) {
					if (value % 1 == 0) {
						val = value.intValue() + "";
					} else {
						//val = NumberUtils.roundDouble(value, 4).toString();
						val = value.toString();
					}
				}
				jsonGenerator.writeString(val);
			}
		});
		newModule.addSerializer(BigDecimal.class, new JsonSerializer<BigDecimal>() {
			@Override
			public void serialize(BigDecimal value, JsonGenerator jsonGenerator, SerializerProvider provider)
					throws IOException, JsonProcessingException {
				String val = "";
				if (value != null) {
					if (value.doubleValue() % 1 == 0) {
						val = value.intValue() + "";
					} else {
						//val = NumberUtils.roundBigDecimal(value, 4).toString();
						val = value.toString();
					}
				}
				jsonGenerator.writeString(val);
			}
		});
		newModule.addSerializer(Integer.class, new JsonSerializer<Integer>() {
			@Override
			public void serialize(Integer value, JsonGenerator jsonGenerator, SerializerProvider provider)
					throws IOException, JsonProcessingException {
				String val = "";
				if (value != null) {
					if (value % 1 == 0) {
						val = value.intValue() + "";
					} else {
						val = value.toString();
					}
				}
				jsonGenerator.writeString(val);
			}
		});
		newModule.addSerializer(Long.class, new JsonSerializer<Long>() {
			@Override
			public void serialize(Long value, JsonGenerator jsonGenerator, SerializerProvider provider)
					throws IOException, JsonProcessingException {
				String val = "";
				if (value != null) {
					if (value % 1 == 0) {
						val = value.intValue() + "";
					} else {
						val = value.toString();
					}
				}
				jsonGenerator.writeString(val);
			}
		});
		newModule.addSerializer(String.class, new JsonSerializer<String>() {
			@Override
			public void serialize(String value, JsonGenerator jsonGenerator, SerializerProvider provider)
					throws IOException, JsonProcessingException {
				String val = "";
				if (value != null) {
					val = value;
				}
				jsonGenerator.writeString(val);
			}
		});
		this.registerModule(newModule);
//		super.setSerializationInclusion(Include.NON_NULL);
		super.getSerializerProvider().setNullValueSerializer(new JsonSerializer<Object>() { 
            @Override 
            public void serialize(Object value, JsonGenerator jg, SerializerProvider sp) throws IOException, JsonProcessingException { 
                jg.writeString(""); 
            } 
        });
	}
}
