package com.up.sell.util;

import java.io.IOException;
import java.io.InputStreamReader;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Date;
import java.util.UUID;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import com.alibaba.fastjson.JSON;
import com.up.sell.redis.RedisBaseDao;

@Controller
public class WxUtils {

	private static final Logger log = LoggerFactory.getLogger(WxUtils.class);

	public static CloseableHttpClient httpClient = HttpClientBuilder.create().build();

	public final static String appId = ConfigurationUtils.getStringValue("appId");
	
	//public static final String appid = "wxd3730802694fecf4";

	// 微信的appsecret秘钥
	public final static String appsecret = ConfigurationUtils.getStringValue("appsecret");
	//public static final String appsecret = "f3e346607af09a2773c7b716865134b3";

	@Autowired
	static HttpServletRequest request;

	@Autowired
	HttpServletResponse response;

//	static RedisBaseDao redisBaseDao;
//
//	public static void setRedisBaseDao(RedisBaseDao redisBaseDao) {
//		WxUtils.redisBaseDao=redisBaseDao;
//	}
	
	/**
	 * get请求获取String类型数据
	 * 
	 * @param url
	 *            请求链接
	 * @return
	 */
	public static String get(String url) {
		StringBuffer sb = new StringBuffer();
		HttpGet httpGet = new HttpGet(url);
		try {
			HttpResponse response = httpClient.execute(httpGet); // 1
			HttpEntity entity = response.getEntity();
			InputStreamReader reader = new InputStreamReader(entity.getContent(), "utf-8");
			char[] charbufer;
			while (0 < reader.read(charbufer = new char[10])) {
				sb.append(charbufer);
			}
		} catch (IOException e) {// 1
			e.printStackTrace();
		} finally {
			httpGet.releaseConnection();
		}
		return sb.toString();
	}

	/**
	 * 获取时间戳
	 * 
	 * @param date
	 * @return
	 */
	public static int getSecondTimestamp(Date date) {
		if (null == date) {
			return 0;
		}
		String timestamp = String.valueOf(date.getTime());
		int length = timestamp.length();
		if (length > 3) {
			return Integer.valueOf(timestamp.substring(0, length - 3));
		} else {
			return 0;
		}
	}

	/**
	 * 获取随机数
	 * 
	 * @return
	 */
	public static String uuid() {
		return UUID.randomUUID().toString().replace("-", "").substring(16);
	}

	/**
	 * 获取公众号jsapi_ticket
	 * 
	 * @param access_token
	 * @return
	 */
	public static String getjsapi_ticket(String access_token) {
		String back = get(
				"https://api.weixin.qq.com/cgi-bin/ticket/getticket?access_token=" + access_token + "&type=jsapi");
		String jsapi_ticket = JSON.parseObject(back).getString("ticket");
		log.info("获取公众号jsapi_ticket的json:{}", back);
		log.info("获取公众号的jsapi_ticket为：{}", jsapi_ticket);
		return jsapi_ticket;
	}


	/**
	 * 发送请求获取token
	 * 
	 * @return
	 */
	public static String getToken(String url,RedisBaseDao redisBaseDao) {
		String token = get("https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=" + appId
				+ "&secret=" + appsecret);
		String access_token = JSON.parseObject(token).getString("access_token");
		if (StringUtils.isBlank(access_token)) {
			log.error("获取到access_token为空");
		} else {
			log.info("发送链接获取到的access_token:{}", access_token);
			redisBaseDao.set("access_token", access_token);
			redisBaseDao.expire("access_token", 7200);
		}
		return access_token;
	}

	/**
	 * 微信sign的SHA1加密
	 * 
	 * @param str
	 * @return
	 */
	public static String SHA1(String str) {
		try {
			MessageDigest digest = java.security.MessageDigest.getInstance("SHA-1"); // 如果是SHA加密只需要将"SHA-1"改成"SHA"即可
			digest.update(str.getBytes());
			byte messageDigest[] = digest.digest();
			// Create Hex String
			StringBuffer hexStr = new StringBuffer();
			// 字节数组转换为 十六进制 数
			for (int i = 0; i < messageDigest.length; i++) {
				String shaHex = Integer.toHexString(messageDigest[i] & 0xFF);
				if (shaHex.length() < 2) {
					hexStr.append(0);
				}
				hexStr.append(shaHex);
			}
			return hexStr.toString();

		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		}
		return null;
	}

}
